@@ -138,6 +138,9 @@
     (map-windows #'(lambda (w) (push w ret)))
     (nreverse ret)))
 
+(defun front-window ()
+  (map-windows #'(lambda (win) (return-from front-window win))))
+
 (defun first-window-satisfying-predicate (pred)
   (block foo
     (map-windows #'(lambda (w) (when (funcall pred w)
@@ -216,6 +219,7 @@
              (@selector #/invokeLispFunction:)
              arg
              t)
+            (#/release arg)
             (apply #'values return-values))))))
 
 
@@ -410,3 +414,16 @@
     (>= (%get-long p) #x1050)))
 
 
+;; This works even if an event loop is not running.
+
+#+windows-target
+(defun shift-key-now-p ()
+  (logbitp 15 (#_GetAsyncKeyState #$VK_SHIFT)))
+
+#+darwin-target
+(defun shift-key-now-p ()
+  (let* ((event (#_CGEventCreate +null-ptr+))
+	 (flags (#_CGEventGetFlags event)))
+    (prog1
+	(logtest flags #$kCGEventFlagMaskShift)
+      (#_CFRelease event))))
