@@ -121,7 +121,7 @@
 
 
 
-(defloadvar *default-ns-application-proxy-class-name*
+(defvar *default-ns-application-proxy-class-name*
     "LispApplicationDelegate")
 
 
@@ -197,26 +197,27 @@
   
   (flet ((cocoa-startup ()
 	   ;; Start up a thread to run periodic tasks.
-	   (process-run-function "housekeeping" #'ccl::housekeeping-loop)
-           (with-autorelease-pool
-             (enable-foreground)
-             (or *NSApp* (setq *NSApp* (init-cocoa-application)))
-             #-cocotron
-             (let* ((icon (#/imageNamed: ns:ns-image #@"NSApplicationIcon")))
-               (unless (%null-ptr-p icon)
-                 (#/setApplicationIconImage: *NSApp* icon)))
-             (setf (ccl::application-ui-object *application*) *NSApp*)
-             (when application-proxy-class-name
-               (let* ((classptr (ccl::%objc-class-classptr
-                                 (ccl::load-objc-class-descriptor application-proxy-class-name)))
-                      (instance (#/init (#/alloc classptr))))
-
-                 (#/setDelegate: *NSApp* instance))))
-           (run-event-loop)))
+           (ccl::with-standard-initial-bindings
+               (process-run-function "housekeeping" #'ccl::housekeeping-loop)
+               (with-autorelease-pool
+                   (enable-foreground)
+                 (or *NSApp* (setq *NSApp* (init-cocoa-application)))
+                 #-cocotron
+                 (let* ((icon (#/imageNamed: ns:ns-image #@"NSApplicationIcon")))
+                   (unless (%null-ptr-p icon)
+                     (#/setApplicationIconImage: *NSApp* icon)))
+                 (setf (ccl::application-ui-object *application*) *NSApp*)
+                 (when application-proxy-class-name
+                   (let* ((classptr (ccl::%objc-class-classptr
+                                     (ccl::load-objc-class-descriptor application-proxy-class-name)))
+                          (instance (#/init (#/alloc classptr))))
+                     
+                     (#/setDelegate: *NSApp* instance))))
+               (run-event-loop))))
     (process-interrupt *cocoa-event-process* #'(lambda ()
-						 (%set-toplevel 
-						  #'cocoa-startup)
-						 (toplevel)))))
+                                                 (%set-toplevel 
+                                                  #'cocoa-startup)
+                                                 (toplevel)))))
 
 (defparameter *font-attribute-names*
   '((:bold . #.#$NSBoldFontMask)
@@ -340,6 +341,141 @@
                           (%make-nsstring title))
                         #@"") ))
 
+(defmethod allocate-instance ((class ns:+ns-window)
+                              &rest initargs
+                              &key
+                              (with-content-rect nil content-rect-p)
+                              (style-mask 0 style-mask-p)
+                              (x 200)
+                              (y 200)
+                              (width 500)
+                              (height 200)
+                              (closable t)
+                              (iconifyable t)
+                              (expandable t)
+                              (metal nil)
+                              (backing :buffered)
+                              (defer t defer-p)
+                              &allow-other-keys)
+  (declare (ignore defer with-content-rect))
+  (unless content-rect-p
+    (setq initargs (cons :with-content-rect
+                         (cons (ns:make-ns-rect x y width height)
+                               initargs))))
+  (unless (and style-mask-p (typep style-mask 'fixnum))
+    (setq initargs (cons :style-mask
+                         (cons (logior #$NSTitledWindowMask
+                                       (if closable #$NSClosableWindowMask 0)
+                                       (if iconifyable #$NSMiniaturizableWindowMask 0)
+                                       (if expandable #$NSResizableWindowMask 0)
+                                       (if metal #$NSTexturedBackgroundWindowMask 0))
+                               initargs))))
+  (unless (typep (getf initargs :backing) 'fixnum)
+    (setq initargs
+          (cons :backing
+                (cons (ecase backing
+                        ((t :retained) #$NSBackingStoreRetained)
+                        ((nil :nonretained) #$NSBackingStoreNonretained)
+                        (:buffered #$NSBackingStoreBuffered))
+                      initargs))))
+  (unless defer-p
+    (setq initargs (cons :defer (cons t initargs))))
+  (apply #'call-next-method class initargs))
+
+(defmethod initialize-instance :after ((w ns:ns-window)
+                                       &key
+                                       (title nil)
+                                       (x 200.0)
+                                       (y 200.0)
+                                       (height 200.0)
+                                       (width 500.0)
+                                       (closable t)
+                                       (iconifyable t)
+                                       (metal nil)
+                                       (expandable t)
+                                       (backing :buffered)
+                                       (defer t)
+                                       (accepts-mouse-moved-events nil)
+                                       (auto-display t)
+                                       (activate nil)
+                                       &allow-other-keys)
+  ;; Several of the keyword args we claim to accept are actually processed
+  ;; by the ALLOCATE-INSTANCE method above and are ignored here.
+  (declare (ignore x y width height closable iconifyable expandable metal
+                   backing defer))
+  (setf (get-cocoa-window-flag w :accepts-mouse-moved-events)
+        accepts-mouse-moved-events
+        (get-cocoa-window-flag w :auto-display)
+        auto-display)
+  ;;; Should maybe have a way of controlling this.
+  (#/setBackgroundColor: w (#/whiteColor ns:ns-color))
+  (when title
+    (set-window-title w title))
+  (when activate
+    (activate-window w)))
+
+
+(defmethod allocate-instance ((class ns:+ns-view)
+                              &rest initargs
+                              &key
+                              (with-frame nil with-frame-p)
+                              (x 0)
+                              (y 0)
+                              (width 0)
+                              (height 0)
+                              &allow-other-keys)
+  (declare (ignorable with-frame))
+  (unless with-frame-p
+    (setq initargs (cons :with-frame
+                         (cons (ns:make-ns-rect x y width height) initargs))))
+  (apply #'call-next-method class initargs))
+
+
+(defmethod initialize-instance :after ((view ns:ns-view)
+                                       &key
+                                       (horizontally-resizable nil hrp)
+                                       (vertically-resizable nil vrp)
+                                       (max-x-margin nil maxxp)
+                                       (min-x-margin nil minxp)
+                                       (max-y-margin nil maxyp)
+                                       (min-y-margin nil minyp)
+                                       (resizes-subviews t rsp)
+                                       view-container
+                                       &allow-other-keys)
+  (let* ((mask (#/autoresizingMask view))
+         (newmask mask))
+    (when hrp
+      (setq newmask (if horizontally-resizable
+                      (logior newmask #$NSViewWidthSizable)
+                      (logandc2 newmask #$NSViewWidthSizable))))
+    (when vrp
+      (setq newmask (if vertically-resizable
+                      (logior newmask #$NSViewHeightSizable)
+                      (logandc2 newmask #$NSViewHeightSizable))))
+    (when minxp
+      (setq newmask (if min-x-margin
+                      (logior newmask #$NSViewMinXMargin)
+                      (logandc2 newmask #$NSViewMinXMargin))))
+    (when maxxp
+      (setq newmask (if max-x-margin
+                      (logior newmask #$NSViewMaxXMargin)
+                      (logandc2 newmask #$NSViewMaxXMargin))))
+    (when minyp
+      (setq newmask (if min-y-margin
+                      (logior newmask #$NSViewMinYMargin)
+                      (logandc2 newmask #$NSViewMinYMargin))))
+    (when maxyp
+      (setq newmask (if max-y-margin
+                      (logior newmask #$NSViewMaxYMargin)
+                      (logandc2 newmask #$NSViewMaxYMargin))))
+    (unless (eql mask newmask)
+      (#/setAutoresizingMask: view newmask)))
+  (when rsp
+    (#/setAutoresizesSubviews: view resizes-subviews))
+  (when view-container
+    (install-view-in-container view view-container)))
+                              
+
 (defun new-cocoa-window (&key
                          (class (find-class 'ns:ns-window))
                          (title nil)
@@ -356,33 +492,29 @@
                          (accepts-mouse-moved-events nil)
                          (auto-display t)
                          (activate t))
-  (ns:with-ns-rect (frame x y width height)
-    (let* ((stylemask
-            (logior #$NSTitledWindowMask
-                    (if closable #$NSClosableWindowMask 0)
-                    (if iconifyable #$NSMiniaturizableWindowMask 0)
-                    (if expandable #$NSResizableWindowMask 0)
-		    (if metal #$NSTexturedBackgroundWindowMask 0)))
-           (backing-type
-            (ecase backing
-              ((t :retained) #$NSBackingStoreRetained)
-              ((nil :nonretained) #$NSBackingStoreNonretained)
-              (:buffered #$NSBackingStoreBuffered)))
-           (w (make-instance
-	       class
-	       :with-content-rect frame
-	       :style-mask stylemask
-	       :backing backing-type
-	       :defer defer)))
-      (setf (get-cocoa-window-flag w :accepts-mouse-moved-events)
-            accepts-mouse-moved-events
-            (get-cocoa-window-flag w :auto-display)
-            auto-display)
-      (#/setBackgroundColor: w (#/whiteColor ns:ns-color))
-      (when activate (activate-window w))
-      (when title (set-window-title w title))
+  (make-instance class
+                 :title title
+                 :x x
+                 :y y
+                 :height height
+                 :width width
+                 :closable closable
+                 :iconifyable iconifyable
+                 :metal metal
+                 :expandable expandable
+                 :backing backing
+                 :defer defer
+                 :accepts-mouse-moved-events accepts-mouse-moved-events
+                 :auto-display auto-display
+                 :activate activate))
+
+(defmethod view-window ((view ns:ns-view))
+  (let* ((w (#/window view)))
+    (unless (%null-ptr-p w)
       w)))
 
+(defmethod install-view-in-container ((view ns:ns-view) (container ns:ns-view))
+  (#/addSubview: container view))
 
-
-
+(defmethod install-view-in-container ((view ns:ns-view) (container ns:ns-window))
+  (#/addSubview: (#/contentView container) view))
