@@ -199,22 +199,25 @@
 (defx86reg temp1.w r9w)
 (defx86reg temp1.b r9b)
 
-(defx86reg ra0 r10)
-(defx86reg ra0.l r10d)
-(defx86reg ra0.w r10w)
-(defx86reg ra0.b r10b)
-
 (defx86reg temp2 r10)
 (defx86reg temp2.l r10d)
 (defx86reg temp2.w r10w)
 (defx86reg temp2.b r10b)
 
+(defx86reg ra0 r10)
+(defx86reg ra0.l r10d)
+(defx86reg ra0.w r10w)
+(defx86reg ra0.b r10b)
+
+#+(or darwin-target windows-target)
+(defx86reg rcontext r11)
 
 (defx86reg save3 r11)
 (defx86reg save3.l r11d)
 (defx86reg save3.w r11w)
 (defx86reg save3.b r11b)
 
+
 (defx86reg save2 r12)
 (defx86reg save2.l r12d)
 (defx86reg save2.w r12w)
@@ -455,7 +458,7 @@
 (defconstant max-32-bit-constant-index (ash (+ #x7fffffff x8664::misc-data-offset) -2))
 (defconstant max-16-bit-constant-index (ash (+ #x7fffffff x8664::misc-data-offset) -1))
 (defconstant max-8-bit-constant-index (+ #x7fffffff x8664::misc-data-offset))
-(defconstant max-1-bit-constant-index (ash (+ #x7fffffff x8664::misc-data-offset) 5))
+(defconstant max-1-bit-constant-index (ash (+ #x7fffffff x8664::misc-data-offset) 3))
 
 )
 (defmacro define-storage-layout (name origin &rest cells)
@@ -648,7 +651,7 @@
   hardlimit                             ; another one
   code                                  ; an area-code; see below
   markbits                              ; bit vector for GC
-  ndwords                               ; "active" size of dynamic area or stack
+  ndnodes                               ; "active" size of dynamic area or stack
   older                                 ; in EGC sense
   younger                               ; also for EGC
   h                                     ; Handle or null pointer
@@ -793,7 +796,7 @@
   FindSymbol
   malloc
   free
-  jvm-init
+  wait-for-signal
   tcr-frame-ptr
   register-xmacptr-dispose-function
   open-debug-output
