@@ -17,66 +17,98 @@
 
 (in-package "CCL")
 
-(defnx1 nx1-the the (&whole call typespec form &environment env)
+(defun nx1-typespec-for-typep (typespec env &key (whine t))
   ;; Allow VALUES types here (or user-defined types that
   ;; expand to VALUES types).  We could do a better job
   ;; of this, but treat them as wild types.
   ;; Likewise, complex FUNCTION types can be legally used
   ;; in type declarations, but aren't legal args to TYPEP;
   ;; treat them as the simple FUNCTION type.
-  (flet ((typespec-for-the (typespec)
-           (let* ((ctype (handler-case (values-specifier-type (nx-target-type typespec) env)
-                           (parse-unknown-type (c)
-                             (nx1-whine :unknown-type-in-declaration (parse-unknown-type-specifier c))
-                             *wild-type*)
-                           (program-error (c)
-                              (nx1-whine :invalid-type typespec c)
-                             *wild-type*))))
-             (if (typep ctype 'function-ctype)
-               'function
-               (nx-target-type (type-specifier ctype))))))
-    (let* ((typespec (typespec-for-the typespec))
-           (*nx-form-type* typespec)
-           (transformed (nx-transform form env)))
-      (flet ((fold-the ()
-               (do* ()
-                    ((or (atom transformed)
-                         (not (eq (car transformed) 'the))))
-                 (destructuring-bind (ftype form) (cdr transformed)
-                   (setq typespec (nx-target-type (nx1-type-intersect call typespec (typespec-for-the ftype)))
-                         *nx-form-type* typespec
-                         transformed form)))))
+  (labels ((ctype-spec (ctype)
+             (typecase ctype
+               (function-ctype 'function)
+               (values-ctype '*)
+               (array-ctype
+                  (let ((new (ctype-spec (array-ctype-element-type ctype))))
+                    (when new
+                      (list (if (array-ctype-complexp ctype) 'array 'simple-array)
+                            new
+                            (array-ctype-dimensions ctype)))))
+               (negation-ctype
+                  (let ((new (ctype-spec (negation-ctype-type ctype))))
+                    (when new
+                      `(not ,new))))
+               (union-ctype
+                  (let* ((types (union-ctype-types ctype))
+                         (new (mapcar #'ctype-spec types)))
+                    (unless (every #'null new)
+                      `(or ,@(mapcar (lambda (new old) (or new (type-specifier old))) new types)))))
+               (intersection-ctype
+                  (let* ((types (intersection-ctype-types ctype))
+                         (new (mapcar #'ctype-spec types)))
+                    (unless (every #'null new)
+                      `(and ,@(mapcar (lambda (new old) (or new (type-specifier old))) new types)))))
+               (t nil))))
+    (let* ((ctype (handler-case (values-specifier-type (nx-target-type typespec) env)
+                    (parse-unknown-type (c)
+                      (if whine
+			(progn
+			  (nx1-whine :unknown-type-in-declaration
+				     (parse-unknown-type-specifier c))
+			  *wild-type*)
+			(specifier-type typespec env)))
+                    (program-error (c)
+		      (if whine
+			(progn
+			  (nx1-whine :invalid-type typespec c)
+			  *wild-type*)
+			(specifier-type typespec)))))
+           (new (ctype-spec ctype)))
+      (nx-target-type (type-specifier (if new (specifier-type new) ctype))))))
+
+(defnx1 nx1-the the (&whole call typespec form &environment env)
+  (let* ((typespec (nx1-typespec-for-typep typespec env))
+         (*nx-form-type* typespec)
+         (transformed (nx-transform form env)))
+    (flet ((fold-the ()
+             (do* ()
+                 ((or (atom transformed)
+                      (not (eq (car transformed) 'the))))
+               (destructuring-bind (ftype form) (cdr transformed)
+                 (setq typespec (nx-target-type (nx1-type-intersect call typespec (nx1-typespec-for-typep ftype env)))
+                       *nx-form-type* typespec
+                       transformed form)))))
+      (fold-the)
+      (do* ((last transformed transformed))
+          ()
+        (setq transformed (nx-transform transformed env))
+        (when (or (atom transformed)
+                  (not (eq (car transformed) 'the)))
+          (return))
         (fold-the)
-        (do* ((last transformed transformed))
-             ()
-          (setq transformed (nx-transform transformed env))
-          (when (or (atom transformed)
-                    (not (eq (car transformed) 'the)))
-            (return))
-          (fold-the)
-          (when (eq transformed last)
-            (return)))
-	(if (and (nx-form-constant-p transformed env)
-                 (or (equal typespec '(values))
-                     (not (typep (nx-form-constant-value transformed env)
-                                 (single-value-type (values-specifier-type typespec))))))
-	  (progn
-            (nx1-whine :type call)
-            (setq typespec '*))
-          (setq typespec (nx-target-type
-                          (or (nx1-type-intersect call
-                                                  typespec
-                                                  (typespec-for-the (nx-form-type transformed env)))
-                              '*))))
-        ;; Wimp out, but don't choke on (the (values ...) form)
-        (when (and (consp typespec) (eq (car typespec) 'values))
+        (when (eq transformed last)
+          (return)))
+      (if (and (nx-form-constant-p transformed env)
+               (or (equal typespec '(values))
+                   (not (typep (nx-form-constant-value transformed env)
+                               (single-value-type (values-specifier-type typespec))))))
+        (progn
+          (nx1-whine :type call)
           (setq typespec '*))
-        (make-acode
-         (%nx1-operator typed-form)
-         typespec
-         (let* ((*nx-form-type* typespec))
-           (nx1-transformed-form transformed env))
-         (nx-declarations-typecheck env))))))
+        (setq typespec (nx-target-type
+                        (or (nx1-type-intersect call
+                                                typespec
+                                                (nx1-typespec-for-typep (nx-form-type transformed env)env))
+                            '*))))
+      ;; Wimp out, but don't choke on (the (values ...) form)
+      (when (and (consp typespec) (eq (car typespec) 'values))
+        (setq typespec '*))
+      (make-acode
+       (%nx1-operator typed-form)
+       typespec
+       (let* ((*nx-form-type* typespec))
+         (nx1-transformed-form transformed env))
+       (nx-declarations-typecheck env)))))
 
 (defnx1 nx1-struct-ref struct-ref (&whole whole structure offset)
   (if (not (fixnump (setq offset (nx-get-fixnum offset))))
@@ -425,10 +457,10 @@
                 (nx1-form newvalue)))
 
 (defnx1 nx1-logior-2 ((logior-2)) (&whole w &environment env arg-1 arg-2)
-  (nx-binary-boole-op w 
-                      env 
-                      arg-1 
-                      arg-2 
+  (nx-binary-boole-op w
+                      env
+                      arg-1
+                      arg-2
                       (%nx1-operator %ilogior2)
                       (%nx1-operator logior2)
 		      (%nx1-operator %natural-logior)))
@@ -442,19 +474,91 @@
                       (%nx1-operator logxor2)
 		      (%nx1-operator %natural-logxor)))
 
-(defnx1 nx1-logand-2 ((logand-2)) (&whole w &environment env arg-1 arg-2)
-  (nx-binary-boole-op w 
-                      env 
-                      arg-1 
-                      arg-2 
-                      (%nx1-operator %ilogand2)
-                      (%nx1-operator logand2)
-		      (%nx1-operator %natural-logand)))
+(defnx1 nx1-logand-2 ((logand-2)) (&environment env arg-1 arg-2)
+  (let* ((nat1 (nx-form-typep arg-1 *nx-target-natural-type* env))
+         (nat2 (nx-form-typep arg-2 *nx-target-natural-type* env)))
+    (cond ((and (nx-form-typep arg-1 *nx-target-fixnum-type* env)
+                (nx-form-typep arg-2 *nx-target-fixnum-type* env))
+           (make-acode (%nx1-operator typed-form)
+                       *nx-target-fixnum-type*
+                       (make-acode (%nx1-operator %ilogand2)
+                                   (nx1-form arg-1 env)
+                                   (nx1-form arg-2 env))))
+          ((and nat1 (typep arg-2 'integer))
+           (make-acode (%nx1-operator typed-form)
+                       *nx-target-natural-type*
+                       (make-acode (%nx1-operator %natural-logand)
+                                   (nx1-form arg-1 env)
+                                   (nx1-form (logand arg-2
+                                                     (1- (ash 1 (target-word-size-case
+                                                                 (32 32)
+                                                                 (64 64)))))
+                                             env))))
+          ((and nat2 (typep arg-1 'integer))
+           (make-acode (%nx1-operator typed-form)
+                       *nx-target-natural-type*
+                       (make-acode (%nx1-operator %natural-logand)
+                                   (nx1-form arg-2 env)
+                                   (nx1-form (logand arg-1
+                                                     (1- (ash 1 (target-word-size-case
+                                                                 (32 32)
+                                                                 (64 64)))))
+                                             env))))
+          ((and nat1 nat2)
+           (make-acode (%nx1-operator typed-form)
+                       *nx-target-natural-type*
+                       (make-acode (%nx1-operator %natural-logand)
+                                   (nx1-form arg-1 env)
+                                   (nx1-form arg-2 env))))
+          (t
+           (make-acode (%nx1-operator typed-form)
+                       'integer
+                       (make-acode (%nx1-operator logand2)
+                                   (nx1-form arg-1 env)
+                                   (nx1-form arg-2 env)))))))
 
-(defnx1 nx1-require ((require-simple-vector) (require-simple-string) (require-integer) (require-list)
-                     (require-fixnum) (require-real) (require-character) (require-number) (require-symbol) (require-s8) (require-u8) (require-s16) (require-u16) (require-s32) (require-u32) (require-s64) (require-u64))
-        (arg)
-  (make-acode (%nx1-default-operator) (nx1-form arg)))
+
+(defnx1 nx1-require ((require-simple-vector)
+                     (require-simple-string)
+                     (require-integer)
+                     (require-list)
+                     (require-fixnum)
+                     (require-real)
+                     (require-character)
+                     (require-number)
+                     (require-symbol)
+                     (require-s8)
+                     (require-u8)
+                     (require-s16)
+                     (require-u16)
+                     (require-s32)
+                     (require-u32)
+                     (require-s64)
+                     (require-u64))
+        (arg &environment env)
+
+  (if (nx-inhibit-safety-checking env)
+    (let* ((op *nx-sfname*)
+           (type (case op
+                   (require-simple-vector 'simple-vector)
+                   (require-simple-string 'simple-string)
+                   (require-integer 'integer)
+		   (require-list 'list)
+		   (require-fixnum 'fixnum)
+		   (require-real 'real)
+		   (require-character 'character)
+		   (require-number 'number)
+		   (require-symbol 'symbol)
+		   (require-s8 '(signed-byte 8))
+		   (require-u8 '(unsigned-byte 8))
+		   (require-s16 '(signed-byte 16))
+		   (require-u16 '(unsigned-byte 16))
+		   (require-s32 '(signed-byte 32))
+		   (require-u32 '(unsigned-byte 32))
+		   (require-s64 '(signed-byte 64))
+		   (require-u64 '(unsigned-byte 64)))))
+      (nx1-form `(the ,type ,arg)))
+    (make-acode (%nx1-default-operator) (nx1-form arg))))
 
 (defnx1 nx1-%marker-marker ((%unbound-marker) (%slot-unbound-marker) (%illegal-marker)) ()
   (make-acode (%nx1-default-operator)))
@@ -561,9 +665,7 @@
           (nx1-form `(%short-float+-2 ,num1 ,num2))
 	  (if (nx-binary-natural-op-p num1 num2 env nil)
 	    (make-acode (%nx1-operator typed-form)
-                        (target-word-size-case
-                         (32 '(unsigned-byte 32))
-                         (64 '(unsigned-byte 64)))
+                        *nx-target-natural-type*
 			(make-acode (%nx1-operator %natural+) f1 f2))
 	    (make-acode (%nx1-operator typed-form) 'number 
 			(make-acode (%nx1-operator add2) f1 f2))))))))
@@ -1205,28 +1307,29 @@
     (note-function-info (%cadr def) (caddr def) env))
   (nx1-treat-as-call w))
 
-
 (defnx1 nx1-function function (arg &aux fn afunc)
-  (if (symbolp arg)
-    (progn
-      (when (macro-function arg *nx-lexical-environment*)
-        (nx-error
-         "~S can't be used to reference lexically visible macro ~S." 
-         'function arg))
-      (if (multiple-value-setq (fn afunc) (nx-lexical-finfo arg))
-        (progn
-          (when afunc 
-            (incf (afunc-fn-refcount afunc))
-            (when (%ilogbitp $fbitbounddownward (afunc-bits afunc))
-              (incf (afunc-fn-downward-refcount afunc))))
-          (nx1-symbol (%cddr fn)))
-        (progn
-          (while (setq fn (assq arg *nx-synonyms*))
-            (setq arg (%cdr fn)))
-          (nx1-form `(%function ',arg)))))
-    (if (and (consp arg) (eq (%car arg) 'setf))
-      (nx1-form `(function ,(nx-need-function-name arg)))
-      (nx1-ref-inner-function nil arg))))
+  (cond ((symbolp arg)
+	 (when (macro-function arg *nx-lexical-environment*)
+	   (nx-error
+	    "~S can't be used to reference lexically visible macro ~S." 
+	    'function arg))
+	 (if (multiple-value-setq (fn afunc) (nx-lexical-finfo arg))
+	   (progn
+	     (when afunc 
+	       (incf (afunc-fn-refcount afunc))
+	       (when (%ilogbitp $fbitbounddownward (afunc-bits afunc))
+		 (incf (afunc-fn-downward-refcount afunc))))
+	     (nx1-symbol (%cddr fn)))
+	   (progn
+	     (while (setq fn (assq arg *nx-synonyms*))
+	       (setq arg (%cdr fn)))
+	     (nx1-form `(%function ',arg)))))
+	((setf-function-name-p arg)
+	 (nx1-form `(function ,(nx-need-function-name arg))))
+	((lambda-expression-p arg)
+	 (nx1-ref-inner-function nil arg))
+	(t
+	 (nx-error "~S is not a function name or lambda expression" arg))))
 
 (defnx1 nx1-nfunction nfunction (name def)
  (nx1-ref-inner-function name def))
@@ -1380,7 +1483,7 @@
    (nx1-ff-call-internal
     address-expression arg-specs-and-result-spec
     (ecase (backend-name *target-backend*)
-      (:linuxppc32 (%nx1-operator eabi-ff-call))
+      ((:linuxppc32 :linuxarm :darwinarm) (%nx1-operator eabi-ff-call))
       ((:darwinppc32 :linuxppc64 :darwinppc64) (%nx1-operator poweropen-ff-call))
       ((:darwinx8632 :linuxx8632 :win32 :solarisx8632 :freebsdx8632) (%nx1-operator i386-ff-call))
       ((:linuxx8664 :freebsdx8664 :darwinx8664 :solarisx8664 :win64) (%nx1-operator ff-call)))))
@@ -2014,6 +2117,15 @@
         (%define-x86-lap-function name `((let ,bindings ,@body))
 				    (dpb (length bindings) $lfbits-numreq 0))))
 
+(defnx1 nx1-arm-lap-function (arm-lap-function) (name bindings &body body)
+  (declare (ftype (function (t t t)) %define-arm-lap-function))
+  (require "ARM-LAP")
+  (setf (afunc-lfun *nx-current-function*)
+        (%define-arm-lap-function name `((let ,bindings ,@body))
+				    (dpb (length bindings) $lfbits-numreq 0))))
+
+                    
+
 
 
 (defun nx1-env-body (body old-env &optional (typecheck (nx-declarations-typecheck *nx-lexical-environment*)))
@@ -2150,45 +2262,74 @@
 
 (defnx1 nx1-symvector ((%symptr->symvector) (%symvector->symptr)) (arg)
   (make-acode (%nx1-default-operator) (nx1-form arg)))
-        
+
+(defnx1 nx1-%ilognot (%ilognot) (n)
+  ;; Bootstrapping nonsense.
+  (if (aref (backend-p2-dispatch *target-backend*)
+            (logand operator-id-mask (%nx1-operator %ilognot)))
+    (make-acode (%nx1-operator typed-form)
+                'fixnum
+                (make-acode (%nx1-operator %ilognot)
+                            (nx1-form n)))
+    (nx1-form (macroexpand `(%ilognot ,n)))))
+
+    
 (defnx1 nx1-ash (ash) (&whole call &environment env num amt)
-  (let* ((unsigned-natural-type (target-word-size-case
-                                 (32 '(unsigned-byte 32))
-                                 (64 '(unsigned-byte 64))))
-         (max (target-word-size-case (32 32) (64 64)))
-         (maxbits (target-word-size-case
-                   (32 29)
-                   (64 60))))
-    (cond ((eq amt 0) (nx1-form `(require-type ,num 'integer) env))
-          ((and (fixnump amt)
-                (< amt 0))
-           (if (nx-form-typep num 'fixnum env)
-             (make-acode (%nx1-operator %iasr)
-                         (make-acode (%nx1-operator fixnum)
-                                     (- amt))
-                         (nx1-form num))
-             (if (nx-form-typep num unsigned-natural-type env)
-               (make-acode (%nx1-operator natural-shift-right)
-                           (nx1-form num)
-                           (make-acode (%nx1-operator fixnum)
-                                       (min (1- max) (- amt))))
+  (flet ((defer-to-backend ()
+             ;; Bootstrapping nonsense
+             (if (svref (backend-p2-dispatch *target-backend*)
+                        (logand operator-id-mask (%nx1-operator ash)))
+               (make-acode (%nx1-operator typed-form)
+                           'integer
+                           (make-acode
+                            (%nx1-operator ash)
+                            (nx1-form num)
+                            (nx1-form amt)))
                (nx1-treat-as-call call))))
-          ((and (fixnump amt)
-                (<= 0 amt maxbits)
-                (or (nx-form-typep num `(signed-byte ,(- (1+ maxbits) amt)) env)
-                    (and (nx-form-typep num 'fixnum env)
-                         (nx-trust-declarations env)
-                         (subtypep *nx-form-type* 'fixnum))))
-           (nx1-form `(%ilsl ,amt ,num)))
-          ((and (fixnump amt)
-                (< amt max)
-                (nx-form-typep num unsigned-natural-type env)
-                (nx-trust-declarations env)
-                (subtypep *nx-form-type* unsigned-natural-type))
-           (make-acode (%nx1-operator natural-shift-left)
-                       (nx1-form num)
-                       (nx1-form amt)))
-          (t (nx1-treat-as-call call)))))
+    (let* ((unsigned-natural-type *nx-target-natural-type*) 
+           (max (target-word-size-case (32 32) (64 64)))
+           (maxbits (target-word-size-case
+                     (32 29)
+                     (64 60))))
+      (cond ((eq amt 0) (nx1-form `(require-type ,num 'integer) env))
+            ((and (fixnump amt)
+                  (< amt 0))
+             (if (nx-form-typep num 'fixnum env)
+               (make-acode (%nx1-operator %iasr)
+                           (make-acode (%nx1-operator fixnum)
+                                       (- amt))
+                           (nx1-form num))
+               (if (nx-form-typep num unsigned-natural-type env)
+                 (if (< (- amt) max)
+                   (make-acode (%nx1-operator natural-shift-right)
+                               (nx1-form num)
+                               (make-acode (%nx1-operator fixnum)
+                                           (- amt)))
+                   (nx1-form `(progn (require-type ,num 'integer) 0) env))
+                 (defer-to-backend))))
+            ((and (fixnump amt)
+                  (<= 0 amt maxbits)
+                  (or (nx-form-typep num `(signed-byte ,(- (1+ maxbits) amt)) env)
+                      (and (nx-form-typep num 'fixnum env)
+                           (nx-trust-declarations env)
+                           (subtypep *nx-form-type* 'fixnum))))
+             (nx1-form `(%ilsl ,amt ,num)))
+            ((and (fixnump amt)
+                  (< 0 amt max)
+                  (nx-form-typep num unsigned-natural-type env)
+                  (nx-trust-declarations env)
+                  (subtypep *nx-form-type* unsigned-natural-type))
+             (make-acode (%nx1-operator natural-shift-left)
+                         (nx1-form num)
+                         (nx1-form amt)))
+            ((fixnump num)
+             (let* ((field-width (1+ (integer-length num)))
+                    ;; num fits in a `(signed-byte ,field-width)
+                    (max-shift (- (1+ maxbits) field-width)))
+               (if (nx-form-typep amt `(mod ,(1+ max-shift)) env)
+                 (nx1-form `(%ilsl ,amt ,num))
+                 (defer-to-backend))))
+            (t (defer-to-backend))))))
 
     
         
