@@ -26,13 +26,6 @@
   (require 'lispequ)
 )
 
-#-bootstrapped
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (when (and (macro-function 'var-decls)
-             (not (macro-function 'var-ref-forms)))
-    (setf (macro-function 'var-ref-forms)
-          (macro-function 'var-decls))))
-
 #+ppc-target (require "PPCENV")
 #+x8632-target (require "X8632ENV")
 #+x8664-target (require "X8664ENV")
@@ -104,7 +97,7 @@
      (%i+ . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
      (%i- . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
      (cxxr . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
-     (%dfp-combine . 0)
+     (ash . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
      (%ilsl . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
      (%ilogand2 . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
      (%ilogior2 . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask))
@@ -119,7 +112,7 @@
      (%fixnum-ref . #.operator-single-valued-mask)
      (%fixnum-ref-natural . #.operator-single-valued-mask)
      (%current-tcr . #.operator-single-valued-mask)
-     (%stack-trap . #.operator-single-valued-mask)
+     (%ilognot . #.(logior operator-single-valued-mask operator-assignment-free-mask operator-acode-subforms-mask operator-side-effect-free-mask operator-cc-invertable-mask))
      (multiple-value-prog1 . 0)
      (multiple-value-bind . 0)
      (multiple-value-call . 0)
@@ -386,10 +379,7 @@
 (declaim (special *nx1-alphatizers* *nx1-operators*))
 
 (defmacro %nx1-default-operator ()
- #-bccl
- `(nx1-default-operator)
- #+bccl
- `(gethash *nx-sfname* *nx1-operators*))
+  `(nx1-default-operator))
 
 (defmacro defnx1 (name sym arglist &body forms)
   (let ((fn `(nfunction ,name ,(parse-macro name arglist forms)))
@@ -441,26 +431,6 @@
 
 (defconstant $eaclosedbit 24)
 
-#+what?
-(progn
-;;; condition codes :
-;;; These are 68K condition code values, but the frontend uses them and
-;;; both backends need to understand them.
-;;; They're really backend-specific; it wouldn't hurt to have the frontend
-;;; use a more "neutral" representation.
-(defconstant $ccT 0)
-(defconstant $ccEQ 7)
-(defconstant $ccNE 6)
-(defconstant $ccVC 8)
-(defconstant $ccMI 11)
-(defconstant $ccPL 10)
-(defconstant $ccGE 12)
-(defconstant $ccLT 13)
-(defconstant $ccGT 14)
-(defconstant $ccLE 15)
-)
-
-
 (defmacro %temp-push (value place &environment env)
   (if (not (consp place))
     `(setq ,place (%temp-cons ,value ,place))
@@ -597,5 +567,7 @@
 (defmacro make-mask (&rest weights)
   `(logior ,@(mapcar #'(lambda (w) `(ash 1 ,w)) weights)))
 
+
+
 (provide "NXENV")
 
