@@ -326,6 +326,7 @@
       `(append-2 ,arg0 ,junk)
       call)))
 
+
 (define-compiler-macro apply  (&whole call fn arg0 &rest args)
   ;; Special-case (apply #'make-instance 'name ...)
   ;; Might be good to make this a little more general, e.g., there
@@ -361,7 +362,17 @@
                   (cons
                    original-fn
                    (nreconc (cdr (reverse args)) (%cdr last))))
-            call))))))
+            (if (and (consp last)
+                     (eq (car last) 'quote)
+                     (proper-list-p (cadr last)))
+              (flet ((quotify (arg)
+                       (if (self-evaluating-p arg)
+                         arg
+                         (list 'quote arg))))
+                (cons 'funcall (cons original-fn
+                                     (nreconc (cdr (reverse args)) (mapcar #'quotify (%cadr last))))))
+              call)))))))
+
 
 
 
@@ -1210,9 +1221,17 @@
       w
       `(/=-2 ,n0 ,n1))))
 
-(define-compiler-macro + (&optional (n0 nil n0p) (n1 nil n1p) &rest more)
+(define-compiler-macro + (&optional (n0 nil n0p) (n1 nil n1p) &rest more &environment env)
   (if more
-    `(+ (+-2 ,n0 ,n1) ,@more)
+    (if (and (nx-trust-declarations env)
+             (subtypep *nx-form-type* 'fixnum)
+             (nx-form-typep n0 'fixnum env)
+             (nx-form-typep n1 'fixnum env)
+             (dolist (m more t)
+               (unless (nx-form-typep m 'fixnum env)
+                 (return nil))))
+      `(+-2 ,n0 (the fixnum (+ ,n1 ,@more)))
+      `(+ (+-2 ,n0 ,n1) ,@more))
     (if n1p
       `(+-2 ,n0 ,n1)
       (if n0p
@@ -1295,19 +1314,18 @@
     w))
 
 
+(define-compiler-macro logandc1 (n0 n1)
+  `(logand (lognot ,n0) ,n1))
+
 (define-compiler-macro logandc2 (n0 n1)
-  (let ((n1var (gensym))
-        (n0var (gensym)))
-    `(let ((,n0var ,n0)
-           (,n1var ,n1))
-       (logandc1 ,n1var ,n0var))))
+  `(logand ,n0 (lognot ,n1)))
+
+
+(define-compiler-macro logorc1 (n0 n1)
+  `(logior (lognot ,n0) ,n1))
 
 (define-compiler-macro logorc2 (n0 n1)
-  (let ((n1var (gensym))
-        (n0var (gensym)))
-    `(let ((,n0var ,n0)
-           (,n1var ,n1))
-       (logorc1 ,n1var ,n0var))))
+  `(logior ,n0 (lognot ,n1)))
 
 (define-compiler-macro lognand (n0 n1)
   `(lognot (logand ,n0 ,n1)))
@@ -1806,14 +1824,15 @@
 ;;; at bits that NIL shares with a cons.
 (define-compiler-macro listp (n)
   (let* ((arch (backend-target-arch *target-backend*))
-         (cons-tag (arch::target-cons-tag arch))
+         (list-tag (logand (arch::target-cons-tag arch)
+                           (1- (ash 1 (arch::target-nlisptagbits arch)))))
          (nil-tag  (arch::target-null-tag arch))
          (symbol-tag (arch::target-symbol-tag arch)))
     (if (= nil-tag symbol-tag)
       (let* ((nvar (gensym)))
         `(let* ((,nvar ,n))
           (if ,nvar (consp ,nvar) t)))
-      `(eql (lisptag ,n) ,cons-tag))))
+      `(eql (lisptag ,n) ,list-tag))))
 
 (define-compiler-macro consp (&whole call n)
   (let* ((arch (backend-target-arch *target-backend*))
@@ -2086,10 +2105,15 @@
     (let* ((typecode (gensym)))
       `(let* ((,typecode (typecode ,x)))
         (declare (type (unsigned-byte 8) ,typecode))
-        #+(or ppc32-target x8632-target)
-        (or (= ,typecode target::tag-fixnum)
-         (and (>= ,typecode target::min-numeric-subtag)
-          (<= ,typecode target::max-real-subtag)))
+        #+(or ppc32-target x8632-target arm-target)
+        (and (<= ,typecode target::max-real-subtag)
+         (logbitp (the (integer 0 ,target::max-real-subtag)
+                    ,typecode)
+                    (logior (ash 1 target::tag-fixnum)
+                           (ash 1 target::subtag-single-float)
+                           (ash 1 target::subtag-double-float)
+                           (ash 1 target::subtag-bignum)
+                           (ash 1 target::subtag-ratio) )))
         #+ppc64-target
         (if (<= ,typecode ppc64::subtag-double-float)
           (logbitp (the (integer 0 #.ppc64::subtag-double-float) ,typecode)
@@ -2479,4 +2503,10 @@
   `(read-char-internal ,stream ,eof-error-p (values ,eof-value ,recursive-p)))
 
 
+(define-compiler-macro concatenate (&whole w type &rest sequences)
+  (if (and (quoted-form-p type)
+           (ignore-errors (subtypep (cadr type) 'string)))
+    `(concat-to-string ,@sequences)
+    w))
+
 (provide "OPTIMIZERS")
