@@ -1,10 +1,10 @@
 #!/bin/sh
-SDK=/Developer/SDKs/MacOSX10.4u.sdk
+SDK=/Developer/SDKs/MacOSX10.5.sdk
 if [ $# -eq 1 ]
 then
 SDK=$1
 fi
-CFLAGS="-Wno-multichar -isysroot ${SDK}" ; export CFLAGS
+CFLAGS="-Wno-multichar -isysroot ${SDK} -mmacosx-version-min=10.5" ; export CFLAGS
 /bin/rm -rf Developer System
 h-to-ffi.sh ${SDK}/System/Library/Frameworks/Carbon.framework/Headers/Carbon.h
 
