@@ -4,7 +4,7 @@ if [ $# -eq 1 ]
 then
 SDK=$1
 fi
-CFLAGS="-isysroot ${SDK} -mmacosx-version-min=10.4"; export CFLAGS
+CFLAGS="-isysroot ${SDK} -mmacosx-version-min=10.5"; export CFLAGS
 rm -rf Developer usr System
 h-to-ffi.sh ${SDK}/usr/include/ar.h
 h-to-ffi.sh ${SDK}/usr/include/arpa/ftp.h
@@ -15,7 +15,7 @@ h-to-ffi.sh ${SDK}/usr/include/arpa/telnet.h
 h-to-ffi.sh ${SDK}/usr/include/arpa/tftp.h
 h-to-ffi.sh ${SDK}/usr/include/bitstring.h
 h-to-ffi.sh ${SDK}/usr/include/bzlib.h
-h-to-ffi.sh ${SDK}/usr/include/c.h
+#h-to-ffi.sh ${SDK}/usr/include/c.h
 h-to-ffi.sh ${SDK}/usr/include/com_err.h
 h-to-ffi.sh ${SDK}/usr/include/crt_externs.h
 h-to-ffi.sh ${SDK}/usr/include/ctype.h
@@ -37,7 +37,7 @@ h-to-ffi.sh ${SDK}/usr/include/err.h
 h-to-ffi.sh ${SDK}/usr/include/errno.h
 h-to-ffi.sh ${SDK}/usr/include/eti.h
 h-to-ffi.sh ${SDK}/usr/include/fcntl.h
-h-to-ffi.sh ${SDK}/usr/include/float.h
+#h-to-ffi.sh ${SDK}/usr/include/float.h
 h-to-ffi.sh ${SDK}/usr/include/fnmatch.h
 h-to-ffi.sh ${SDK}/usr/include/form.h
 h-to-ffi.sh ${SDK}/usr/include/fsproperties.h
@@ -65,7 +65,7 @@ h-to-ffi.sh ${SDK}/usr/include/kerberosIV/des.h
 h-to-ffi.sh ${SDK}/usr/include/kerberosIV/krb.h
 h-to-ffi.sh ${SDK}/usr/include/krb.h
 h-to-ffi.sh ${SDK}/usr/include/krb5.h
-h-to-ffi.sh ${SDK}/usr/include/kvm.h
+#h-to-ffi.sh ${SDK}/usr/include/kvm.h
 h-to-ffi.sh ${SDK}/usr/include/lber.h
 h-to-ffi.sh ${SDK}/usr/include/lber_types.h
 h-to-ffi.sh ${SDK}/usr/include/ldap.h
@@ -524,7 +524,7 @@ h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h ${SDK}/usr/include/sys/lock.
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h -include ${SDK}/usr/include/sys/fcntl.h ${SDK}/usr/include/sys/lockf.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h ${SDK}/usr/include/sys/malloc.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h -include ${SDK}/usr/include/machine/param.h ${SDK}/usr/include/sys/mbuf.h
-h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h  ${SDK}/usr/include/sys/md5.h
+#h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h  ${SDK}/usr/include/sys/md5.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h ${SDK}/usr/include/sys/mman.h
 h-to-ffi.sh ${SDK}/usr/include/sys/mount.h
 h-to-ffi.sh ${SDK}/usr/include/sys/msgbuf.h
@@ -581,7 +581,7 @@ h-to-ffi.sh ${SDK}/usr/include/sys/utsname.h
 #h-to-ffi.sh ${SDK}/usr/include/sys/ux_exception.h
 h-to-ffi.sh ${SDK}/usr/include/sys/vadvise.h
 h-to-ffi.sh ${SDK}/usr/include/sys/vcmd.h
-h-to-ffi.sh ${SDK}/usr/include/sys/version.h
+#h-to-ffi.sh ${SDK}/usr/include/sys/version.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h -include ${SDK}/usr/include/sys/time.h -include ${SDK}/usr/include/sys/vmparam.h ${SDK}/usr/include/sys/vm.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h ${SDK}/usr/include/sys/vmmeter.h
 h-to-ffi.sh -include ${SDK}/usr/include/sys/types.h -include ${SDK}/usr/include/sys/time.h ${SDK}/usr/include/sys/vmparam.h
