@@ -846,10 +846,7 @@
   (trlgei allocptr 0)
   (blr))
 
-(defppclapfunction %metering-info ((ptr arg_z))
-  (ref-global imm0 metering-info)
-  (stw imm0 target::macptr.address ptr)
-  (blr))
+
 
 (defppclapfunction %misc-address-fixnum ((misc-object arg_z))
   (check-nargs 1)
@@ -1024,45 +1021,57 @@
   (beq @lose)
   (%cdr arg_y arg_z)
   (strcx. arg_y rzero imm0)
-  (isync)
   (bne @again)
+  (li imm0 (+ (target-nil-value) (target::kernel-global free-static-conses)))
+  @decf
+  (lrarx imm1 rzero imm0)
+  (subi imm1 imm1 '1)
+  (strcx. imm1 rzero imm0)
+  (bne @decf)
+  (isync)
   (blr)
   @lose
   (li imm0 target::reservation-discharge)
   (strcx. rzero rzero imm0)
   (blr))
 
-(defppclapfunction %augment-static-conses ((head arg_y) (tail arg_z))
-  (li imm0 (+ (target-nil-value) (target::kernel-global static-conses)))
-  @again
-  (lrarx temp0 rzero imm0)
-  (str temp0 target::cons.cdr tail)     ; static, no write-barrier issues
-  (strcx. head rzero imm0)
-  (bne @again)
-  (isync)
-  (li arg_z nil)
-  (blr))
+
 
 (defppclapfunction %staticp ((x arg_z))
   (check-nargs 1)
-  (ref-global temp0 tenured-area)
+  (ref-global temp0 static-cons-area)
   (ldr imm1 target::area.low temp0)
   (sub imm0 x imm1)
-  (ldr imm1 target::area.static-dnodes temp0)
+  (ldr imm1 target::area.ndnodes temp0)
   (srri imm0 imm0 target::dnode-shift)
   (li arg_z nil)
-  (cmplr imm0 imm1)
-  (bgelr)
-  (box-fixnum arg_z imm0)
+  (sub imm1 imm1 imm0)
+  (cmplri imm1 0)
+  (la imm1 128 imm1)
+  (blelr)
+  (box-fixnum arg_z imm1)
   (blr))
 
 (defppclapfunction %static-inverse-cons ((n arg_z))
   (check-nargs 1)
-  (ref-global temp0 tenured-area)
-  (ldr imm1 target::area.low temp0)
-  (add imm1 n imm1)
-  (add imm1 n imm1)
+  (extract-lisptag imm0 arg_z)
+  (cmpri imm0 0)
+  (ref-global temp0 static-cons-area)
+  (bne @fail)
+  (la n '-128 n)
+  (ldr imm0 target::area.ndnodes temp0)
+  (ldr imm1 target::area.high temp0)
+  (box-fixnum arg_y imm0)
+  (sub imm1 imm1 n)
+  (cmplr arg_z arg_y)
+  (sub imm1 imm1 n)
+  (bgt @fail)
   (la arg_z target::fulltag-cons imm1)
+  (ldr arg_y target::cons.car arg_z)
+  (cmpri arg_y target::unbound-marker)
+  (bnelr)
+  @fail
+  (li arg_z nil)
   (blr))
   
 
