@@ -799,38 +799,49 @@
   (lock)
   (cmpxchgl (% temp0) (@ (+ (target-nil-value) (x8632::kernel-global static-conses))))
   (jnz @again)
+  (lock)
+  (subl ($ '1) (@ (+ (target-nil-value) (x8632::kernel-global free-static-conses))))
   @lose
   (movl (% eax) (% arg_z))
   (single-value-return))
 
-(defx8632lapfunction %augment-static-conses ((head arg_y) (tail arg_z))
-  @again
-  (movl (@ (+ (target-nil-value) (x8632::kernel-global static-conses))) (% eax))
-  (movl (% eax) (@ target::cons.cdr (% tail)))
-  (lock)
-  (cmpxchgl (% head) (@ (+ (target-nil-value) (x8632::kernel-global static-conses))))
-  (jnz @again)
-  @lose
-  (movl ($ (target-nil-value)) (% arg_z))
-  (single-value-return))
+
 
 (defx8632lapfunction %staticp ((x arg_z))
   (check-nargs 1)
-  (ref-global tenured-area temp0)
+  (ref-global static-cons-area temp0)
   (movl (% x) (% imm0))
+  (movl ($ (target-nil-value)) (% arg_z))
   (subl (@ target::area.low (% temp0)) (% imm0))
   (shrl ($ target::dnode-shift) (% imm0))
-  (cmpl (@ target::area.static-dnodes (% temp0)) (% imm0))
-  (leal (@ (% imm0) target::fixnumone) (% arg_z))
-  (movl ($ (target-nil-value)) (%l imm0))
-  (cmovael (% imm0) (% arg_z))
+  (mark-as-imm temp1)
+  (movl (@ target::area.ndnodes (% temp0)) (% temp1))
+  (subl (% imm0) (% temp1))
+  (lea (@ 128 (% temp1)) (% temp1))
+  (leal (@ (% temp1) target::fixnumone) (% temp1))
+  (cmoval (% temp1) (% arg_z))
+  (mark-as-node temp1)
   (single-value-return))
 
 (defx8632lapfunction %static-inverse-cons ((n arg_z))
   (check-nargs 1)
-  (ref-global tenured-area temp0)
-  (movl (@ target::area.low (% temp0)) (% imm0))
-  (leal (@ target::fulltag-cons (% imm0) (% n) 2) (% arg_z))
+  (testl ($ target::tagmask) (% arg_z))
+  (jne @fail)
+  (subl ($ '128) (% arg_z))
+  (ref-global static-cons-area temp0)
+  (movl (@ target::area.ndnodes (% temp0)) (% imm0))
+  (box-fixnum imm0 arg_y)
+  (rcmpl (% arg_z) (% arg_y))
+  (ja @fail)
+  (movl (@ target::area.high (% temp0)) (% imm0))
+  (subl (% arg_z) (% imm0))
+  (subl (% arg_z) (% imm0))
+  (lea (@ x8632::fulltag-cons (% imm0)) (% arg_z))
+  (cmpl ($ x8632::subtag-unbound)  (@ x8632::cons.car (% arg_z)))
+  (je @fail)
+  (single-value-return)
+  @fail
+  (movl ($ (target-nil-value)) (% arg_z))
   (single-value-return))
 
 ;;; Get the thread-specific value of %fs.
