@@ -230,118 +230,282 @@
     (jmp-subprim  .SPvalues)))
 
 
+;;; In each of these "simple BOOLE functions", the LEN argument
+;;; describes the size of the bit vectors in whole or partial
+;;; native-sized words.  The 0th word (and, if the number of
+;;; words is even, the last word) are not aligned on 16-byte
+;;; boundaries; any intervening pairs of words are aligned on
+;;; 16-byte boundaries, and we can use aligned SSE2 instructions
+;;; to process these bits 128 at a time.
 
-(defx86lapfunction %boole-clr ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq ($ 0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-clr ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ 1) (%l temp0))
+  (pxor (% xmm0) (% xmm0))
+  (jmp @test)
+  @loop
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq ($ 0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-set ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq ($ -1) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-set ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm0) (% xmm0))
+  (jmp @test)
+  @loop
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq ($ -1) (@ x8664::misc-data-offset (% dest)))
+  (single-value-return 3)
+  (:align 4))
+
+(defx86lapfunction %boole-1 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-1 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-2 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-2 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
-  (single-value-return 3))
-
-(defx86lapfunction %boole-c1 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
+(defx86lapfunction %boole-c1 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movdqa (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (movdqa (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
   (notq (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-c2 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-c2 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-and ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (andq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-and ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pand (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (andq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-ior ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (orq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-ior ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (por (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (orq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-xor ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (xorq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+(defx86lapfunction %boole-xor ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pxor (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (xorq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-eqv ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (xorq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-eqv ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pxor (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (movaps(% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (xorq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-nand ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (andq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-nand ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pand (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (movaps(% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (andq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-nor ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (orq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-nor ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (por (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (movaps(% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (orq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-andc1 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
+(defx86lapfunction %boole-andc1 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (pand (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
   (notq (% imm0))
-  (andq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (andq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-andc2 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-andc2 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (pand (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (andq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (andq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-orc1 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
+(defx86lapfunction %boole-orc1 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (por (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (movaps (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b0)) (% imm0))
   (notq (% imm0))
-  (orq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (orq (@ x8664::misc-data-offset (% b1)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
-(defx86lapfunction %boole-orc2 ((idx 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
-  (movq (@ idx (% rsp)) (% temp0))
-  (movq (@ x8664::misc-data-offset (% b1) (% temp0)) (% imm0))
+(defx86lapfunction %boole-orc2 ((len 8) #|(ra 0)|# (b0 arg_x) (b1 arg_y) (dest arg_z))
+  (movq (@ len (% rsp)) (% temp0))
+  (orl ($ '1) (%l temp0))
+  (pcmpeqb (% xmm1) (% xmm1))
+  (jmp @test)
+  @loop
+  (movaps (@ x8664::misc-data-offset (% b1) (% temp0)) (% xmm0))
+  (pxor (% xmm1) (% xmm0))
+  (por (@ x8664::misc-data-offset (% b0) (% temp0)) (% xmm0))
+  (movq (% xmm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  @test
+  (subq ($ '2) (% temp0))
+  (jg @loop)  
+  (movq (@ x8664::misc-data-offset (% b1)) (% imm0))
   (notq (% imm0))
-  (orq (@ x8664::misc-data-offset (% b0) (% temp0)) (% imm0))
-  (movq (% imm0) (@ x8664::misc-data-offset (% dest) (% temp0)))
+  (orq (@ x8664::misc-data-offset (% b0)) (% imm0))
+  (movq (% imm0) (@ x8664::misc-data-offset (% dest)))
   (single-value-return 3))
 
 (defparameter *simple-bit-boole-functions* ())
@@ -366,9 +530,11 @@
        #'%boole-orc2))
 
 (defun %simple-bit-boole (op b1 b2 result)
-  (let* ((f (svref *simple-bit-boole-functions* op)))
-    (dotimes (i (ash (the fixnum (+ (length result) 63)) -6) result)
-      (funcall f i b1 b2 result))))
+  (funcall (svref *simple-bit-boole-functions* op)
+           (ash (the fixnum (+ (length result) 63)) -6)
+           b1
+           b2
+           result))
 
 (defx86lapfunction %aref2 ((array arg_x) (i arg_y) (j arg_z))
   (check-nargs 3)
