@@ -562,6 +562,7 @@
 ;;;     allocated by WITH-VARIABLE-C-FRAME in %FF-CALL
 ;;; (c) re-establish the same foreign stack frame and store the result regs
 ;;;     (%rax/%xmm0) there
+#-win64-target
 (defx86lapfunction %do-ff-call ((nfp 0) (frame arg_x) (fp-regs arg_y) (entry arg_z))
   (popq (% ra0))
   (popq (% rax))
@@ -588,6 +589,23 @@
   (restore-simple-frame)
   (single-value-return))
 
+#+win64-target
+(defx86lapfunction %do-ff-call ((frame arg_y) (entry arg_z))
+  (save-simple-frame)
+  (movq (@ 16 (% frame)) (% fp0))
+  (movq (@ 24 (% frame)) (% fp1))
+  (movq (@ 32 (% frame)) (% fp2))
+  (movq (@ 40 (% frame)) (% fp3))
+  (call-subprim .SPffcall)
+  (movq (:rcontext x8664::tcr.foreign-sp) (% mm5))
+  (movq (% mm5) (@ (% frame)))
+  (movq (% frame) (:rcontext x8664::tcr.foreign-sp))
+  (movq (% rax) (@ 8 (% frame)))
+  (movq (% fp0) (@ 16 (% frame)))
+  (movl ($ nil) (%l arg_z))
+  (restore-simple-frame)
+  (single-value-return))
+
 (defx86lapfunction %do-ff-call-return-registers ((fp-regs 8)(nfp 0) (frame arg_x) (regbuf arg_y) (entry arg_z))
   (popq (% ra0))
   (popq (% rax))
@@ -613,7 +631,7 @@
   (restore-simple-frame)
   (single-value-return))
   
-
+#-win64-target
 (defun %ff-call (entry &rest specs-and-vals)
   (declare (dynamic-extent specs-and-vals))
   (let* ((len (length specs-and-vals))
@@ -730,6 +748,93 @@
                    (:signed-doubleword (%get-signed-natural argptr 8))
                    (:single-float (%get-single-float argptr 16))
                    (:double-float (%get-double-float argptr 16)))))))))))
+
+#+win64-target
+(defun %ff-call (entry &rest specs-and-vals)
+  (declare (dynamic-extent specs-and-vals))
+  (let* ((len (length specs-and-vals))
+         (total-words 0))
+    (declare (fixnum len total-words))
+    (let* ((result-spec (or (car (last specs-and-vals)) :void))
+           (nargs (ash (the fixnum (1- len)) -1)))
+      (declare (fixnum nargs))
+      (ecase result-spec
+        ((:address :unsigned-doubleword :signed-doubleword
+                   :single-float :double-float
+                   :signed-fullword :unsigned-fullword
+                   :signed-halfword :unsigned-halfword
+                   :signed-byte :unsigned-byte
+                   :void)
+         (do* ((i 0 (1+ i))
+               (specs specs-and-vals (cddr specs))
+               (spec (car specs) (car specs)))
+              ((= i nargs))
+           (declare (fixnum i))
+           (case spec
+             ((:address :unsigned-doubleword :signed-doubleword
+                        :single-float :double-float
+                        :signed-fullword :unsigned-fullword
+                        :signed-halfword :unsigned-halfword
+                        :signed-byte :unsigned-byte)
+              (incf total-words))
+             (t (if (typep spec 'unsigned-byte)
+                  (incf total-words spec)
+                  (error "unknown arg spec ~s" spec)))))
+         ;; It's necessary to ensure that the C frame is the youngest thing on
+         ;; the foreign stack here.
+         (with-macptrs ((argptr))
+             (with-variable-c-frame
+                 total-words frame
+                 (%setf-macptr-to-object argptr frame)
+                 (let* ((arg-offset 16))
+                   (declare (fixnum arg-offset))
+                   (do* ((i 0 (1+ i))
+                         (specs specs-and-vals (cddr specs))
+                         (spec (car specs) (car specs))
+                         (val (cadr specs) (cadr specs)))
+                        ((= i nargs))
+                     (declare (fixnum i))
+                     (case spec
+                       (:address
+                        (setf (%get-ptr argptr arg-offset) val)
+                        (incf arg-offset 8))
+                       ((:signed-doubleword :signed-fullword :signed-halfword
+                                            :signed-byte)
+                        (setf (%%get-signed-longlong argptr arg-offset) val)
+                        (incf arg-offset 8))
+                       ((:unsigned-doubleword :unsigned-fullword :unsigned-halfword
+                                              :unsigned-byte)
+                        (setf (%%get-unsigned-longlong argptr arg-offset) val)
+                        (incf arg-offset 8))
+                       (:double-float
+                        (setf (%get-double-float argptr arg-offset) val)
+                        (incf arg-offset 8))
+                       (:single-float
+                        (setf (%get-single-float argptr arg-offset) val)
+                        (incf arg-offset 8))
+                       (t
+                        (let* ((p 0))
+                          (declare (fixnum p))
+                          (dotimes (i (the fixnum spec))
+                            (setf (%get-ptr argptr arg-offset) (%get-ptr val p))
+                            (incf p 8)
+                            (incf arg-offset 8)))))))
+                 (%do-ff-call frame entry)
+                 (ecase result-spec
+                   (:void nil)
+                   (:address (%get-ptr argptr 8))
+                   (:unsigned-byte (%get-unsigned-byte argptr 8))
+                   (:signed-byte (%get-signed-byte argptr 8))
+                   (:unsigned-halfword (%get-unsigned-word argptr 8))
+                   (:signed-halfword (%get-signed-word argptr 8))
+                   (:unsigned-fullword (%get-unsigned-long argptr 8))
+                   (:signed-fullword (%get-signed-long argptr 8))
+                   (:unsigned-doubleword (%get-natural argptr 8))
+                   (:signed-doubleword (%get-signed-natural argptr 8))
+                   (:single-float (%get-single-float argptr 16))
+                   (:double-float (%get-double-float argptr 16))))))))))
+
+
                                  
 
 ;;; end of x86-def.lisp
