@@ -135,7 +135,7 @@
     (movq (@ src-offset (% rsp)) (% rsrc))
     (cmpq (% dest) (% rsrc))
     (jne @front)
-    (cmpq (% src-byte-offset) (% dest-byte-offset))
+    (cmpq (% rsrc-byte-offset) (% dest-byte-offset))
     (jg @back)
     @front
     (testq (% nbytes) (% nbytes))
@@ -885,38 +885,47 @@
   (lock)
   (cmpxchgq (% temp0) (@ (+ (target-nil-value) (x8664::kernel-global static-conses))))
   (jnz @again)
+  (lock)
+  (subq ($ x8664::fixnumone) (@ (+ (target-nil-value) (x8664::kernel-global free-static-conses))))
   @lose
   (movq (% rax) (% arg_z))
   (single-value-return))
 
-(defx86lapfunction %augment-static-conses ((head arg_y) (tail arg_z))
-  @again
-  (movq (@ (+ (target-nil-value) (x8664::kernel-global static-conses))) (% rax))
-  (movq (% rax) (@ target::cons.cdr (% tail)))
-  (lock)
-  (cmpxchgq (% head) (@ (+ (target-nil-value) (x8664::kernel-global static-conses))))
-  (jnz @again)
-  @lose
-  (movl ($ (target-nil-value)) (% arg_z.l))
-  (single-value-return))
+
   
 (defx86lapfunction %staticp ((x arg_z))
   (check-nargs 1)
-  (ref-global tenured-area temp0)
+  (ref-global static-cons-area temp0)
   (movq (% x) (% imm0))
+  (movl ($ (target-nil-value)) (% arg_z.l))
   (subq (@ target::area.low (% temp0)) (% imm0))
   (shrq ($ target::dnode-shift) (% imm0))
-  (cmpq (@ target::area.static-dnodes (% temp0)) (% imm0))
-  (leaq (@ (% imm0) target::fixnumone) (% arg_z))
-  (movl ($ (target-nil-value)) (%l imm0))
-  (cmovaeq (% imm0) (% arg_z))
+  (movq (@ target::area.ndnodes (% temp0)) (% imm1))
+  (subq (% imm0) (% imm1))
+  (lea (@ 128 (% imm1)) (% imm1))
+  (leaq (@ (% imm1) target::fixnumone) (% imm1))
+  (cmovaq (% imm1) (% arg_z))
   (single-value-return))
 
 (defx86lapfunction %static-inverse-cons ((n arg_z))
   (check-nargs 1)
-  (ref-global tenured-area temp0)
-  (movq (@ target::area.low (% temp0)) (% imm0))
-  (leaq (@ target::fulltag-cons (% imm0) (% n) 2) (% arg_z))
+  (testl ($ target::tagmask) (% arg_z.l))
+  (jne @fail)
+  (subq ($ '128) (% arg_z))
+  (ref-global static-cons-area temp0)
+  (movq (@ target::area.ndnodes (% temp0)) (% imm0))
+  (box-fixnum imm0 arg_y)
+  (rcmpq (% arg_z) (% arg_y))
+  (ja @fail)
+  (movq (@ target::area.high (% temp0)) (% imm0))
+  (subq (% arg_z) (% imm0))
+  (subq (% arg_z) (% imm0))
+  (lea (@ x8664::fulltag-cons (% imm0)) (% arg_z))
+  (cmpb ($ target::unbound-marker) (@ target::cons.car (% arg_z)))
+  (je @fail)
+  (single-value-return)
+  @fail
+  (movl ($ (target-nil-value)) (% arg_z.l))
   (single-value-return))
 
 
