@@ -45,12 +45,12 @@
   ;; some advantage in treating those integers as signed (they might
   ;; be more likely to be fixnums, for instance), so ensure that they
   ;; aren't.
-  #+x86-target
+  #+(or x86-target arm-target)
   (%setf-macptr addr (%int-to-ptr
                       (if (< entry 0)
                         (logand entry (1- (ash 1 target::nbits-in-word)))
                         entry)))
-  #-(or ppc-target x86-target) (dbg "Fix entry->addr"))
+  #-(or ppc-target x86-target arm-target) (dbg "Fix entry->addr"))
 
 
 
@@ -310,12 +310,7 @@
                      
                      
 
-(defun open-shared-library (name)
-  "If the library denoted by name can be loaded by the operating system,
-return an object of type SHLIB that describes the library; if the library
-is already open, increment a reference count. If the library can't be
-loaded, signal a SIMPLE-ERROR which contains an often-cryptic message from
-the operating system."
+(defun open-shared-library-internal (name)
   (let* ((handle (with-cstrs ((name name))
                         (ff-call
                          (%kernel-import target::kernel-import-GetSharedLibrary)
@@ -336,7 +331,7 @@ the operating system."
                          (pref p :address)
                          (%null-ptr))))))
     (if (%null-ptr-p link-map)
-      (error "Error opening shared library ~s: ~a" name (dlerror))
+      (values nil (dlerror))
       (prog1 (let* ((lib (shlib-from-map-entry link-map)))
 	       (incf (shlib.opencount lib))
                (setf (shlib.handle lib) handle)
@@ -400,12 +395,7 @@ the operating system."
     found-lib))
 
 
-(defun open-shared-library (name)
-  "If the library denoted by name can be loaded by the operating system,
-return an object of type SHLIB that describes the library; if the library
-is already open, increment a reference count. If the library can't be
-loaded, signal a SIMPLE-ERROR which contains an often-cryptic message from
-the operating system."
+(defun open-shared-library-internal (name)
   (rlet ((type :signed))
     (let ((result (with-cstrs ((cname name))
 		    (ff-call (%kernel-import target::kernel-import-GetSharedLibrary)
@@ -421,9 +411,8 @@ the operating system."
 	  (shared-library-from-header-module-or-name (%null-ptr) result name))
 	 ((= 0 (pref type :signed))
 	  ;; neither a dylib nor bundle was found
-	  (error "Error opening shared library ~s: ~a" name
-		 (%get-cstring result)))
-	 (t (error "Unknown error opening shared library ~s." name))))))
+          (values nil (%get-cstring result)))
+	 (t (values nil "unknown error"))))))
 
 ;;; Walk over all registered entrypoints, invalidating any whose container
 ;;; is the specified library.  Return true if any such entrypoints were
@@ -489,6 +478,10 @@ the operating system."
   (defvar *get-module-base-name-addr*)
   (defvar *get-module-handle-ex-addr*)
 
+  (defun nbackslash-to-forward-slash (namestring)
+    (dotimes (i (length namestring) namestring)
+      (when (eql (schar namestring i) #\\)
+        (setf (schar namestring i) #\/))))
 
   (defun init-windows-ffi ()
     (%revive-macptr *windows-invalid-handle*)
@@ -501,20 +494,19 @@ the operating system."
   (init-windows-ffi)
   
   (defun hmodule-pathname (hmodule)
-    (do* ((bufsize 64))
+    (do* ((bufsize 128))
          ()
       (%stack-block ((name bufsize))
         (let* ((needed (ff-call *get-module-file-name-addr*
-                                :address *current-process-handle*
                                 :address hmodule
                                 :address name
                                 :signed-fullword bufsize
                                 :signed-fullword)))
           (if (eql 0 needed)
             (return nil)
-            (if (< bufsize needed)
-              (setq bufsize needed)
-              (return (%str-from-ptr name needed))))))))
+            (if (<= bufsize needed)
+              (setq bufsize (+ bufsize bufsize))
+              (return (nbackslash-to-forward-slash (%str-from-ptr name needed)))))))))
 
   (defun hmodule-basename (hmodule)
     (do* ((bufsize 64))
@@ -588,12 +580,7 @@ the operating system."
                 (return lib))))))))
 
 
-  (defun open-shared-library (name)
-    "If the library denoted by name can be loaded by the operating system,
-return an object of type SHLIB that describes the library; if the library
-is already open, increment a reference count. If the library can't be
-loaded, signal a SIMPLE-ERROR which contains an often-cryptic message from
-the operating system."
+  (defun open-shared-library-internal (name)
     (let* ((hmodule (with-cstrs ((name name))
                       (ff-call
                        (%kernel-import target::kernel-import-GetSharedLibrary)
@@ -607,12 +594,49 @@ the operating system."
           (incf (shlib.opencount shlib))
           (setf (shlib.handle shlib) hmodule)
           shlib)
-        (error "Can't open shared library ~s" name))))
+        (values nil (%windows-error-string (get-last-windows-error))))))
 
-(init-shared-libraries)
+  (init-shared-libraries)
+
+  (defun revive-shared-libraries ()
+    (dolist (lib *shared-libraries*)
+      (setf (shlib.map lib) nil
+            (shlib.handle lib) nil
+            (shlib.pathname lib) nil
+            (shlib.base lib) nil)
+      (let* ((soname (shlib.soname lib))
+             (soname-len (length soname)))
+        (block found
+          (for-each-loaded-module
+           (lambda (m)
+             (let* ((module-soname (hmodule-basename m)))
+               (when (%simple-string= soname module-soname 0 0 soname-len (length module-soname))
+                 (let* ((m (%inc-ptr m 0)))
+                   (setf (shlib.base lib) m
+                         (shlib.map lib) m
+                         (shlib.pathname lib) (hmodule-pathname m)))
+                 (return-from found)))))))))
+
+  (defun reopen-user-libraries ()
+    (dolist (lib *shared-libraries*)
+      (unless (shlib.map lib)
+        (let* ((handle (with-cstrs ((name (shlib.soname lib)))
+                         (ff-call
+                          (%kernel-import target::kernel-import-GetSharedLibrary)
+                          :address name
+                          :unsigned-fullword 0
+                          :address))))
+          (unless (%null-ptr-p handle)
+            (setf (shlib.handle lib) handle
+                  (shlib.base lib) handle
+                  (shlib.map lib) handle
+                  (shlib.pathname lib) (hmodule-pathname handle)
+                  (shlib.opencount lib) 1))))))
+           
+              
 
 ;;; end windows-target
-)  
+  )  
 
 
 (defun ensure-open-shlib (c force)
@@ -652,7 +676,7 @@ return a fixnum representation of that address, else return NIL."
 			     :address))
       (unless (%null-ptr-p addr)	; No function can have address 0
 	(or (macptr->fixnum addr) (%inc-ptr addr 0))))
-    #+x8632-target
+    #+(or x8632-target arm-target)
     (let* ((addr (ff-call (%kernel-import target::kernel-import-FindSymbol)
 			  :address handle
 			  :address n
@@ -730,40 +754,41 @@ return a fixnum representation of that address, else return NIL."
 ;;;
 
 (defun legacy-shlib-containing-address (addr name)
-  (dotimes (i (ff-call *dyld-image-count* :unsigned-fullword))
-    (let ((header (ff-call *dyld-get-image-header* :unsigned-fullword i :address)))
-      (when (and (not (%null-ptr-p header))
-                 (or (eql (pref header :mach_header.filetype) #$MH_DYLIB)
-                     (eql (pref header :mach_header.filetype) #$MH_BUNDLE)))
-        ;; make sure the image is either a bundle or a dylib
-        ;; (otherwise we will crash, likely OS bug, tested OS X 10.1.5)
-        (with-cstrs ((cname name))
-          ;; also we must check is symbol name is defined in the
-          ;; image otherwise in certain cases there is a crash,
-          ;; another likely OS bug happens in the case where a
-          ;; bundle imports a dylib and then we call
-          ;; nslookupsymbolinimage on the bundle image
-          (when (/= 0
-                    (ff-call *ns-is-symbol-name-defined-in-image* :address header
-                             :address cname :unsigned))
-            (let ((symbol (ff-call *nslookup-symbol-in-image* :address header :address cname
-                                   :unsigned-fullword #$NSLOOKUPSYMBOLINIMAGE_OPTION_RETURN_ON_ERROR
-                                   :address)))
-              (unless (%null-ptr-p symbol)
-                ;; compare the found address to the address we are looking for
-                (let ((foundaddr (ff-call *nsaddress-of-symbol* :address symbol :address)))
-                  ;; (format t "Foundaddr ~s~%" foundaddr)
-                  ;; (format t "Compare to addr ~s~%" addr)
-                  (when (eql foundaddr addr)
-                    (let* ((imgname (ff-call *dyld-get-image-name* :unsigned-fullword i :address))
-                           (libname (unless (%null-ptr-p imgname) (%get-cstring imgname)))
-                           (libmodule (%int-to-ptr 0))
-                           (libheader (%int-to-ptr 0)))
-                      (if (eql (pref header :mach_header.filetype) #$MH_BUNDLE)
-                        (setf libmodule (ff-call *nsmodule-for-symbol* :address symbol :address))
-                        (setf libheader header))
-                      ;; make sure that this shared library is on *shared-libraries*
-                      (return (shared-library-from-header-module-or-name libheader libmodule libname)))))))))))))
+  (when *ns-is-symbol-name-defined-in-image*
+    (dotimes (i (ff-call *dyld-image-count* :unsigned-fullword))
+      (let ((header (ff-call *dyld-get-image-header* :unsigned-fullword i :address)))
+        (when (and (not (%null-ptr-p header))
+                   (or (eql (pref header :mach_header.filetype) #$MH_DYLIB)
+                       (eql (pref header :mach_header.filetype) #$MH_BUNDLE)))
+          ;; make sure the image is either a bundle or a dylib
+          ;; (otherwise we will crash, likely OS bug, tested OS X 10.1.5)
+          (with-cstrs ((cname name))
+            ;; also we must check is symbol name is defined in the
+            ;; image otherwise in certain cases there is a crash,
+            ;; another likely OS bug happens in the case where a
+            ;; bundle imports a dylib and then we call
+            ;; nslookupsymbolinimage on the bundle image
+            (when (/= 0
+                      (ff-call *ns-is-symbol-name-defined-in-image* :address header
+                               :address cname :unsigned))
+              (let ((symbol (ff-call *nslookup-symbol-in-image* :address header :address cname
+                                     :unsigned-fullword #$NSLOOKUPSYMBOLINIMAGE_OPTION_RETURN_ON_ERROR
+                                     :address)))
+                (unless (%null-ptr-p symbol)
+                  ;; compare the found address to the address we are looking for
+                  (let ((foundaddr (ff-call *nsaddress-of-symbol* :address symbol :address)))
+                    ;; (format t "Foundaddr ~s~%" foundaddr)
+                    ;; (format t "Compare to addr ~s~%" addr)
+                    (when (eql foundaddr addr)
+                      (let* ((imgname (ff-call *dyld-get-image-name* :unsigned-fullword i :address))
+                             (libname (unless (%null-ptr-p imgname) (%get-cstring imgname)))
+                             (libmodule (%int-to-ptr 0))
+                             (libheader (%int-to-ptr 0)))
+                        (if (eql (pref header :mach_header.filetype) #$MH_BUNDLE)
+                          (setf libmodule (ff-call *nsmodule-for-symbol* :address symbol :address))
+                          (setf libheader header))
+                        ;; make sure that this shared library is on *shared-libraries*
+                        (return (shared-library-from-header-module-or-name libheader libmodule libname))))))))))))))
 
 (defun shlib-containing-address (address name)
   (if (zerop *dladdr-entry*)
@@ -973,7 +998,10 @@ return that address encapsulated in a MACPTR, else returns NIL."
     (setup-lookup-calls)
     (reopen-user-libraries))
   #+windows-target
-  (init-windows-ffi)
+  (progn
+    (init-windows-ffi)
+    (revive-shared-libraries)
+    (reopen-user-libraries))
   (when *eeps*
     (without-interrupts 
      (maphash #'(lambda (k v) 
@@ -989,4 +1017,25 @@ return that address encapsulated in a MACPTR, else returns NIL."
                   (resolve-foreign-variable v nil))
               *fvs*))))
 
+(defun open-shared-library (name &optional (process #+darwin-target :initial
+                                                    #-darwin-target :current))
+  "If the library denoted by name can be loaded by the operating system,
+return an object of type SHLIB that describes the library; if the library
+is already open, increment a reference count. If the library can't be
+loaded, signal a SIMPLE-ERROR which contains an often-cryptic message from
+the operating system."
+    (multiple-value-bind (lib error-string)
+        (if (or (eq process :current)
+                (eq process *current-process*)
+                (and (eq process :initial)
+                     (eq *current-process* *initial-process*)))
+          (open-shared-library-internal name)
+          
+          (call-in-process (lambda () (open-shared-library-internal  name))
+                           (if (eq process :initial)
+                             *initial-process*
+                             process)))
+      (or lib
+          (error "Error opening shared library ~a : ~a." name error-string))))
+
 
