@@ -818,6 +818,8 @@
      (%single-float-exp (%short-float x)))))
 
 
+(defun positive-realpart-p (n)
+  (> (realpart n) 0))
 
 (defun expt (b e)
   "Return BASE raised to the POWER."
@@ -825,7 +827,7 @@
 	((integerp e)
          (if (minusp e) (/ 1 (%integer-power b (- e))) (%integer-power b e)))
         ((zerop b)
-         (if (plusp (realpart e)) b (report-bad-arg e '(number (0) *))))
+         (if (plusp (realpart e)) b (report-bad-arg e '(satisfies positive-realpart-p))))
         ((and (realp b) (plusp b) (realp e))
          (if (or (typep b 'double-float)
                  (typep e 'double-float))
@@ -839,6 +841,10 @@
            #+64-bit-target
            (%single-float-expt (%short-float b) (%short-float e))
            ))
+	((typep (realpart e) 'double-float)
+	 ;; Avoid intermediate single-float result from LOG
+	 (let ((promoted-base (* 1d0 b)))
+	   (exp (* e (log promoted-base)))))
         (t (exp (* e (log b))))))
 
 
@@ -910,6 +916,7 @@
 
 
 (eval-when (:execute :compile-toplevel)
+  (defconstant single-float-pi (coerce pi 'single-float))
   (defconstant double-float-half-pi (asin 1.0d0))
   (defconstant single-float-half-pi (asin 1.0f0))
 )
@@ -963,24 +970,30 @@
 
 
 
-(defun %df-atan2 (y x &optional result)
+(defun %df-atan2 (y x)
   (if (zerop x)
     (if (zerop y)
       (if (plusp (float-sign x))
-        y
+        (if (eql y -0.0d0)
+          -0.0d0
+          0.0d0)
         (float-sign y pi))
       (float-sign y double-float-half-pi))
-    (%double-float-atan2! y x (or result (%make-dfloat)))))
+    (%double-float-atan2! y x (%make-dfloat))))
 
 #+32-bit-target
-(defun %sf-atan2! (y x &optional result)
+(defun %sf-atan2! (y x)
   (if (zerop x)
     (if (zerop y)
       (if (plusp (float-sign x))
-        y
-        (float-sign y pi))
+        ;; Don't return Y (which may be stack-consed) here.
+        ;; We know that (ZEROP Y) is true, so:
+        (if (eql y -0.0s0)
+          -0.0s0
+          0.0s0)
+        (float-sign y single-float-pi))
       (float-sign y single-float-half-pi))
-    (%single-float-atan2! y x (or result (%make-sfloat)))))
+    (%single-float-atan2! y x (%make-sfloat))))
 
 #+64-bit-target
 (defun %sf-atan2 (y x)
@@ -988,7 +1001,7 @@
     (if (zerop y)
       (if (plusp (float-sign x))
         y
-        (float-sign y pi))
+        (float-sign y single-float-pi))
       (float-sign y single-float-half-pi))
     (%single-float-atan2 y x)))
 
