@@ -142,17 +142,64 @@
                 (%incf-ptr bufptr nthere)))))))
         
 
+(defun %fasl-read-utf-8-string (s string nchars nextra)
+  (declare (fixnum nchars nextra))
+  (if (eql 0 nextra)
+    (dotimes (i nchars)
+      (setf (%scharcode string i) (%fasl-read-byte s)))
+    (flet ((trailer-byte ()
+             (when (> nextra 0)
+               (decf nextra)
+               (let* ((b (%fasl-read-byte s)))
+                 (declare ((unsigned-byte 8) b))
+                 (and (>= b #x80)
+                      (< b #xc0)
+                      (logand b #x3f))))))
+      (declare (inline trailer-byte))
+      (dotimes (i nchars)
+        (let* ((b0 (%fasl-read-byte s)))
+          (declare ((unsigned-byte 8) b0))
+          (setf (%scharcode string i)
+                (or
+                 (cond ((< b0 #x80) b0)
+                       ((and (>= b0 #xc2)
+                             (< b0 #xe0))
+                        (let* ((b1 (trailer-byte)))
+                          (and b1 (logior (ash (logand b0 #x1f) 6) b1))))
+                       ((and (>= b0 #xe0)
+                             (< b0 #xf0))
+                        (let* ((b1 (trailer-byte))
+                               (b2 (trailer-byte)))
+                          (and b1 b2 (logior (ash (logand b0 #x0f) 12)
+                                             (logior (ash b1 6)
+                                                     b2)))))
+                       ((and (>= b0 #xf0)
+                             (< b0 #xf5))
+                        (let* ((b1 (trailer-byte))
+                               (b2 (trailer-byte))
+                               (b3 (trailer-byte)))
+                          (and b1
+                               b2
+                               b3
+                               (logior (ash (logand b0 #x7) 18)
+                                       (logior (ash b1 12)
+                                               (logior (ash b2 6)
+                                                       b3)))))))
+                 (char-code #\Replacement_Character))))))))
+
+
 (defun %fasl-vreadstr (s)
-  (let* ((nbytes (%fasl-read-count s))
+  (let* ((nchars (%fasl-read-count s))
+         (nextra (%fasl-read-count s))
          (copy t)
-         (n nbytes)
+         (n nchars)
          (str (faslstate.faslstr s)))
-    (declare (fixnum n nbytes))
+    (declare (fixnum nchars n nextra))
     (if (> n (length str))
-        (setq str (make-string n :element-type 'base-char))
-        (setq copy nil))
-    (%fasl-read-n-bytes s str 0 nbytes)
-    (values str n copy)))
+      (setq str (make-string n :element-type 'base-char))
+      (setq copy nil))
+    (%fasl-read-utf-8-string s str nchars nextra)
+    (values str nchars copy)))
 
 
 (defun %fasl-read-n-string (s string start n)
@@ -161,7 +208,7 @@
         (n n (1- n)))
        ((<= n 0))
     (declare (fixnum i n))
-    (setf (schar string i) (code-char (%fasl-read-count s)))))
+    (setf (%scharcode string i) (%fasl-read-byte s))))
 
 (defun %fasl-nvreadstr (s)
   (let* ((nchars (%fasl-read-count s))
@@ -205,9 +252,10 @@
 
 (defun %fasl-vmake-symbol (s &optional idx)
   (let* ((n (%fasl-read-count s))
+         (nextra (%fasl-read-count s))
          (str (make-string n :element-type 'base-char)))
     (declare (fixnum n))
-    (%fasl-read-n-bytes s str 0 n)
+    (%fasl-read-utf-8-string s str n nextra)
     (let* ((sym (make-symbol str)))
       (when idx (ensure-binding-index sym))
       (%epushval s sym))))
@@ -424,10 +472,12 @@
   (%epushval s (host-single-float-from-unsigned-byte-32 (%fasl-read-long s))))
 
 (deffaslop $fasl-vstr (s)
-  (let* ((n (%fasl-read-count s))
-         (str (make-string (the fixnum n) :element-type 'base-char)))
+  (let* ((nchars (%fasl-read-count s))
+         (nextra (%fasl-read-count s))
+         (str (make-string (the fixnum nchars) :element-type 'base-char)))
     (%epushval s str)
-    (%fasl-read-n-bytes s str 0 n)))
+    (%fasl-read-utf-8-string s str nchars nextra)))
+
 
 (deffaslop $fasl-nvstr (s)
   (let* ((n (%fasl-read-count s))
@@ -736,10 +786,10 @@
 
 
 ;;; files compiled with code coverage do this
-;; list of lfuns and (source-fn-name . vector-of-lfuns), the latter put there by fasloading.
+;; list of lfuns and (source-fn-name vector-of-lfuns external-format id), the latter put there by fasloading.
 (defvar *code-covered-functions* nil)
 
-(defun register-code-covered-functions (functions)
+(defun register-code-covered-functions (functions &optional external-format id)
   ;; unpack the parent-note references - see comment at fcomp-digest-code-notes
   (labels ((reg (lfun refs)
 	     (unless (memq lfun refs)
@@ -766,8 +816,11 @@
 				     (let ((p (probe-file p)) (q (probe-file q)))
 				       (and p q (equalp p q)))))))))
     (when (null a)
-      (push (setq a (list nil nil)) *code-covered-functions*))
-    (setf (car a) *loading-file-source-file* (cdr a) functions))
+      (push (setq a (list nil nil nil nil)) *code-covered-functions*))
+    (setf (car a) *loading-file-source-file*
+          (cadr a) functions
+          (caddr a) external-format
+          (cadddr a) id))
   nil)
 
 ;;; The loader itself
