@@ -395,7 +395,7 @@
   (:report
    (lambda (c s)
      (let* ((p (package-error-package c)))
-       (format s "The following symbols need to be imported to ~S before they can be exported ~& from that package:~%~s:" p (slot-value c 'to-be-imported) p)))))
+       (format s "The following symbols need to be imported to ~S before they can be exported ~& from that package:~%~s:" p (slot-value c 'to-be-imported))))))
 
 
 (define-condition package-name-conflict-error (package-error simple-error) ())
@@ -632,11 +632,12 @@
 (defun %active-restart (name)
   (dolist (cluster %restarts%)
     (dolist (restart cluster)
-      (let* ((rname (%restart-name restart))
-	     (rtest (%restart-test restart)))
-	(when (and (or (eq restart name) (eq rname name))
-		   (or (null rtest) (funcall rtest nil)))
-	  (return-from %active-restart (values restart cluster))))))
+      (when (or (eq restart name)
+                (let* ((rname (%restart-name restart))
+                       (rtest (%restart-test restart)))
+                  (and (eq rname name)
+                       (or (null rtest) (funcall rtest nil)))))
+                (return-from %active-restart (values restart cluster)))))
   (error 'inactive-restart :restart-name name))
 
 (defun invoke-restart (restart &rest values)
