@@ -225,7 +225,10 @@
                                           0)))))
     (if (< fd 0)
       (if (and (null if-exists)
-               (eql fd (- #$EEXIST)))
+               (or (eql fd (- #$EEXIST))
+                   #+windows-target
+                   (and (eql fd (- #$EPERM))
+                        (probe-file path))))
         (return-from %create-file nil)
         (signal-file-error fd path))
       (fd-close fd))
@@ -368,7 +371,7 @@
 				      (pathname-name path)
 				      (pathname-type path)
 				      nil)))
-		       nil nil (pathname-host path)))))
+		       nil nil (pathname-host path) nil #+windows-target (pathname-device path)))))
 
 (defun %directory-list-namestring (list &optional logical-p)
   (if (null list)
@@ -528,7 +531,6 @@
                                             (defaults *default-pathname-defaults*))
   (require-type reference-host '(or null string))
   (multiple-value-bind (sstr start end) (get-pathname-sstring string start end)
-    #-windows-target
     (if (and (> end start)
              (eql (schar sstr start) #\~))
       (setq sstr (tilde-expand (subseq sstr start end))
@@ -849,7 +851,7 @@ a host-structure or string."
        (%directory-string-list sstr start (setq pos (%i+ pos 1)) host)
        pos)
       (values (and (neq host :unspecific)
-		   (neq start end)
+                   ;;(neq start end)
 		   '(:absolute))
 	      start))))
 
@@ -1243,6 +1245,22 @@ a host-structure or string."
             (*readtable* *readtable*))
         (load-from-stream file-name print))
       (return-from %load file-name))
+    (when (and (stringp file-name)
+               (eql (length "http://") (string-lessp "http://" file-name)))
+      (when verbose
+        (format t "~&;Loading from URL ~S..." file-name)
+        (force-output))
+      (let* ((vec (if if-does-not-exist
+                    (snarf-url file-name)
+                    (handler-case (snarf-url file-name)
+                      (error () (return-from %load nil)))))
+             (*package* *package*)
+             (*readtable* *readtable*)
+             (*loading-file-source-file* file-name)
+             (*loading-files* (cons file-name (specialv *loading-files*))))
+        (with-input-from-vector (stream vec :external-format external-format)
+          (load-from-stream stream print)))
+      (return-from %load file-name))
     (unless (streamp file-name)
       (multiple-value-setq (*load-truename* *load-pathname* source-file)
         (find-load-file (merge-pathnames file-name)))
@@ -1350,10 +1368,9 @@ a host-structure or string."
 
 (defun delete-file (path)
   "Delete the specified FILE."
-  (let* ((namestring (native-translated-namestring path)))
-    (when (%realpath namestring)
-      (let* ((err (%delete-file namestring)))
-        (or (eql 0 err) (signal-file-error err path))))))
+  (let* ((namestring (native-translated-namestring path))
+	 (err (%delete-file namestring)))
+    (or (eql 0 err) (signal-file-error err path))))
 
 (defvar *known-backends* ())
 
@@ -1380,7 +1397,7 @@ a host-structure or string."
   module)
 
 (defparameter *loading-modules* () "Internal. Prevents circularity")
-(defparameter *module-provider-functions* '(module-provide-search-path module-provide-asdf)
+(defparameter *module-provider-functions* '(module-provide-search-path)
   "A list of functions called by REQUIRE to satisfy an unmet dependency.
 Each function receives a module name as a single argument; if the function knows how to load that module, it should do so, add the module's name as a string to *MODULES* (perhaps by calling PROVIDE) and return non-NIL."
   )
@@ -1441,20 +1458,6 @@ Each function receives a module name as a single argument; if the function knows
 		(when (setq path (find-load-file (merge-pathnames mod-path path-cand)))
 		  (return path)))))))
 
-(defun module-provide-asdf (module)
-  (let* ((asdf-package (find-package "ASDF")))
-    (when asdf-package
-      (let* ((verbose-out (find-symbol "*VERBOSE-OUT*" asdf-package))
-             (find-system (find-symbol "FIND-SYSTEM" asdf-package))
-             (operate (find-symbol "OPERATE" asdf-package))
-             (load-op (find-symbol "LOAD-OP" asdf-package)))
-        (when (and verbose-out find-system operate load-op)
-          (progv (list verbose-out) (list (make-broadcast-stream))
-            (let* ((system (funcall find-system module nil)))
-              (when system
-                (funcall operate load-op module)
-                t))))))))
-
 (defun wild-pathname-p (pathname &optional field-key)
   "Predicate for determining whether pathname contains any wildcards."
   (flet ((wild-p (name) (or (eq name :wild)
