@@ -421,50 +421,98 @@
       (require-type integer 'integer)
       nil)))
 
-(defun %cons-random-state (seed-1 seed-2)
-  #+32-bit-target
-  (%istruct 'random-state seed-1 seed-2)
-  #+64-bit-target
-  (%istruct 'random-state (the fixnum (+ (the fixnum seed-2)
-                          (the fixnum (ash (the fixnum seed-1) 16))))))
-
 ;;; random associated stuff except for the print-object method which
 ;;; is still in "lib;numbers.lisp"
-(defun initialize-random-state (seed-1 seed-2)
-  (unless (and (fixnump seed-1) (%i<= 0 seed-1) (%i< seed-1 #x10000))
-    (report-bad-arg seed-1 '(unsigned-byte 16)))
-  (unless (and (fixnump seed-2) (%i<= 0 seed-2) (%i< seed-2 #x10000))
-    (report-bad-arg seed-2 '(unsigned-byte 16)))
-    (%cons-random-state seed-1 seed-2))
+
+(defun init-random-state-seeds ()
+  (let* ((ticks (ldb (byte 32 0)
+		     (+ (mixup-hash-code (%current-tcr))
+			(let* ((iface (primary-ip-interface)))
+			  (or (and iface (ip-interface-addr iface))
+			      0))
+			(mixup-hash-code
+			 (logand (get-internal-real-time)
+				 (1- target::target-most-positive-fixnum))))))
+	 (high (ldb (byte 16 16) (if (zerop ticks) #x10000 ticks)))
+	 (low (ldb (byte 16 0) ticks)))
+    (declare (fixnum high low))
+    (values high low)))
+
+(defun %cons-mrg31k3p-state (x0 x1 x2 x3 x4 x5)
+  (let ((array (make-array 6 :element-type '(unsigned-byte 32)
+			   :initial-contents (list x0 x1 x2 x3 x4 x5))))
+    (%istruct 'random-state array)))
+
+(defun initialize-mrg31k3p-state (x0 x1 x2 x3 x4 x5)
+  (let ((args (list x0 x1 x2 x3 x4 x5)))
+    (declare (dynamic-extent args))
+    (dolist (a args)
+      (unless (and (integerp a) (<= 0 a) (< a mrg31k3p-limit))
+	(report-bad-arg a `(integer 0 (,mrg31k3p-limit)))))
+    (when (and (zerop x0) (zerop x1) (zerop x2))
+      (error "The first three arguments must not all be zero."))
+    (when (and (zerop x3) (zerop x4) (zerop x5))
+      (error "The second three arguments must not all be zero."))
+    (%cons-mrg31k3p-state x0 x1 x2 x3 x4 x5)))
+
+#+windows-target
+(defun random-mrg31k3p-state ()
+  (flet ((random-u32 ()
+	   (%stack-block ((buf 4))
+	     ;; BOOLEAN RtlGenRandom(PVOID buf, ULONG len)
+	     (let ((r (external-call "SystemFunction036" :address buf
+				     :unsigned 4 :byte)))
+	       (if (plusp r)
+		 (%get-unsigned-long buf)
+		 (init-random-state-seeds))))))
+    (loop repeat 6
+	  for n = (random-u32)
+	  ;; The first three seed elements must not be all zero, and
+	  ;; likewise for the second three.  Avoid the issue by
+	  ;; excluding zero values.
+	  collect (1+ (mod n (1- mrg31k3p-limit))) into seed
+	  finally (return (apply #'%cons-mrg31k3p-state seed)))))
+
+#-windows-target
+(defun random-mrg31k3p-state ()
+  (with-open-file (stream "/dev/urandom" :element-type '(unsigned-byte 32)
+			  :if-does-not-exist nil)
+    (loop repeat 6
+	  for n = (if stream (read-byte stream) (init-random-state-seeds))
+	  ;; The first three seed elements must not be all zero, and
+	  ;; likewise for the second three.  Avoid the issue by
+	  ;; excluding zero values.
+	  collect (1+ (mod n (1- mrg31k3p-limit))) into seed
+	  finally (return (apply #'%cons-mrg31k3p-state seed)))))
+
+(defun initial-random-state ()
+  (initialize-mrg31k3p-state 314159 42 1776 271828 6021023 1066))
 
 (defun make-random-state (&optional state)
-  "Make a random state object. If STATE is not supplied, return a copy
-  of the default random state. If STATE is a random state, then return a
-  copy of it. If STATE is T then return a random state generated from
-  the universal time."
-  (let* ((seed-1 0)
-         (seed-2 0))
-    (if (eq state t)
-      (multiple-value-setq (seed-1 seed-2) (init-random-state-seeds))
-      (progn
-        (setq state (require-type (or state *random-state*) 'random-state))
-        #+32-bit-target
-        (setq seed-1 (random.seed-1 state) seed-2 (random.seed-2 state))
-        #+64-bit-target
-        (let* ((seed (random.seed-1 state)))
-          (declare (type (unsigned-byte 32) seed))
-          (setq seed-1 (ldb (byte 16 16) seed)
-                seed-2 (ldb (byte 16 0) seed)))))
-    (%cons-random-state seed-1 seed-2)))
+  "Make a new random state object. If STATE is not supplied, return a
+  copy of the current random state. If STATE is a random state, then
+  return a copy of it. If STATE is T then return a randomly
+  initialized random state."
+  (if (eq state t)
+    (random-mrg31k3p-state)
+    (progn
+      (setq state (require-type (or state *random-state*) 'random-state))
+      (let ((seed (coerce (random.mrg31k3p-state state) 'list)))
+	(apply #'%cons-mrg31k3p-state seed)))))
 
 (defun random-state-p (thing) (istruct-typep thing 'random-state))
 
+(defun %random-state-equalp (x y)
+  ;; x and y are both random-state objects
+  (equalp (random.mrg31k3p-state x) (random.mrg31k3p-state y)))
+
 ;;; transcendental stuff.  Should go in level-0;l0-float
 ;;; but shleps don't work in level-0.  Or do they ?
 ; Destructively set z to x^y and return z.
 (defun %double-float-expt! (b e result)
   (declare (double-float b e result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float temp (#_pow b e))
     (%df-check-exception-2 'expt b e (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -473,6 +521,7 @@
 (defun %single-float-expt! (b e result)
   (declare (single-float b e result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float temp (#_powf b e))
     (%sf-check-exception-2 'expt b e (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -495,6 +544,7 @@
 (defun %double-float-sin! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_sin n))
     (%df-check-exception-1 'sin n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -503,6 +553,7 @@
 (defun %single-float-sin! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_sinf n))
     (%sf-check-exception-1 'sin n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -517,6 +568,7 @@
 (defun %double-float-cos! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_cos n))
     (%df-check-exception-1 'cos n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -525,6 +577,7 @@
 (defun %single-float-cos! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_cosf n))
     (%sf-check-exception-1 'cos n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -539,6 +592,7 @@
 (defun %double-float-acos! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_acos n))
     (%df-check-exception-1 'acos n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -547,6 +601,7 @@
 (defun %single-float-acos! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_acosf n))
     (%sf-check-exception-1 'acos n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -561,6 +616,7 @@
 (defun %double-float-asin! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_asin n))
     (%df-check-exception-1 'asin n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -569,6 +625,7 @@
 (defun %single-float-asin! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_asinf n))
     (%sf-check-exception-1 'asin n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -583,6 +640,7 @@
 (defun %double-float-cosh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_cosh n))
     (%df-check-exception-1 'cosh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -591,6 +649,7 @@
 (defun %single-float-cosh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "coshf" :single-float n :single-float))
     (%sf-check-exception-1 'cosh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -605,6 +664,7 @@
 (defun %double-float-log! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_log n))
     (%df-check-exception-1 'log n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -613,6 +673,7 @@
 (defun %single-float-log! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_logf n))
     (%sf-check-exception-1 'log n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -626,6 +687,7 @@
 (defun %double-float-tan! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_tan n))
     (%df-check-exception-1 'tan n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -634,6 +696,7 @@
 (defun %single-float-tan! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_tanf n))
     (%sf-check-exception-1 'tan n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -648,6 +711,7 @@
 (defun %double-float-atan! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_atan n))
     (%df-check-exception-1 'atan n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -657,6 +721,7 @@
 (defun %single-float-atan! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_atanf n))
     (%sf-check-exception-1 'atan n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -671,6 +736,7 @@
 (defun %double-float-atan2! (x y result)
   (declare (double-float x y result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_atan2 x y))
     (%df-check-exception-2 'atan2 x y (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -679,6 +745,7 @@
 (defun %single-float-atan2! (x y result)
   (declare (single-float x y result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_atan2f x y))
     (%sf-check-exception-2 'atan2 x y (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -693,7 +760,11 @@
 (defun %double-float-exp! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
+    #-freebsdx8632-target
     (%setf-double-float TEMP (#_exp n))
+    #+freebsdx8632-target
+    (%setf-double-float TEMP (external-call "__ieee754_exp" :double-float n :double-float))
     (%df-check-exception-1 'exp n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
 
@@ -701,7 +772,11 @@
 (defun %single-float-exp! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
+    #-freebsdx8632-target
     (%setf-short-float TEMP (#_expf n))
+    #+freebsdx8632-target
+    (%setf-short-float TEMP (external-call "__ieee754_expf" :single-float n :single-float))
     (%sf-check-exception-1 'exp n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
 
@@ -723,6 +798,7 @@
 (defun %double-float-sinh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_sinh n))
     (%df-check-exception-1 'sinh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -731,6 +807,7 @@
 (defun %single-float-sinh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "sinhf" :single-float n :single-float))
     (%sf-check-exception-1 'sinh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -745,6 +822,7 @@
 (defun %double-float-tanh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_tanh n))
     (%df-check-exception-1 'tanh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -753,6 +831,7 @@
 (defun %single-float-tanh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "tanhf" :single-float n :single-float))
     (%sf-check-exception-1 'tanh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -777,6 +856,7 @@
 (defun %single-float-asinh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "asinhf" :float n :float))
     (%sf-check-exception-1 'asinh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -793,6 +873,7 @@
 (defun %double-float-asinh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_asinh n))
     (%df-check-exception-1 'asinh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -802,6 +883,7 @@
 (defun %single-float-asinh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_asinhf n))
     (%sf-check-exception-1 'asinh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -827,6 +909,7 @@
 (defun %single-float-acosh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "acoshf" :float n :float))
     (%sf-check-exception-1 'acosh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -845,6 +928,7 @@
 (defun %double-float-acosh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_acosh n))
     (%df-check-exception-1 'acosh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -853,6 +937,7 @@
 (defun %single-float-acosh! (n result)
   (declare (single-float n result))
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_acoshf n))
     (%sf-check-exception-1 'acosh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -878,6 +963,7 @@
 (defun %single-float-atanh! (n result)
   (declare (single-float n result)) 
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (external-call "atanhf" :float n :float))
     (%sf-check-exception-1 'atanh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
@@ -896,6 +982,7 @@
 (defun %double-float-atanh! (n result)
   (declare (double-float n result))
   (with-stack-double-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-double-float TEMP (#_atanh n))
     (%df-check-exception-1 'atanh n (%ffi-exception-status))
     (%setf-double-float result TEMP)))
@@ -904,6 +991,7 @@
 (defun %single-float-atanh! (n result)
   (declare (single-float n result)) 
   (target::with-stack-short-floats ((temp))
+    #+arm-target (%set-fpscr-status 0)
     (%setf-short-float TEMP (#_atanhf n))
     (%sf-check-exception-1 'atanh n (%ffi-exception-status))
     (%setf-short-float result TEMP)))
