@@ -386,7 +386,9 @@
   ;; Note that %component-match-p is case sensitive.  Need a
   ;; case-insensitive version for hosts. 
   ;; In addition, host components do not support wildcards.
-  (or (eq path-host wild-host)
+  (or (null wild-host) (eq wild-host :wild)
+      (null path-host) (eq path-host :wild)
+      (eq path-host wild-host)
       (and (stringp path-host)
 	   (stringp wild-host)
 	   (string-equal path-host wild-host))))
@@ -620,19 +622,12 @@
 (defun user-homedir-pathname (&optional host)
   "Return the home directory of the user as a pathname."
   (declare (ignore host))
-  (let* ((native
-          (ignore-errors
-            (truename
-             (native-to-directory-pathname (or #+ccl-0711 (getenv "HOME")
-                                               (get-user-home-dir (getuid))))))))
-    (if (and native (eq :absolute (car (pathname-directory native))))
-      native
-      ;; Another plausible choice here is
-      ;; #p"/tmp/.hidden-directory-of-some-irc-bot-in-eastern-europe/"
-      ;; Of course, that might already be the value of $HOME.  Anyway,
-      ;; the user's home directory just contains "config files" (like
-      ;; SSH keys), and spoofing it can't hurt anything.
+  (let* ((native (get-user-home-dir (getuid)))
+	 (pathname (and native (native-to-directory-pathname native))))
+    (if (and pathname (eq :absolute (car (pathname-directory pathname))))
+      pathname
       (make-pathname :directory '(:absolute) :defaults nil))))
+  
 
 
 
@@ -680,14 +675,20 @@
                  (error (condition) (if no-error
                                       (return-from full-pathname nil)
                                       (error condition)))))
-         (dir (%pathname-directory path)))
-    (if (eq (car dir) :absolute)
-      path
-      (cons-pathname (absolute-directory-list dir)
-                       (%pathname-name path)
-                       (%pathname-type path)
-                       (pathname-host path)
-                       (pathname-version path)))))
+         (dir (%pathname-directory path))
+	 (device #+windows-target
+	         (or (pathname-device path)
+		     (pathname-device (mac-default-directory)))
+		 #-windows-target
+		 nil))
+    (cons-pathname (if (eq (car dir) :absolute)
+		     dir
+		     (absolute-directory-list dir))
+		   (%pathname-name path)
+		   (%pathname-type path)
+		   (pathname-host path)
+		   (pathname-version path)
+		   device)))
 
 
 
