@@ -370,11 +370,9 @@ a given process."
       #'(lambda (process initial-form)
 	  (let* ((*current-process* process))
 	    (add-to-all-processes process)
-	    (multiple-value-bind (syms values)
-		(initial-bindings (process-initial-bindings process))
-	      (progv syms values
-                (setq *whostate* "Active")
-		(run-process-initial-form process initial-form)))))
+            (with-initial-bindings (process-initial-bindings process)
+              (setq *whostate* "Active")
+              (run-process-initial-form process initial-form))))
       process
       initial-form)
      process))
@@ -638,19 +636,23 @@ some point in the near future, and then return to what it was doing."
   (when (eq process *initial-process*)
     (with-standard-abort-handling "Exit Lisp"
       (prepare-to-quit)
-      ;; We may have abruptly terminated a thread
-      ;; which owned the output lock on *STDOUT*.
-      ;; Don't block waiting on that lock if so.
-      (let* ((s *stdout*)
-	     (lock (ioblock-outbuf-lock (basic-stream-ioblock s)))
-	     (locked (make-lock-acquisition)))
-	(declare (dynamic-extent locked))
-	(when (or (null lock) (%try-recursive-lock-object lock locked))
-	  (unwind-protect
-	       (progn
-		 (fresh-line s)
-		 (finish-output s)))
-	  (when (lock-acquisition.status locked) (release-lock lock)))))
+      ;; We may have abruptly terminated a thread which owned the
+      ;; output lock on a stream we want to flush.  Don't block
+      ;; waiting on the lock if so.
+      (flet ((flush-stream (s)
+	       (let* ((lock (ioblock-outbuf-lock (basic-stream-ioblock s)))
+		      (locked (make-lock-acquisition)))
+		 (declare (dynamic-extent locked))
+		 (when (or (null lock)
+			   (%try-recursive-lock-object lock locked))
+		   (unwind-protect
+			(progn
+			  (fresh-line s)
+			  (finish-output s))
+		     (when (lock-acquisition.status locked)
+		       (release-lock lock)))))))
+	(flush-stream *stdout*)
+	(flush-stream *stderr*)))
     (%set-toplevel thunk)
     (toplevel)))
 
@@ -744,4 +746,19 @@ had invoked abort."
     (cond ((car result) (values-list (cdr result)))
           (t default))))
 
+(defun call-in-process (f process)
+  (let* ((return-values nil)
+	 (done (make-semaphore)))
+    (process-interrupt process
+		       #'(lambda ()
+			   (unwind-protect
+				(progn
+				  (setq return-values
+					(multiple-value-list (funcall f))))
+			     (signal-semaphore done))))
+    (wait-on-semaphore done)
+    (apply #'values return-values)))
+
+(defun call-in-initial-process (f)
+  (call-in-process f *initial-process*))
 
