@@ -101,18 +101,16 @@ whose name or ID matches <p>, or to any process if <p> is null"
 (define-toplevel-command :break q () "return to toplevel" (toplevel))
 (define-toplevel-command :break r () "list restarts" (list-restarts))
 
-(define-toplevel-command :break set (n frame value) "Set <n>th item of frame <frame> to <value>"
-  (let* ((frame-sp (nth-raw-frame frame *break-frame* nil)))
-    (if frame-sp
-        (toplevel-print (list (set-nth-value-in-frame frame-sp n nil value)))
-        (format *debug-io* "No frame with number ~D~%" frame))))
 
 (define-toplevel-command :break nframes ()
   "print the number of stack frames accessible from this break loop"
   (do* ((p *break-frame* (parent-frame p nil))
-        (i 0 (1+ i))
+        (i 0 )
         (last (last-frame-ptr)))
-      ((eql p last) (toplevel-print (list i)))))
+      ((eql p last) (toplevel-print (list i)))
+    (declare (fixnum i))
+    (when (function-frame-p p nil)
+      (incf i))))
 
 (define-toplevel-command :global ? () "help"
   (format t "~&The following toplevel commands are available:")
@@ -148,12 +146,12 @@ whose name or ID matches <p>, or to any process if <p> is null"
                        :detailed-p t))
 
 (define-toplevel-command :break return-from-frame (i &rest values) "Return VALUES from the I'th stack frame"
-  (let* ((frame-sp (nth-raw-frame  i *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame  i *break-frame* nil)))
     (if frame-sp
       (apply #'return-from-frame frame-sp values))))
 
 (define-toplevel-command :break apply-in-frame (i function &rest args) "Applies FUNCTION to ARGS in the execution context of the Ith stack frame"
-  (let* ((frame-sp (nth-raw-frame  i *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame  i *break-frame* nil)))
     (if frame-sp
       (apply-in-frame frame-sp function args))))
                          
@@ -166,12 +164,12 @@ whose name or ID matches <p>, or to any process if <p> is null"
                        :detailed-p :raw))
 
 (define-toplevel-command :break v (n frame-number) "Return value <n> in frame <frame-number>"
-  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame frame-number *break-frame* nil)))
     (if frame-sp
       (toplevel-print (list (nth-value-in-frame frame-sp n nil))))))
 
 (define-toplevel-command :break arg (name frame-number) "Return value of argument named <name> in frame <frame-number>"
-  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame frame-number *break-frame* nil)))
     (when frame-sp
       (multiple-value-bind (lfun pc) (cfp-lfun frame-sp)
         (when (and lfun pc)
@@ -183,7 +181,7 @@ whose name or ID matches <p>, or to any process if <p> is null"
                 (toplevel-print (list value))))))))))
 
 (define-toplevel-command :break set-arg (name frame-number new) "Set value of argument named <name> in frame <frame-number> to value <new>."
-  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame frame-number *break-frame* nil)))
     (when frame-sp
       (multiple-value-bind (lfun pc) (cfp-lfun frame-sp)
         (when (and lfun pc)
@@ -193,7 +191,7 @@ whose name or ID matches <p>, or to any process if <p> is null"
 
 (define-toplevel-command :break local (name frame-number) "Return value of local denoted by <name> in frame <frame-number> <name> can either be a symbol - in which case the most recent
 binding of that symbol is used - or an integer index into the frame's set of local bindings."
-  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame frame-number *break-frame* nil)))
     (when frame-sp
       (multiple-value-bind (lfun pc) (cfp-lfun frame-sp)
         (when (and lfun pc)
@@ -205,7 +203,7 @@ binding of that symbol is used - or an integer index into the frame's set of loc
                 (toplevel-print (list value))))))))))
 
 (define-toplevel-command :break set-local (name frame-number new) "Set value of argument denoted <name> (see :LOCAL) in frame <frame-number> to value <new>."
-  (let* ((frame-sp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((frame-sp (nth-function-frame frame-number *break-frame* nil)))
     (when frame-sp
       (multiple-value-bind (lfun pc) (cfp-lfun frame-sp)
         (when (and lfun pc)
@@ -224,7 +222,7 @@ binding of that symbol is used - or an integer index into the frame's set of loc
 ;;; Ordinarily, form follows function.
 (define-toplevel-command :break function (frame-number)
   "Returns the function invoked in backtrace frame <frame-number>.  This may be useful for, e.g., disassembly"
-  (let* ((cfp (nth-raw-frame frame-number *break-frame* nil)))
+  (let* ((cfp (nth-function-frame frame-number *break-frame* nil)))
     (when (and cfp (not (catch-csp-p cfp nil)))
       (let* ((function (cfp-lfun cfp)))
         (when function
@@ -299,6 +297,8 @@ commands but aren't")
   (let ((fd (stream-device stream :input)))
     (and fd (eof-transient-p fd))))
 
+(defvar *save-interactive-source-locations* t)
+
 ;;; This is the part common to toplevel loop and inner break loops.
 (defun read-loop (&key (input-stream *standard-input*)
                        (output-stream *standard-output*)
@@ -316,7 +316,8 @@ commands but aren't")
          *** ** * +++ ++ + /// // / -
          (eof-value (cons nil nil))
          (eof-count 0)
-         (*show-available-restarts* (and *show-restarts-on-break* *break-condition*)))
+         (*show-available-restarts* (and *show-restarts-on-break* *break-condition*))
+         (map (make-hash-table :test #'eq :shared nil)))
     (declare (dynamic-extent eof-value))
     (loop
       (restart-case
@@ -330,7 +331,10 @@ commands but aren't")
                   (toplevel-read :input-stream input-stream
                                  :output-stream output-stream
                                  :prompt-function prompt-function
-                                 :eof-value eof-value)
+                                 :eof-value eof-value
+				 :map (when *save-interactive-source-locations*
+                                        (clrhash map)
+                                        map))
                 (if (eq form eof-value)
                   (progn
                     (when (> (incf eof-count) *consecutive-eof-limit*)
@@ -342,7 +346,7 @@ commands but aren't")
                         (stream-clear-input input-stream)
                         (abort-break))
                       (exit-interactive-process *current-process*)))
-                  (progn
+                  (let ((*nx-source-note-map* (and *save-interactive-source-locations* map)))
                     (setq eof-count 0)
                     (or (check-toplevel-command form)
                         (let* ((values (toplevel-eval form env)))
@@ -391,10 +395,11 @@ commands but aren't")
 (defun toplevel-read (&key (input-stream *standard-input*)
                            (output-stream *standard-output*)
                            (prompt-function #'print-listener-prompt)
-                           (eof-value *eof-value*))
+                           (eof-value *eof-value*)
+		           (map nil))
   (force-output output-stream)
   (funcall prompt-function output-stream)
-  (read-toplevel-form input-stream :eof-value eof-value))
+  (read-toplevel-form input-stream :eof-value eof-value :map map))
 
 (defvar *always-eval-user-defvars* nil)
 
@@ -484,7 +489,10 @@ commands but aren't")
       (let ((hook *debugger-hook*)
             (*debugger-hook* nil))
         (funcall hook condition hook)))
-    (%break-message "Error" condition)
+    (let ((msg (if *batch-flag* ;; Give a little more info if exiting
+                 (format nil "Error of type ~s" (type-of condition))
+                 "Error")))
+      (%break-message msg condition))
     (let* ((s *error-output*))
       (dolist (bogusness bogus-globals)
         (let ((oldval (pop oldvals)))
@@ -550,18 +558,24 @@ commands but aren't")
         (*print-readably* nil)
         (*print-right-margin* nil)
         (*signal-printing-errors* nil)
-        (s (make-indenting-string-output-stream prefixchar nil)))
+        (s (make-indenting-string-output-stream prefixchar nil))
+        (sub (make-string-output-stream))
+        (indent 0))
     (format s "~A ~A: " prefixchar msg)
-    (setf (indenting-string-output-stream-indent s) (column s))
+    (setf (indenting-string-output-stream-indent s) (setq indent (column s)))
+    (decf (stream-line-length sub) indent)
     ;(format s "~A" condition) ; evil if circle
-    (report-condition condition s)
+    (report-condition condition sub)
+    (format s "~A" (get-output-stream-string sub))
     (if (not (and (typep condition 'simple-program-error)
                   (simple-program-error-context condition)))
       (format *error-output* "~&~A~%~A While executing: ~S"
               (get-output-stream-string s) prefixchar (%real-err-fn-name error-pointer))
       (format *error-output* "~&~A"
               (get-output-stream-string s)))
-    (format *error-output* ", in process ~a(~d).~%" (process-name *current-process*) (process-serial-number *current-process*))
+    (if *current-process*
+      (format *error-output* ", in process ~a(~d).~%" (process-name *current-process*) (process-serial-number *current-process*))
+      (format *error-output* ", in an uninitialized process~%"))
   (force-output *error-output*)))
 					; returns NIL
 
@@ -622,7 +636,7 @@ commands but aren't")
 (defvar *break-condition* nil "condition argument to innermost break-loop.")
 (defvar *break-frame* nil "frame-pointer arg to break-loop")
 (defvar *break-loop-when-uninterruptable* t)
-(defvar *show-restarts-on-break* #+ccl-0711 t #-ccl-0711 nil)
+(defvar *show-restarts-on-break* nil)
 (defvar *show-available-restarts* nil)
 
 (defvar *error-reentry-count* 0)
@@ -667,6 +681,7 @@ commands but aren't")
                                       (%current-frame-ptr)
                                       #+ppc-target *fake-stack-frames*
                                       #+x86-target (%current-frame-ptr)
+                                      #+arm-target (or (current-fake-stack-frame) (%current-frame-ptr))
                                       (db-link)
                                       (1+ *break-level*)))
          (*backtrace-contexts* (cons context *backtrace-contexts*)))
