@@ -683,8 +683,11 @@
   (let ((defenv (definition-environment env)))
     (and defenv (eq 'compile-file (car (defenv.type defenv))))))
 
+;; This is EVAL.
 (defun cheap-eval (form)
-  (cheap-eval-in-environment form nil))
+  ;; Don't record source locations for explicit calls to EVAL.
+  (let ((*nx-source-note-map* nil))
+    (cheap-eval-in-environment form nil)))
 
 ; used by nfcomp too
 ; Should preserve order of decl-specs; it sometimes matters.
@@ -720,7 +723,8 @@
 (fset 'nx-source-note (nlambda bootstrapping-source-note (form) (declare (ignore form)) nil))
 
 (defun cheap-eval-in-environment (form env &aux sym)
-  (declare (resident))
+  ;; Allow ADVICE, TRACE to have effects on self-calls.
+  (declare (notinline cheap-eval-in-environment))
   ;; records source locations if *nx-source-note-map* is bound by caller
   (setq *loading-toplevel-location* (or (nx-source-note form) *loading-toplevel-location*))
   (flet ((progn-in-env (body&decls parse-env base-env)
@@ -815,7 +819,8 @@
                      (let ((*loading-toplevel-location* *loading-toplevel-location*))
                        (cheap-eval-in-environment protected-form env))
                    (progn-in-env cleanup-forms env env)))
-               (funcall (cheap-eval-function nil (cheap-eval-transform form `(lambda () (progn ,form))) env))))
+               (let ((fn (cheap-eval-function nil (cheap-eval-transform form `(lambda () (progn ,form))) env)))
+                 (funcall fn))))
             ((and (symbolp sym) (macro-function sym env))
              (cheap-eval-in-environment (cheap-eval-macroexpand-1 form env) env))
             ((or (symbolp sym)
@@ -826,7 +831,8 @@
                  (setq *loading-toplevel-location* form-location))
                (apply #'call-check-regs (if (symbolp sym) sym (cheap-eval-function nil sym env))
                       (nreverse args))))
-            (t (signal-simple-condition 'simple-program-error "Car of ~S is not a function name or lambda-expression." form))))))
+            (t
+             (signal-simple-condition 'simple-program-error "Car of ~S is not a function name or lambda-expression." form))))))
 
 
 (%fhave 'eval #'cheap-eval)
