@@ -47,6 +47,10 @@
 	    SOCKET-ERROR-CODE
 	    SOCKET-ERROR-IDENTIFIER
 	    SOCKET-ERROR-SITUATION
+	    SOCKET-CREATION-ERROR
+	    SOCKET-CREATION-ERROR-CODE
+	    SOCKET-CREATION-ERROR-IDENTIFIER
+	    SOCKET-CREATION-ERROR-SITUATION
 	    WITH-OPEN-SOCKET))
   #+windows-target
   (defmacro check-winsock-error (form)
@@ -133,6 +137,10 @@
 		"SOCKET-ERROR-CODE"
 		"SOCKET-ERROR-IDENTIFIER"
 		"SOCKET-ERROR-SITUATION"
+		"SOCKET-CREATION-ERROR"
+		"SOCKET-CREATION-ERROR-CODE"
+		"SOCKET-CREATION-ERROR-IDENTIFIER"
+		"SOCKET-CREATION-ERROR-SITUATION"
 		"WITH-OPEN-SOCKET")
   (:export  "MAKE-SOCKET"
 	    "ACCEPT-CONNECTION"
@@ -161,20 +169,27 @@
 	    "SOCKET-ERROR-CODE"
 	    "SOCKET-ERROR-IDENTIFIER"
 	    "SOCKET-ERROR-SITUATION"
+	    "SOCKET-CREATION-ERROR"
+	    "SOCKET-CREATION-ERROR-CODE"
+	    "SOCKET-CREATION-ERROR-IDENTIFIER"
+	    "SOCKET-CREATION-ERROR-SITUATION"
 	    "WITH-OPEN-SOCKET"))
 
-
-
 (define-condition socket-error (simple-stream-error)
   ((code :initarg :code :reader socket-error-code)
    (identifier :initform :unknown :initarg :identifier :reader socket-error-identifier)
-   (Situation :initarg :situation :reader socket-error-situation)))
+   (situation :initarg :situation :reader socket-error-situation)))
 
 (define-condition socket-creation-error (simple-error)
   ((code :initarg :code :reader socket-creation-error-code)
    (identifier :initform :unknown :initarg :identifier :reader socket-creation-error-identifier)
    (situation :initarg :situation :reader socket-creation-error-situation)))
 
+(defparameter *gai-error-identifiers*
+  (list #$EAI_AGAIN :try-again
+	#$EAI_FAIL :no-recovery
+	#$EAI_NONAME :host-not-found))
+
 (defvar *socket-error-identifiers*
   #-windows-target
   (list #$EADDRINUSE :address-in-use
@@ -222,55 +237,46 @@
     res))
 
 #-windows-target
-(defun %gai-strerror (errno)
-  (let* ((err (abs errno))
-         (p (#_gai_strerror err)))
+(defun %gai-strerror (err)
+  (let ((p (#_gai_strerror err)))
     (if (%null-ptr-p p)
       (format nil "Unknown nameserver error ~d" err)
       (%get-cstring p))))
-    
-
 
 (defun socket-error (stream where errno &optional nameserver-p)
   "Creates and signals (via error) one of two socket error 
 conditions, based on the state of the arguments."
-  #+windows-target (declare (ignore nameserver-p))
-  (when (< errno 0)
-    (setq errno (- errno)))
+  (unless nameserver-p
+    (setq errno (abs errno)))
   (if stream
     (error (make-condition 'socket-error
 			   :stream stream
 			   :code errno
 			   :identifier (getf *socket-error-identifiers* errno :unknown)
 			   :situation where
-			   ;; TODO: this is a constant arg, there is a way to put this
-			   ;; in the class definition, just need to remember how...
 			   :format-control "~a (error #~d) during ~a"
 			   :format-arguments (list
-                                              #+windows-target
-                                              (%windows-error-string errno)
-                                              #-windows-target
-					      (if nameserver-p
-						(%gai-strerror errno)
-						(%strerror errno))
+					      #+windows-target
+					      (%windows-error-string errno)
+					      #-windows-target
+					      (%strerror errno)
 					      errno where)))
-    (error (make-condition 'socket-creation-error
-			   :code errno
-			   :identifier (getf *socket-error-identifiers* errno :unknown)
-			   :situation where
-			   ;; TODO: this is a constant arg, there is a way to put this
-			   ;; in the class definition, just need to remember how...
-			   :format-control "~a (error #~d) during socket creation or nameserver operation in ~a"
-			   :format-arguments (list
-                                              #+windows-target
-                                              (%windows-error-string errno)
-                                              #-windows-target
-					      (if nameserver-p
-						(%gai-strerror errno)
-						(%strerror errno))
-					      errno where)))))
-    
-
+    (let ((identifiers (if nameserver-p
+			 *gai-error-identifiers*
+			 *socket-error-identifiers*)))
+      (error (make-condition 'socket-creation-error
+			     :code errno
+			     :identifier (getf identifiers errno :unknown)
+			     :situation where
+			     :format-control "~a (error #~d) during socket creation or nameserver operation in ~a"
+			     :format-arguments (list
+						#+windows-target
+						(%windows-error-string errno)
+						#-windows-target
+						(if nameserver-p
+						  (%gai-strerror errno)
+						  (%strerror errno))
+						errno where))))))
 
 ;; If true, this will try to allow other cooperative processes to run
 ;; while socket io is happening.  Since CCL threads are preemptively
@@ -446,6 +452,7 @@ operating system resources associated with the socket."))
 
 (defmethod socket-type ((stream udp-socket)) :datagram)
 (defmethod socket-connect ((stream udp-socket)) nil)
+(defmethod socket-format ((stream udp-socket)) :binary)
 
 (defgeneric socket-os-fd (socket)
   (:documentation
@@ -942,8 +949,9 @@ accept-connection on it again."))
                                 (<= subtype x8632::max-8-bit-ivector-subtag))
             #+x8664-target (and (>= subtype x8664::min-8-bit-ivector-subtag)
                                 (<= subtype x8664::max-8-bit-ivector-subtag))
-      (report-bad-arg buf `(or (array character)
-			       (array (unsigned-byte 8))
+            #+arm-target (and (<= arm::min-8-bit-ivector-subtag subtype)
+                                (<= subtype arm::max-8-bit-ivector-subtag))
+      (report-bad-arg buf `(or (array (unsigned-byte 8))
 			       (array (signed-byte 8))))))
   (values buf offset))
 
@@ -1003,8 +1011,7 @@ a packet to arrive. Returns four values:
 	  (setq vec (make-array ret-size
 				:element-type
 				(ecase (socket-format socket)
-				  ((:text) 'base-char)
-				  ((:binary :bivalent) '(unsigned-byte 8))))
+				  ((:binary) '(unsigned-byte 8))))
 		vec-offset 0))
 	(%copy-ptr-to-ivector bufptr 0 vec vec-offset ret-size))
       (values (cond ((null buffer)
@@ -1432,41 +1439,44 @@ unsigned IP address."
 (defun %get-ip-interfaces ()
   (let* ((socket (#_socket #$AF_INET #$SOCK_DGRAM #$IPPROTO_IP)))
     (unwind-protect
-    (rlet ((realoutlen #>DWORD 0))
-      (do* ((reservedlen (* 4 (record-length #>INTERFACE_INFO))
-                         (* 2 reservedlen)))
-           ()
-        (%stack-block ((buf reservedlen))
-          (unless (eql 0 (#_WSAIoctl
-                          socket
-                          #$SIO_GET_INTERFACE_LIST
-                          (%null-ptr)
-                          0
-                          buf
-                          reservedlen
-                          realoutlen
-                          (%null-ptr)
-                          (%null-ptr)))
-            (return))
-          (let* ((noutbytes (pref realoutlen #>DWORD)))
-            (when (< noutbytes reservedlen)
-              (let* ((interfaces nil))
-                (do* ((offset 0 (+ offset (record-length #>INTERFACE_INFO)))
-                      (nameidx 0 (1+ nameidx)))
-                     ((>= offset noutbytes))
-                  (with-macptrs ((p (%inc-ptr buf offset)))
-                    (push (make-ip-interface 
-                           :name (format nil "ip~d" nameidx)
-                           :addr (ntohl
-                                  (pref (pref p #>INTERFACE_INFO.iiAddress)
-                                        #>sockaddr_gen.AddressIn.sin_addr.S_un.S_addr))
-                           :netmask (ntohl
-                                     (pref (pref p #>INTERFACE_INFO.iiNetmask)
-                                        #>sockaddr_gen.AddressIn.sin_addr.S_un.S_addr))
-                           :flags (pref p #>INTERFACE_INFO.iiFlags)
-                           :address-family #$AF_INET)
-                          interfaces)))
-                (return interfaces)))))))
+         (rlet ((realoutlen #>DWORD 0))
+           (do* ((reservedlen (* 4 (record-length #>INTERFACE_INFO))
+                              (* 2 reservedlen)))
+                ()
+             (%stack-block ((buf reservedlen))
+               (if (eql 0  (#_WSAIoctl
+                            socket
+                            #$SIO_GET_INTERFACE_LIST
+                            (%null-ptr)
+                            0
+                            buf
+                            reservedlen
+                            realoutlen
+                            (%null-ptr)
+                            (%null-ptr)))
+               (let* ((noutbytes (pref realoutlen #>DWORD)))
+                 (when (< noutbytes reservedlen)
+                   (let* ((interfaces nil))
+
+                     (do* ((offset 0 (+ offset (record-length #>INTERFACE_INFO)))
+                           (nameidx 0 (1+ nameidx)))
+                          ((>= offset noutbytes))
+                       (with-macptrs ((p (%inc-ptr buf offset)))
+                         (push (make-ip-interface 
+                                :name (format nil "ip~d" nameidx)
+                                :addr (ntohl
+                                       (pref (pref p #>INTERFACE_INFO.iiAddress)
+                                             #>sockaddr_gen.AddressIn.sin_addr.S_un.S_addr))
+                                :netmask (ntohl
+                                          (pref (pref p #>INTERFACE_INFO.iiNetmask)
+                                                #>sockaddr_gen.AddressIn.sin_addr.S_un.S_addr))
+                                :flags (pref p #>INTERFACE_INFO.iiFlags)
+                                :address-family #$AF_INET)
+                               interfaces)))
+                     (return interfaces))))
+               (let* ((err (#_WSAGetLastError)))
+                 (unless (eql err #$WSAEFAULT)
+                   (return)))))))
       (#_closesocket socket))))
 
       
@@ -1506,3 +1516,83 @@ unsigned IP address."
 	  
 (defmethod stream-io-error ((stream socket) errno where)
   (socket-error stream where errno))
+
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(defun snarf-url (url &key max-redirects (user-agent "CCL") &aux conn)
+  "GET the contents of the url as a (VECTOR (UNSIGNED-BYTE 8))"
+  (labels ((is-prefix (prefix string) (eql (length prefix) (string-lessp prefix string)))
+	   (header (prefix lines)
+	     (let ((line (find prefix lines :test #'is-prefix)))
+	       (and line (string-trim ccl::wsp (subseq line (length prefix))))))
+	   (header-value (prefix lines)
+	     (let ((line (find prefix lines :test #'is-prefix)))
+	       (and line (parse-integer line :start (length prefix)))))
+	   (split-url (string)
+             (if (is-prefix "/" string)
+               (list nil 80 string)
+               (if (not (is-prefix "http://" string))
+                 (error "Unknown scheme in ~s" string)
+                 (let* ((start (length "http://"))
+                        (end (length string))
+                        (ppos (or (position #\/ string :start start) end))
+                        (hend (or (position #\: string :start start :end ppos) ppos)))
+                   (list (subseq string start hend)
+                         (if (< hend ppos) (parse-integer string :start (1+ hend) :end ppos) 80)
+                         (if (< ppos end) (subseq string ppos) "/"))))))
+	   (read-header (conn)
+	     (loop as lines = (loop for line = (read-line conn nil)
+				    until (= 0 (length line)) ; eof or empty line
+				    collect line)
+		   as status = (let ((status-line (pop lines)))
+				 (or (parse-integer status-line
+						    :start (position #\Space status-line)
+						    :junk-allowed t)
+				     0))
+		   while (= status 100)
+		   finally (return (values lines status)))))
+    (unwind-protect
+       (loop with original-url = url
+	     with redirects = (or max-redirects 20)
+	     with (host port path) = (split-url original-url)
+	     do (setq conn (make-socket :remote-host host
+					:remote-port port
+					:external-format '(:character-encoding :us-ascii
+							   :line-termination :crlf)))
+	     do (format conn "GET ~a HTTP/1.1~%Host: ~a:~d~%Connection: close~%User-Agent: ~a~2%"
+			path host port user-agent)
+	     do (finish-output conn)
+	     do (multiple-value-bind (header-lines status) (read-header conn)
+		  (when (= status 200)
+		    (let ((encoding (header "transfer-encoding:" header-lines)))
+		      ;; Here would recognize chunked encoding if cared about that...
+		      (when (and encoding (not (string-equal encoding "identity")))
+			(error "Unsupported encoding ~s" encoding)))
+		    (return
+		      (let* ((count (header-value "content-length:" header-lines)))
+			(if count
+			    (let ((vec (make-array count :element-type '(unsigned-byte 8))))
+			      (loop for i from 0 below count
+				    do (setf (aref vec i) (read-byte conn)))
+			      vec)
+			    (let ((vec (make-array 1000
+						   :element-type '(unsigned-byte 8)
+						   :fill-pointer 0
+						   :adjustable t)))
+			      (loop for byte = (read-byte conn nil) while byte
+				    do (vector-push-extend byte vec))
+			      (subseq vec 0 (length vec)))))))
+		  (unless (and (<= 300 status 399) (<= 0 (decf redirects)))
+		    (if (<= 300 status 399)
+			(error "Too many redirects")
+			(error "Unknown response ~s" status)))
+		  (let* ((new (or (header "location:" header-lines)
+				  (error "Missing Location: header"))))
+		    (destructuring-bind (new-host new-port new-path) (split-url new)
+		      (when new-host
+			(setq host new-host port new-port))
+		      (setq path new-path))
+		    (close conn)
+		    (setq conn nil)))
+      (when conn (close conn))))))
