@@ -420,7 +420,8 @@
     (if (constant-symbol-p var)
       (let* ((old-value (%sym-global-value var)))
 	(unless (eq old-value (%unbound-marker-8))
-	  (if (similar-as-constants-p (%sym-global-value var) value)
+	  (if (or (eql old-value value)
+                  (and (not *strict-checking*) (similar-as-constants-p old-value value)))
 	    (return)
 	    ;; This should really be a cell error, allow options other than
 	    ;; redefining (such as don't redefine and continue)...
@@ -738,21 +739,6 @@ vector
 
 ;;;;;FUNCTION BINDING Functions
 
-;;; A symbol's entrypoint contains:
-;;;  1) something tagged as $t_lfun if the symbol is
-;;;     not fbound as a macro or special form;
-;;;  2) a cons, otherwise, where the cdr is a fixnum
-;;;     whose value happens to be the same bit-pattern
-;;;     as a "jsr_subprim $sp-apply-macro" instruction.
-;;;     The car of this cons is either:
-;;;     a) a function -> macro-function;
-;;;     b) a symbol: special form not redefined as a macro.
-;;;     c) a cons whose car is a function -> macro function defined
-;;;        on a special form.
-
-
-
-
 (defun symbol-function (name)
   "Return the definition of NAME, even if it is a macro or a special form.
    Error if NAME doesn't have a definition."
@@ -761,11 +747,6 @@ vector
 
 (%fhave 'fdefinition #'symbol-function)
 
-
-(defun kernel-function-p (f)
-  (declare (ignore f))
-  nil)
-
 (defun %make-function (name fn env)
   (compile-user-function fn name env))
     
@@ -830,63 +811,23 @@ vector
   (assoc str alist :test #'string-equal))
 
 (defstatic *pathname-escape-character*
-  #+windows-target #\'
+  #+windows-target #\>
   #-windows-target #\\
   "Not CL.  A Coral addition for compatibility between CL spec and the shell.")
 
+(defun caar (x) (car (car x)))
+(defun cadr (x) (car (cdr x)))
+(defun cdar (x) (cdr (car x)))
+(defun cddr (x) (cdr (cdr x)))
 
-(defun caar (x)
-  "Return the car of the 1st sublist."
- (car (car x)))
-
-(defun cadr (x)
-  "Return the 2nd object in a list."
- (car (cdr x)))
-
-(defun cdar (x)
-  "Return the cdr of the 1st sublist."
- (cdr (car x)))
-
-(defun cddr (x)
-  "Return all but the 1st two objects of a list."
-
- (cdr (cdr x)))
-
-(defun caaar (x)
-  "Return the 1st object in the caar of a list."
- (car (car (car x))))
-
-(defun caadr (x)
-  "Return the 1st object in the cadr of a list."
- (car (car (cdr x))))
-
-(defun cadar (x)
-  "Return the car of the cdar of a list."
- (car (cdr (car x))))
-
-(defun caddr (x)
-  "Return the 1st object in the cddr of a list."
- (car (cdr (cdr x))))
-
-(defun cdaar (x)
-  "Return the cdr of the caar of a list."
- (cdr (car (car x))))
-
-(defun cdadr (x)
-  "Return the cdr of the cadr of a list."
- (cdr (car (cdr x))))
-
-(defun cddar (x)
-  "Return the cdr of the cdar of a list."
- (cdr (cdr (car x))))
-
-(defun cdddr (x)
-  "Return the cdr of the cddr of a list."
- (cdr (cdr (cdr x))))
-
-(defun cadddr (x)
-  "Return the car of the cdddr of a list."
- (car (cdr (cdr (cdr x)))))
+(defun caaar (x) (car (car (car x))))
+(defun caadr (x) (car (car (cdr x))))
+(defun cadar (x) (car (cdr (car x))))
+(defun caddr (x) (car (cdr (cdr x))))
+(defun cdaar (x) (cdr (car (car x))))
+(defun cdadr (x) (cdr (car (cdr x))))
+(defun cddar (x) (cdr (cdr (car x))))
+(defun cdddr (x) (cdr (cdr (cdr x))))
 
 (%fhave 'type-of #'%type-of)
 
@@ -1093,7 +1034,14 @@ vector
   (when (null size)(setq size (if (listp dims)(apply #'* dims) dims)))
   (let ((vector (%alloc-misc size subtype)))  ; may not get here in that case
     (if initial-element-p
-      (dotimes (i (uvsize vector)) (declare (fixnum i))(uvset vector i initial-element))
+      (cond ((and (eql subtype target::subtag-simple-base-string)
+                  (eql initial-element #\Null)))
+            ((and (eql subtype target::subtag-double-float-vector)
+                  (eql initial-element 0.0d0)))
+            ((and (eql subtype target::subtag-single-float-vector)
+                  (eql initial-element 0.0s0)))
+            (t (or (eql initial-element 0)
+                   (%init-misc initial-element vector))))
       (if initial-contents-p
         (if (null dims) (uvset vector 0 initial-contents)
             (init-uvector-contents vector 0 dims initial-contents))))
