@@ -25,10 +25,7 @@
 
             
 
-(defun nbackslash-to-forward-slash (namestring)
-  (dotimes (i (length namestring) namestring)
-    (when (eql (schar namestring i) #\\)
-      (setf (schar namestring i) #\/))))
+
 
 (defconstant univeral-time-start-in-windows-seconds 9435484800)
 
@@ -58,34 +55,37 @@
       (get-encoded-cstring encoding-name pointer)
       (%get-cstring pointer))))
 
-(defun nanoseconds (n)
-  (unless (and (typep n 'fixnum)
-               (>= (the fixnum n) 0))
-    (check-type n (real 0 #.(1- (ash 1 (1- target::nbits-in-word))))))
+(defun nanoseconds (seconds)
+  (when (and (typep seconds 'fixnum)
+             (>= (the fixnum seconds) 0))
+    (return-from nanoseconds (values seconds 0)))
+  (check-type seconds (real 0 #.(1- (ash 1 (1- target::nbits-in-word)))))
   (multiple-value-bind (q r)
-      (floor n)
+      (floor seconds)
     (if (zerop r)
       (setq r 0)
       (setq r (floor (* r 1000000000))))
     (values q r)))
 
-(defun milliseconds (n)
-  (unless (and (typep n 'fixnum)
-               (>= (the fixnum n) 0))
-    (check-type n (real 0 #.(1- (ash 1 (1- target::nbits-in-word))))))
+(defun milliseconds (seconds)
+  (when (and (typep seconds 'fixnum)
+             (>= (the fixnum seconds) 0))
+    (return-from milliseconds (values seconds 0)))
+  (check-type seconds (real 0 #.(1- (ash 1 (1- target::nbits-in-word)))))
   (multiple-value-bind (q r)
-      (floor n)
+      (floor seconds)
     (if (zerop r)
       (setq r 0)
       (setq r (floor (* r 1000))))
     (values q r)))
 
-(defun microseconds (n)
-  (unless (and (typep n 'fixnum)
-               (>= (the fixnum n) 0))
-    (check-type n (real 0 #.(1- (ash 1 (1- target::nbits-in-word))))))
+(defun microseconds (seconds)
+  (when (and (typep seconds 'fixnum)
+             (>= (the fixnum seconds) 0))
+    (return-from microseconds (values seconds 0)))
+  (check-type seconds (real 0 #.(1- (ash 1 (1- target::nbits-in-word)))))
   (multiple-value-bind (q r)
-      (floor n)
+      (floor seconds)
     (if (zerop r)
       (setq r 0)
       (setq r (floor (* r 1000000))))
@@ -132,11 +132,9 @@ atomically decremented."
 (defun %timed-wait-on-semaphore-ptr (semptr duration notification)
   (or (%wait-on-semaphore-ptr semptr 0 0 notification)
       (with-process-whostate ("Semaphore timed wait")
-        (multiple-value-bind (secs millis) (milliseconds duration)
-          (let* ((now (get-internal-real-time))
-                 (stop (+ now
-                          (* secs 1000)
-                          millis)))
+        (let* ((now (get-internal-real-time))
+               (stop (+ now (floor (* duration internal-time-units-per-second)))))
+          (multiple-value-bind (secs millis) (milliseconds duration)
             (loop
               (multiple-value-bind (success err)
                   (progn
@@ -148,10 +146,10 @@ atomically decremented."
                   (return nil))
                 (unless (zerop duration)
                   (let* ((diff (- stop now)))
-                    (multiple-value-bind (remaining-seconds remaining-millis)
-                        (floor diff 1000)
+                    (multiple-value-bind (remaining-seconds remaining-itus)
+                        (floor diff internal-time-units-per-second)
                       (setq secs remaining-seconds
-                            millis remaining-millis)))))))))))
+                            millis (floor remaining-itus (/ internal-time-units-per-second 1000)))))))))))))
 
 (defun timed-wait-on-semaphore (s duration &optional notification)
   "Wait until the given semaphore has a postive count which can be
@@ -169,6 +167,48 @@ atomically decremented, or until a timeout expires."
   "Atomically increment the count of a given semaphore."
   (%signal-semaphore-ptr (semaphore-value s)))
 
+(defun %timed-wait-for-signal (signo seconds millis)
+  (let* ((status (ff-call
+                  (%kernel-import target::kernel-import-wait-for-signal)
+                  :int signo
+                  :unsigned seconds
+                  :unsigned millis
+                  :int)))
+    (values (eql status 0) status)))
+
+(defun wait-for-signal (s duration)
+  (if duration
+    (check-type duration (real 0 #x7fffffff))
+    (setq duration #x7fffffff))
+  (or (multiple-value-bind (result err)
+          (%timed-wait-for-signal s 0 0)
+        (or result
+            (if (or (eql err #$EINTR) ; probably not possible
+                    (eql err #-windows-target #$ETIMEDOUT #+windows-target #$WAIT_TIMEOUT))
+              nil
+              (error "Error waiting for signal ~d: ~a." s (%strerror err)))))
+      (with-process-whostate ("signal wait")
+        (let* ((now (get-internal-real-time))
+               (stop (+ now (floor (* duration internal-time-units-per-second)))))
+          (multiple-value-bind (secs millis) (milliseconds duration)
+            (loop
+              (multiple-value-bind (success err)
+                  (progn
+                    (%timed-wait-for-signal s secs millis))
+                (when success
+                  (return t))
+                (if (or (eql err #-windows-target #$ETIMEDOUT #+windows-target #$WAIT_TIMEOUT)
+                        (>= (setq now (get-internal-real-time)) stop))
+                  (return nil)
+                  (unless (eql err #$EINTR)
+                    (error "Error waiting for signal ~d: ~a." s (%strerror err))))
+                (unless (zerop duration)
+                  (let* ((diff (- stop now)))
+                    (multiple-value-bind (remaining-seconds remaining-itus)
+                        (floor diff internal-time-units-per-second)
+                      (setq secs remaining-seconds
+                            millis (floor remaining-itus (/ internal-time-units-per-second 1000)))))))))))))
+  
 (defun %os-getcwd (buf noctets)
   ;; Return N < 0, if error
   ;;        N < noctets: success, string is of length N (octets).
@@ -252,10 +292,10 @@ it has been changed, this is the directory Clozure CL was started in."
 
 (defun %rmdir (name)
   (let* ((last (1- (length name))))
+    (when (and (>= last 0)
+	       (eql (char name last) #\/))
+      (setq name (subseq name 0 last)))
     (with-filename-cstrs ((name name))
-      (when (and (>= last 0)
-		 (eql (%get-byte name last) (char-code #\/)))
-	(setf (%get-byte name last) 0))
       (int-errno-call (#+windows-target #__wrmdir #-windows-target #_rmdir  name)))))
 
 
@@ -824,8 +864,8 @@ environment variable. Returns NIL if there is no user with the ID uid."
               (return nil))))))))
 
 (defun %delete-file (name)
-  (with-cstrs ((n name))
-    (int-errno-call (#+windows-target #__unlink #-windows-target #_unlink n))))
+  (with-filename-cstrs ((n name))
+    (int-errno-call (#+windows-target #__wunlink #-windows-target #_unlink n))))
 
 (defun os-command (string)
   "Invoke the Posix function system(), which invokes the user's default
@@ -845,11 +885,15 @@ of the shell itself."
   (with-macptrs (p)
     (%setf-macptr p (#_strerror errno))
     (if (%null-ptr-p p)
-      (format nil "OS Error %d" errno)
+      (format nil "OS Error ~d" errno)
       (%get-cstring p))))
 
 #+windows-target
-(defun %windows-error-string (error-number)  
+(progn
+(defun get-last-windows-error ()
+  (#_GetLastError))
+
+(defun %windows-error-string (error-number)
   (rlet ((pbuffer :address (%null-ptr)))
     (if (eql 0
              (#_FormatMessageW (logior #$FORMAT_MESSAGE_ALLOCATE_BUFFER
@@ -867,6 +911,7 @@ of the shell itself."
              (q (%get-native-utf-16-cstring p)))
         (#_LocalFree p)
         q))))
+)
         
                       
 
@@ -1076,7 +1121,7 @@ any EXTERNAL-ENTRY-POINTs known to be defined by it to become unresolved."
                     (make-fd-stream read-pipe
                                     :direction :input
                                     :element-type element-type
-                                    :interactive nil
+                                    :interactive t
                                     :basic t
                                     :sharing sharing
                                     :encoding (external-format-character-encoding external-format)
@@ -1184,6 +1229,7 @@ any EXTERNAL-ENTRY-POINTs known to be defined by it to become unresolved."
                                                    :direction :input
                                                    :sharing :private
                                                    :encoding encoding
+                                                   :interactive t
                                                    :line-termination line-termination)))
                                      in-fds) out-streams)))
       (%stack-block ((in-fd-set *fd-set-size*))
@@ -1292,8 +1338,7 @@ any EXTERNAL-ENTRY-POINTs known to be defined by it to become unresolved."
                 (external-process-%exit-code proc) -1)
           (signal-semaphore (external-process-signal proc))))))
 
-  (defparameter *silently-ignore-catastrophic-failure-in-run-program*
-    #+ccl-0711 t #-ccl-0711 nil
+  (defparameter *silently-ignore-catastrophic-failure-in-run-program* nil
     "If NIL, signal an error if run-program is unable to start the program.
 If non-NIL, treat failure to start the same as failure from the program
 itself, by setting the status and exit-code fields.")
@@ -1502,7 +1547,7 @@ space, and prefixed with PREFIX."
                     (make-fd-stream (fd-uninheritable read-pipe :direction :input)
                                     :direction :input
                                     :element-type element-type
-                                    :interactive nil
+                                    :interactive t
                                     :basic t
                                     :sharing sharing
                                     :encoding (external-format-character-encoding external-format)
@@ -1779,6 +1824,7 @@ space, and prefixed with PREFIX."
                                                            :direction :input
                                                            :sharing :private
                                                            :encoding encoding
+                                                           :interactive t
                                                            :line-termination line-termination)))
                                    in-fds)
                            out-streams))
@@ -1903,7 +1949,8 @@ not, why not; and what its result code was if it completed."
       (when verbose
 	(format t "~%~%;;;~%;;; ~a requires access to ~a~%;;; Type (:y ~D) to yield control to this thread.~%;;;~%"
 		*current-process* (shared-resource-name resource)
-                (process-serial-number *current-process*)))
+                (process-serial-number *current-process*))
+        (force-output t))
       (with-lock-grabbed ((shared-resource-lock resource))
 	(append-dll-node request (shared-resource-requestors resource)))
       (wait-on-semaphore (shared-resource-request-signal request))
@@ -1999,7 +2046,7 @@ not, why not; and what its result code was if it completed."
                 1))
             #+(or linux-target solaris-target)
             (or
-             (let* ((n (#_sysconf #$_SC_NPROCESSORS_ONLN)))
+             (let* ((n (#_sysconf #$_SC_NPROCESSORS_CONF)))
                (declare (fixnum n))
                (if (> n 0) n))
              #+linux-target
@@ -2062,8 +2109,138 @@ not, why not; and what its result code was if it completed."
     (gettimeofday tv)
     (+ (pref tv :timeval.tv_sec) unix-to-universal-time)))
 
+#+windows-target
+(defloadvar *windows-allocation-granularity*
+    (rlet ((info #>SYSTEM_INFO))
+      (#_GetSystemInfo info)
+      (pref info #>SYSTEM_INFO.dwAllocationGranularity)))
+
 #-windows-target
-(progn
+(defun %memory-map-fd (fd len bits-per-element)
+  (let* ((nbytes (+ *host-page-size*
+                    (logandc2 (+ len
+                                 (1- *host-page-size*))
+                              (1- *host-page-size*))))         
+         (ndata-elements
+          (ash len
+               (ecase bits-per-element
+                 (1 3)
+                 (8 0)
+                 (16 -1)
+                 (32 -2)
+                 (64 -3))))
+         (nalignment-elements
+          (ash target::nbits-in-word
+               (ecase bits-per-element
+                 (1 0)
+                 (8 -3)
+                 (16 -4)
+                 (32 -5)
+                 (64 -6)))))
+    (if (>= (+ ndata-elements nalignment-elements)
+            array-total-size-limit)
+      (progn
+        (fd-close fd)
+        (error "Can't make a vector with ~s elements in this implementation." (+ ndata-elements nalignment-elements)))
+      (let* ((addr (#_mmap (%null-ptr)
+                           nbytes
+                           #$PROT_NONE
+                           (logior #$MAP_ANON #$MAP_PRIVATE)
+                           -1
+                           0)))              
+        (if (eql addr (%int-to-ptr (1- (ash 1 target::nbits-in-word)))) ; #$MAP_FAILED
+          (let* ((errno (%get-errno)))
+            (fd-close fd)
+            (error "Can't map ~d bytes: ~a" nbytes (%strerror errno)))
+              ;;; Remap the first page so that we can put a vector header
+              ;;; there; use the first word on the first page to remember
+              ;;; the file descriptor.
+          (progn
+            (#_mmap addr
+                    *host-page-size*
+                    (logior #$PROT_READ #$PROT_WRITE)
+                    (logior #$MAP_ANON #$MAP_PRIVATE #$MAP_FIXED)
+                    -1
+                    0)
+            (setf (pref addr :int) fd)
+            (let* ((header-addr (%inc-ptr addr (- *host-page-size*
+                                                            (* 2 target::node-size)))))
+              
+              (when (> len 0)
+                (let* ((target-addr (%inc-ptr header-addr (* 2 target::node-size))))
+                  (unless (eql target-addr
+                               (#_mmap target-addr
+                                       len
+                                       #$PROT_READ
+                                       (logior #$MAP_PRIVATE #$MAP_FIXED)
+                                       fd
+                                       0))
+                    (let* ((errno (%get-errno)))
+                      (fd-close fd)
+                      (#_munmap addr nbytes)
+                      (error "Mapping failed: ~a" (%strerror errno))))))
+              (values header-addr ndata-elements nalignment-elements))))))))
+
+#+windows-target
+(defun %memory-map-fd (fd len bits-per-element)
+  (let* ((nbytes (+ *windows-allocation-granularity*
+                    (logandc2 (+ len
+                                 (1- *windows-allocation-granularity*))
+                              (1- *windows-allocation-granularity*))))         
+         (ndata-elements
+          (ash len
+               (ecase bits-per-element
+                 (1 3)
+                 (8 0)
+                 (16 -1)
+                 (32 -2)
+                 (64 -3))))
+         (nalignment-elements
+          (ash target::nbits-in-word
+               (ecase bits-per-element
+                 (1 0)
+                 (8 -3)
+                 (16 -4)
+                 (32 -5)
+                 (64 -6)))))
+    (if (>= (+ ndata-elements nalignment-elements)
+            array-total-size-limit)
+      (progn
+        (fd-close fd)
+        (error "Can't make a vector with ~s elements in this implementation." (+ ndata-elements nalignment-elements)))
+      (let* ((mapping (#_CreateFileMappingA (%int-to-ptr fd) (%null-ptr) #$PAGE_READONLY 0 0 (%null-ptr))))
+        (if (%null-ptr-p mapping)
+          (let* ((err (#_GetLastError)))
+            (fd-close fd)
+            (error "Couldn't create a file mapping - ~a." (%windows-error-string err)))
+          (loop
+            (let* ((base (#_VirtualAlloc (%null-ptr) nbytes #$MEM_RESERVE #$PAGE_NOACCESS)))
+              (if (%null-ptr-p base)
+                (let* ((err (#_GetLastError)))
+                  (#_CloseHandle mapping)
+                  (fd-close fd)
+                  (error "Couldn't reserve ~d bytes of address space for mapped file - ~a"
+                         nbytes (%windows-error-string err)))
+                ;; Now we have to free the memory and hope that we can reallocate it ...
+                (progn
+                  (#_VirtualFree base 0 #$MEM_RELEASE)
+                  (unless (%null-ptr-p (#_VirtualAlloc base *windows-allocation-granularity* #$MEM_RESERVE #$PAGE_NOACCESS))
+                    (let* ((fptr (%inc-ptr base *windows-allocation-granularity*)))
+                      (if (%null-ptr-p (#_MapViewOfFileEx mapping #$FILE_MAP_READ 0 0 0 fptr))
+                        (#_VirtualFree base 0 #$MEM_RELEASE)
+                        (let* ((prefix-page (%inc-ptr base (- *windows-allocation-granularity*
+                                                              *host-page-size*))))
+                          (#_VirtualAlloc prefix-page *host-page-size* #$MEM_COMMIT #$PAGE_READWRITE)
+                          (setf (paref prefix-page (:* :address) 0) mapping
+                                (paref prefix-page (:* :address) 1) (%int-to-ptr fd))
+                          (return (values
+                                   (%inc-ptr prefix-page (- *host-page-size*
+                                                            (* 2 target::node-size)))
+                                   ndata-elements
+                                   nalignment-elements)))))))))))))))
+                       
+
+
 (defun map-file-to-ivector (pathname element-type)
   (let* ((upgraded-type (upgraded-array-element-type element-type))
          (upgraded-ctype (specifier-type upgraded-type)))
@@ -2078,72 +2255,12 @@ not, why not; and what its result code was if it completed."
         (let* ((len (fd-size fd)))
           (if (< len 0)
             (signal-file-error fd pathname)
-            (let* ((nbytes (+ *host-page-size*
-                              (logandc2 (+ len
-                                           (1- *host-page-size*))
-                                        (1- *host-page-size*))))
-
-                   (ndata-elements
-                    (ash len
-                         (ecase bits-per-element
-                           (1 3)
-                           (8 0)
-                           (16 -1)
-                           (32 -2)
-                           (64 -3))))
-                   (nalignment-elements
-                    (ash target::nbits-in-word
-                         (ecase bits-per-element
-                           (1 0)
-                           (8 -3)
-                           (16 -4)
-                           (32 -5)
-                           (64 -6)))))
-              (if (>= (+ ndata-elements nalignment-elements)
-                      array-total-size-limit)
-                (progn
-                  (fd-close fd)
-                  (error "Can't make a vector with ~s elements in this implementation." (+ ndata-elements nalignment-elements)))
-                (let* ((addr (#_mmap (%null-ptr)
-                                     nbytes
-                                     #$PROT_NONE
-                                     (logior #$MAP_ANON #$MAP_PRIVATE)
-                                     -1
-                                     0)))              
-                  (if (eql addr (%int-to-ptr (1- (ash 1 target::nbits-in-word)))) ; #$MAP_FAILED
-                    (let* ((errno (%get-errno)))
-                      (fd-close fd)
-                      (error "Can't map ~d bytes: ~a" nbytes (%strerror errno)))
-              ;;; Remap the first page so that we can put a vector header
-              ;;; there; use the first word on the first page to remember
-              ;;; the file descriptor.
-                    (progn
-                      (#_mmap addr
-                              *host-page-size*
-                              (logior #$PROT_READ #$PROT_WRITE)
-                              (logior #$MAP_ANON #$MAP_PRIVATE #$MAP_FIXED)
-                              -1
-                              0)
-                      (setf (pref addr :int) fd)
-                      (let* ((header-addr (%inc-ptr addr (- *host-page-size*
-                                                            (* 2 target::node-size)))))
-                        (setf (pref header-addr :unsigned-long)
-                              (logior (element-type-subtype upgraded-type)
-                                      (ash (+ ndata-elements nalignment-elements) target::num-subtag-bits)))
-                        (when (> len 0)
-                          (let* ((target-addr (%inc-ptr header-addr (* 2 target::node-size))))
-                            (unless (eql target-addr
-                                         (#_mmap target-addr
-                                                 len
-                                                 #$PROT_READ
-                                                 (logior #$MAP_PRIVATE #$MAP_FIXED)
-                                                 fd
-                                                 0))
-                              (let* ((errno (%get-errno)))
-                                (fd-close fd)
-                                (#_munmap addr nbytes)
-                                (error "Mapping failed: ~a" (%strerror errno))))))
-                        (with-macptrs ((v (%inc-ptr header-addr target::fulltag-misc)))
+            (multiple-value-bind (header-address ndata-elements nalignment-elements)
+                (%memory-map-fd fd len bits-per-element)
+              (setf (%get-natural header-address 0)
+                    (logior (element-type-subtype upgraded-type)
+                            (ash (+ ndata-elements nalignment-elements) target::num-subtag-bits)))
+              (with-macptrs ((v (%inc-ptr header-address target::fulltag-misc)))
                           (let* ((vector (rlet ((p :address v)) (%get-object p 0))))
                             ;; Tell some parts of Clozure CL - notably the
                             ;; printer - that this thing off in foreign
@@ -2155,7 +2272,7 @@ not, why not; and what its result code was if it completed."
                                         :element-type upgraded-type
                                         :displaced-to vector
                                         :adjustable t
-                                        :displaced-index-offset nalignment-elements)))))))))))))))
+                                        :displaced-index-offset nalignment-elements))))))))))
 
 (defun map-file-to-octet-vector (pathname)
   (map-file-to-ivector pathname '(unsigned-byte 8)))
@@ -2171,20 +2288,42 @@ not, why not; and what its result code was if it completed."
            (ctype (specifier-type element-type))
            (arch (backend-target-arch *target-backend*)))
       (values (%inc-ptr pv (- (* 2 target::node-size) target::fulltag-misc))
-              (- (funcall (arch::target-array-data-size-function arch)
+              (- (funcall (locally
+			      ;; Don't really care about speed, but need to turn off typechecking for bootstrapping reasons
+			      (declare (optimize (speed 3) (safety 0)))
+			    (arch::target-array-data-size-function arch))
                           (ctype-subtype ctype)
                           (length v))
                  target::node-size)))))
 
-  
+
+#-windows-target
+(defun %unmap-file (data-address size-in-octets)
+  (let* ((base-address (%inc-ptr data-address (- *host-page-size*)))
+         (fd (pref base-address :int)))
+    (#_munmap base-address (+ *host-page-size* size-in-octets))
+    (fd-close fd)))
+
+#+windows-target
+(defun %unmap-file (data-address size-in-octets)
+  (declare (ignore size-in-octets))
+  (let* ((prefix-page (%inc-ptr data-address (- *host-page-size*)))
+         (prefix-allocation (%inc-ptr data-address (- *windows-allocation-granularity*)))
+         (mapping (paref prefix-page (:* :address) 0))
+         (fd (%ptr-to-int (paref prefix-page (:* :address) 1))))
+    (#_UnmapViewOfFile data-address)
+    (#_CloseHandle mapping)
+    (#_VirtualFree prefix-allocation 0 #$MEM_RELEASE)
+    (fd-close fd)))
+
+    
+
 ;;; Argument should be something returned by MAP-FILE-TO-IVECTOR;
 ;;; this should be called at most once for any such object.
 (defun unmap-ivector (displaced-vector)
   (multiple-value-bind (data-address size-in-octets)
       (mapped-vector-data-address-and-size displaced-vector)
-  (let* ((v (array-displacement displaced-vector))
-         (base-address (%inc-ptr data-address (- *host-page-size*)))
-         (fd (pref base-address :int)))
+  (let* ((v (array-displacement displaced-vector)))
       (let* ((element-type (array-element-type displaced-vector)))
         (adjust-array displaced-vector 0
                       :element-type element-type
@@ -2192,13 +2331,14 @@ not, why not; and what its result code was if it completed."
                       :displaced-index-offset 0))
       (with-lock-grabbed (*heap-ivector-lock*)
         (setq *heap-ivectors* (delete v *heap-ivectors*)))
-      (#_munmap base-address (+ size-in-octets *host-page-size*))      
-      (fd-close fd)
+      (%unmap-file data-address size-in-octets)
       t)))
 
 (defun unmap-octet-vector (v)
   (unmap-ivector v))
 
+#-windows-target
+(progn
 (defun lock-mapped-vector (v)
   (multiple-value-bind (address nbytes)
       (mapped-vector-data-address-and-size v)
@@ -2238,6 +2378,7 @@ not, why not; and what its result code was if it completed."
     (percentage-of-resident-pages address nbytes)))
 )
 
+
 #+windows-target
 (defun cygpath (winpath)
   "Try to use the Cygwin \"cygpath\" program to map a Windows-style
