@@ -509,6 +509,8 @@
                     (return nil))))))
         ((and (hash-table-p x) (hash-table-p y))
          (%hash-table-equalp x y))
+	((and (random-state-p x) (random-state-p y))
+	 (%random-state-equalp x y))
         (t nil)))
 
 
@@ -605,6 +607,26 @@
 (defun check-deferred-call-args (w def wargs)
   (destructuring-bind (arglist spread-p) wargs
     (multiple-value-bind (deftype reason) (nx1-check-call-args def arglist spread-p)
+      (when (and (eq deftype :deferred-mismatch)
+                 (eq (car reason) :unknown-gf-keywords)
+                 (consp def)
+                 (not (logbitp $lfbits-gfn-bit (def-info.lfbits (cdr def)))))
+        ;; If didn't have a defgeneric, check against global defn
+        (let* ((global-def (fboundp (car def)))
+               (bad-keys (cadr reason)))
+          (when (typep global-def 'generic-function)
+            (setq bad-keys
+                  (multiple-value-bind (bits keyvect) (innermost-lfun-bits-keyvect global-def)
+                    (when (and bits
+                               (logbitp  $lfbits-keys-bit bits)
+                               (not (logbitp $lfbits-aok-bit bits)))
+                      (loop for key in bad-keys
+                        unless (or (find key keyvect)
+                                   (nx1-valid-gf-keyword-p global-def key))
+                        collect key)))))
+          (if bad-keys
+            (setq reason (list* :unknown-gf-keys bad-keys (cddr reason)))
+            (setq deftype nil))))
       (when deftype
         (when (eq deftype :deferred-mismatch)
           (setq deftype (if (consp def) :environment-mismatch :global-mismatch)))
@@ -737,6 +759,8 @@
 
 
 (defun type-keyword-code (type-keyword &optional target)
+  ;; Don't really care about speed, but turn off typechecking for bootstrapping reasons
+  (declare (optimize (speed 3) (safety 0)))
   (let* ((backend (if target (find-backend target) *target-backend*))
          (alist (arch::target-uvector-subtags (backend-target-arch backend)))
          (entry (assq type-keyword alist)))
