@@ -78,35 +78,12 @@
       )))
 
 #+darwinx8664-target
-;;; Apple has decided that compliance with some Unix standard or other
-;;; requires gratuitously renaming ucontext/mcontext structures and
-;;; their components.  Do you feel more compliant now ?
 (progn
-  (eval-when (:compile-toplevel :execute)
-    (def-foreign-type nil
-        (:struct :portable_mcontext64
-                 (:es :x86_exception_state64_t)
-                 (:ss :x86_thread_state64_t)
-                 (:fs :x86_float_state64_t)))
-    (def-foreign-type nil
-        (:struct :portable_uc_stack
-                 (:ss_sp (:* :void))
-                 (:ss_size (:unsigned 64))
-                 (:ss_flags  (:signed 32))))
-    (def-foreign-type nil
-        (:struct :portable_ucontext64
-                 (:onstack (:signed 32))
-                 (:sigmask (:unsigned 32))
-                 (:stack (:struct :portable_uc_stack))
-                 (:link :address)
-                 (:uc_mcsize (:unsigned 64))
-                 (:uc_mcontext64 (:* (:struct :portable_mcontext64))))))
-  (defun xp-mxcsr (xp)
-    (%get-unsigned-long
-     (pref (pref xp :portable_ucontext64.uc_mcontext64) :portable_mcontext64.fs) 32))
   (defconstant gp-regs-offset 0)
+  (defun xp-mxcsr (xp)
+     (pref xp :ucontext_t.uc_mcontext.__fs.__fpu_mxcsr))
   (defmacro xp-gp-regs (xp)
-    `(pref (pref ,xp :portable_ucontext64.uc_mcontext64) :portable_mcontext64.ss))
+    `(pref ,xp :ucontext_t.uc_mcontext.__ss))
 
   (defconstant flags-register-offset 17)
   (defconstant rip-register-offset 16)  
@@ -187,9 +164,9 @@
 (progn
   (defconstant gp-regs-offset 0)
   (defmacro xp-gp-regs (xp)
-    `(pref (pref ,xp :ucontext.uc_mcontext) :mcontext.ss))
+    `(pref ,xp :ucontext_t.uc_mcontext.__ss))
   (defun xp-mxcsr (xp)
-    (%get-unsigned-long (pref (pref xp :ucontext.uc_mcontext) :mcontext.fs) 32))
+    (pref xp :ucontext_t.uc_mcontext.__fs.__fpu_mxcsr))
   (defconstant flags-register-offset 9)
   (defconstant eip-register-offset 10)
   (defparameter *encoded-gpr-to-indexed-gpr*
@@ -340,7 +317,10 @@
       (locally (declare (optimize (speed 3) (safety 0))
                         (type (simple-array (unsigned-byte 8) (*)) containing-object))
         (aref containing-object (the fixnum (+ byte-offset delta))))
-      (%get-unsigned-byte (%int-to-ptr byte-offset) delta))))
+      ;; xcf.relative-pc is a fixnum, but it might be negative.
+      (let* ((encoded-pc (%get-ptr xcf-ptr target::xcf.relative-pc))
+	     (pc (ash (%ptr-to-int encoded-pc) (- target::fixnumshift))))
+	(%get-unsigned-byte (%int-to-ptr pc) delta)))))
 
 ;;; If the byte following a uuo (which is "skip" bytes long, set
 ;;; the xcf's relative PC to the value contained in the 32-bit
@@ -381,6 +361,8 @@
     (defconstant float-inexact-code
       (conditional-os-constant '(os::FPE_FLTRES os::EXCEPTION_FLT_INEXACT_RESULT)))))
 
+(defparameter *pending-gc-notification-hook* nil)
+
 ;;; UUOs are handled elsewhere.  This should handle all signals other than
 ;;; those generated by UUOs (and the non-UUO cases of things like SIGSEGV.)
 ;;; If the signal number is 0, other arguments (besides the exception context XP)
@@ -455,7 +437,7 @@
 			       (%map-areas #'(lambda (x)
 					       (when (eq object x)
 						 (return-from watchedp t)))
-					   area-watched area-watched)))
+					   area-watched)))
 			(let ((result nil))
 			  (with-other-threads-suspended
 			    (when (watchedp object)
@@ -489,7 +471,12 @@
                                      :address addr
                                      :write-p (not (zerop code)))
                      ()
-                     frame-ptr))))
+                     frame-ptr)))
+          ((= signal #-win32-target #$SIGTRAP #+win32-target 5)
+           (when (= code 0)
+             (let* ((hook *pending-gc-notification-hook*))
+               (declare (special *pending-gc-notification-hook*))
+               (when hook (funcall hook))))))
     skip))
 
 (defun x86-faulting-instruction (xp)
