@@ -22,13 +22,14 @@
 
 #+ppc-target (require "PPC-BACKTRACE")
 #+x86-target (require "X86-BACKTRACE")
+#+arm-target (require "ARM-BACKTRACE")
 
 
 (defparameter *backtrace-show-internal-frames* nil)
 (defparameter *backtrace-print-level* 2)
 (defparameter *backtrace-print-length* 5)
 
-(defparameter *backtrace-format* #+ccl-0711 :direct #-ccl-0711 :traditional
+(defparameter *backtrace-format* :traditional
   "If :TRADITIONAL, shows calls to non-toplevel functions using FUNCALL, and shows frame address values.
    If :DIRECT, uses a more streamlined format.")
 
@@ -41,7 +42,7 @@
                         nil       ;; condition - not used
                         frame-ptr ;; current
                         #+ppc-target *fake-stack-frames*
-                        #+x86-target frame-ptr
+                        #+(or x86-target arm-target) frame-ptr
                         (%fixnum-ref tcr target::tcr.db-link)
                         0         ;; break level - not used
                         )))
@@ -330,9 +331,9 @@ object."
       unavailable
       (let* ((addrs (cdr info))
              (i (* 3 idx))
-             (addr (svref addrs i))
-             (startpc (svref addrs (the fixnum (+ i 1))))
-             (endpc (svref addrs (the fixnum (+ i 2)))))
+             (addr (uvref addrs i))
+             (startpc (uvref addrs (the fixnum (+ i 1))))
+             (endpc (uvref addrs (the fixnum (+ i 2)))))
         (declare (fixnum i addr startpc endpc))
         (if (or (< pc startpc)
                 (>= pc endpc))
@@ -356,7 +357,7 @@ object."
         (progn (setf (uvref value 0) newval) t)
 
         (let* ((addrs (cdr (function-symbol-map lfun)))
-               (addr (svref addrs (the fixnum (* 3 idx)))))
+               (addr (uvref addrs (the fixnum (* 3 idx)))))
           (declare (fixnum  addr))
           (if (= #o77 (ldb (byte 6 0) addr))
             (raw-frame-set cfp context (ash addr (- (+ target::word-shift 6))) newval)
@@ -378,9 +379,9 @@ object."
           (declare (fixnum nname naddr))
           (when (eq (svref names nname) name)
             (let* ((value
-                    (let* ((addr (svref addrs naddr))
-                           (startpc (svref addrs (the fixnum (1+ naddr))))
-                           (endpc (svref addrs (the fixnum (+ naddr 2)))))
+                    (let* ((addr (uvref addrs naddr))
+                           (startpc (uvref addrs (the fixnum (1+ naddr))))
+                           (endpc (uvref addrs (the fixnum (+ naddr 2)))))
                       (declare (fixnum addr startpc endpc))
                       (if (or (< pc startpc)
                               (>= pc endpc))
