@@ -33,6 +33,7 @@
      *lisp-startup-functions*
      defloadvar
      defstatic
+     defstaticvar
      *break-on-warnings*
 					; misc
      record-source-file
@@ -57,7 +58,9 @@
      find-source-note-at-pc
      caller-functions
      *svn-program*
-     
+     watch
+     unwatch
+
      show-documentation
      %set-toplevel
      toplevel-loop
@@ -316,6 +319,7 @@
      unlock-file
      file-locked-p
      directoryp
+     delete-directory
 
 
      *module-search-path*
@@ -451,9 +455,13 @@
      weak-gc-method
      *trace-max-indent* 
      *trace-level* 
-     *static-cons-chunk*
      static-cons
-
+     free-static-conses
+     reserved-static-conses
+     get-gc-notification-threshold
+     set-gc-notification-threshold
+     *pending-gc-notification-hook*
+     
      population
      make-population
      population-type
@@ -544,7 +552,8 @@
 
      process-input-wait
      process-output-wait
-					; termination
+     wait-for-signal
+                                        ; termination
      terminate-when-unreachable
      terminate
      drain-termination-queue
@@ -706,8 +715,25 @@
      unmap-octet-vector
      ;; Miscellany
      heap-utilization
-
+     collect-heap-utilization
+     parse-unsigned-integer
+     parse-signed-integer
+     pui-stream
+     psi-stream
+     with-output-to-vector
+     with-input-from-vector
+     make-vector-output-stream
+     make-vector-input-stream
+     unsigned-integer-to-binary
+     signed-integer-to-binary
+     vector-input-stream
+     vector-output-stream
+     get-output-stream-vector  
+     *vector-output-stream-default-initial-allocation*   
      external-process-creation-failure
+     object-direct-size
+     add-feature
+     remove-feature
 
      ) "CCL"
    )
