@@ -1229,6 +1229,8 @@
                                               (format nil "Arglist (~(~a~))" type)
                                               "Arglist unknown")))
                                  (list arglist label (if type :colon '(:comment (:plain)))))))
+                       (list (list (ccl::lfun-bits o) "Bits" :colon))
+                       (list (list (ccl::%lfun-info o) "Plist" :colon))
                        (when doc (list (list (substitute #\space #\newline doc) "Documentation" :colon)))
                        (when sn (list (list sn "Source Location" :colon))))))
     (setf (slot-value f 'header-lines) lines)))
@@ -1277,7 +1279,7 @@
             ((< n nclosed)
              (let* ((value (ccl::nth-immediate o (1+ (- nclosed n))))
                     (map (car (ccl::function-symbol-map (ccl::closure-function o))))
-                    (label (or (and map (svref map (+ n (- (length map) nclosed))))
+                    (label (or (and map (aref map (+ n (- (length map) nclosed))))
                                n))
                     (cellp (ccl::closed-over-value-p value)))
                (when cellp
@@ -1326,7 +1328,7 @@
 (defun disassembly-line-n (f n)
   (if (< (decf n) 0)
     (values nil "Disassembly:" :comment)
-    (let ((line (svref (disasm-info f) n)))
+    (let ((line (aref (disasm-info f) n)))
       (if (consp line)
         (destructuring-bind (object label instr) line
           (values object (cons label instr) :static))
@@ -1334,7 +1336,7 @@
 
 (defun disassembly-line-n-inspector (f n)
   (unless (< (decf n) 0)
-    (let ((line (svref (disasm-info f) n)))
+    (let ((line (aref (disasm-info f) n)))
       (and (consp line)
 	   (car line)
 	   (make-inspector (car line))))))
@@ -1342,7 +1344,7 @@
 (defun disassembly-line-n-strings (f n)
   (if (< (decf n) 0)
     (values "Disassembly:" nil)
-    (let ((line (svref (disasm-info f) n)))
+    (let ((line (aref (disasm-info f) n)))
       (if (consp line)
         (destructuring-bind (object label instr) line
           (declare (ignore object))
@@ -1598,7 +1600,7 @@
             (addresses f) addresses)))
 
 (defmethod compute-frame-info ((f error-frame) n)
-  (let* ((frame (svref (addresses f) n))
+  (let* ((frame (aref (addresses f) n))
          (context (context f))
          (marker (unavailable-value-marker f)))
     
@@ -1646,7 +1648,8 @@
              :tcr tcr
              :context context
              :break-condition (ccl::bt.break-condition context))
-           :tsp-range (make-tsp-stack-range tcr context)
+           #-arm-target
+           :tsp-range #-arm-target (make-tsp-stack-range tcr context)
            :vsp-range (make-vsp-stack-range tcr context)
            :csp-range (make-csp-stack-range tcr context)
            initargs)))
@@ -1662,7 +1665,7 @@
   (frame-count (inspector-object f)))
 
 (defmethod line-n ((f stack-inspector) n)
-  (let* ((frame (svref (addresses (inspector-object f)) n)))
+  (let* ((frame (aref (addresses (inspector-object f)) n)))
     (ccl::cfp-lfun frame)))
 
 
@@ -1752,6 +1755,7 @@
 ;;; Each of these stack ranges defines the entire range of (control/value/temp)
 ;;; addresses; they can be used to addresses of stack-allocated objects
 ;;; for printing.
+#-arm-target
 (defun make-tsp-stack-range (tcr bt-info)
   (list (cons (ccl::%catch-tsp (ccl::bt.top-catch bt-info))
               (ccl::%fixnum-ref (ccl::%fixnum-ref tcr target::tcr.ts-area)
@@ -1776,6 +1780,13 @@
               (ccl::%fixnum-ref (ccl::%fixnum-ref tcr target::tcr.vs-area)
                                 target::area.high))))
 
+#+arm-target 
+(defun make-vsp-stack-range (tcr bt-info)
+  (list (cons (ccl::%fixnum-ref (ccl::catch-frame-sp (ccl::bt.top-catch bt-info))
+                                target::lisp-frame.savevsp)
+              (ccl::%fixnum-ref (ccl::%fixnum-ref tcr target::tcr.vs-area)
+                                target::area.high))))
+
 #+ppc-target
 (defun make-csp-stack-range (tcr bt-info)
   (list (cons (ccl::%svref (ccl::bt.top-catch bt-info) target::catch-frame.csp-cell)
@@ -1793,6 +1804,12 @@
   (list (cons (ccl::%svref (ccl::bt.top-catch bt-info) target::catch-frame.foreign-sp-cell)
               (ccl::%fixnum-ref (ccl::%fixnum-ref tcr target::tcr.cs-area)
                                 target::area.high))))
+
+#+arm-target
+(defun make-csp-stack-range (tcr bt-info)
+  (list (cons (ccl::catch-frame-sp (ccl::bt.top-catch bt-info))
+              (ccl::%fixnum-ref (ccl::%fixnum-ref tcr target::tcr.cs-area)
+                                target::area.high))))
 
 ;;; Inspector
 
