@@ -101,7 +101,9 @@
                         (:linuxx8632 "ccl:x86-headers;")
                         (:win32 "ccl:win32-headers;")
                         (:solarisx8632 "ccl:solarisx86-headers;")
-                        (:freebsdx8632 "ccl:freebsd-headers;"))
+                        (:freebsdx8632 "ccl:freebsd-headers;")
+                        (:linuxarm "ccl:arm-headers;")
+                        (:darwinarm "ccl:darwin-arm-headers;"))
                     :interface-package-name
                     #.(ftd-interface-package-name *target-ftd*)
                     :attributes
@@ -1077,8 +1079,10 @@ Which one name refers to depends on foreign-type-spec in the obvious manner."
            (overall-alignment 1)
            (first-field-p t)
            (attributes (ftd-attributes *target-ftd*))
-           (poweropen-alignment (getf attributes :poweropen-alignment)))
-          
+           (poweropen-alignment (getf attributes :poweropen-alignment))
+           (bits-per-word (getf attributes :bits-per-word))
+           (use-natural-alignment (getf attributes :natural-alignment)))
+        
       (dolist (field fields)
         (destructuring-bind (var type &optional bits) field
           (declare (ignore bits))
@@ -1093,7 +1097,9 @@ Which one name refers to depends on foreign-type-spec in the obvious manner."
                                     (setq first-field-p nil)
                                     natural-alignment)
                                   (min 32 natural-alignment))
-                                natural-alignment)))
+                                (if use-natural-alignment
+                                  natural-alignment
+                                  (min bits-per-word natural-alignment)))))
                  (parsed-field
                   (make-foreign-record-field :type field-type
                                              :name var)))
@@ -1349,7 +1355,9 @@ Which one name refers to depends on foreign-type-spec in the obvious manner."
 (defun %foreign-array-access-form (base-form type index-form)
   (etypecase type
     ((or foreign-pointer-type foreign-array-type)
-     (let* ((to (foreign-pointer-type-to type))
+     (let* ((to (if (foreign-array-type-p type)
+                  (foreign-array-type-element-type type)
+                  (foreign-pointer-type-to type)))
             (size (foreign-type-bits to))
             (bit-offset `(the fixnum (* ,size (the fixnum ,index-form)))))
        (invoke-foreign-type-method :extract-gen to base-form bit-offset)))))
@@ -1436,7 +1444,7 @@ result-type-specifer is :VOID or NIL"
           (format out " (#x~16,'0x) " (if (typep addr 'integer)
                                         (logand #xffffffffffffffff (ash addr 2))
                                         (%ptr-to-int addr))))
-	#+x8632-target
+	#+(or x8632-target arm-target)
 	(format out " (#x~8,'0x) " addr)
         #+x8664-target
         (format out " (#x~16,'0x) " addr)
