@@ -93,7 +93,8 @@ are running on, or NIL if we can't find any useful information."
                   (do* ((line (read-line f nil nil)
                               (read-line f nil nil))
                         (target #+ppc-target "machine"
-                                #+x86-target "model name"))
+                                #+x86-target "model name"
+                                #+arm-target "Hardware"))
                        ((null line))
                     (let* ((matched (cpu-info-match target line)))
                       (when matched (return matched)))))))
@@ -421,9 +422,11 @@ are running on, or NIL if we can't find any useful information."
   (rlet ((count #>mach_msg_type_number_t #$TASK_EVENTS_INFO_COUNT)
          (info #>task_events_info))
     (#_task_info (#_mach_task_self) #$TASK_EVENTS_INFO info count)
-    (values (pref info #>task_events_info.cow_faults)
-            (pref info #>task_events_info.faults)
-            (pref info #>task_events_info.pageins)))
+    (let* ((faults (pref info #>task_events_info.faults))
+           (pageins (pref info #>task_events_info.pageins)))
+      (values (- faults pageins)
+              pageins
+              0)))
   #+windows-target
   ;; Um, don't know how to determine this, or anything like it.
   (values 0 0 0))
@@ -526,15 +529,44 @@ are running on, or NIL if we can't find any useful information."
 
 
 
-;;; site names and machine-instance is in the init file.
 
-(defun add-feature (symbol)
-  "Not CL but should be."
-  (if (symbolp symbol)
-      (if (not (memq symbol *features*))
-          (setq *features* (cons symbol *features*)))))
+(defun add-feature (thing)
+  (when (typep thing 'symbol)
+    (let* ((gvector-or-fixnum (%symptr-binding-address '*features*)))
+      (if (typep gvector-or-fixnum 'fixnum)
+        ;; Thread-local binding of *FEATURES*.
+        (if (not (member thing *features* :test #'eq))
+          (setq *features* (cons thing *features*)))
+        (loop
+          (let* ((old (%svref gvector-or-fixnum target::symbol.vcell-cell)))
+            (when (member thing old :test #'eq)
+              (return))
+            (let* ((new (cons thing old)))
+              (when (store-gvector-conditional target::symbol.vcell-cell
+                                               gvector-or-fixnum
+                                               old
+                                               new)
+                (return)))))))
+    thing))
+
+(defun remove-feature (thing)
+  (let* ((gvector-or-fixnum (%symptr-binding-address '*features*)))
+    (if (typep gvector-or-fixnum 'fixnum)
+      ;; Thread-local binding of *FEATURES*.
+      (setq *features* (delete thing *features*))
+      (loop
+        (let* ((old (%svref gvector-or-fixnum target::symbol.vcell-cell)))
+          (unless (member thing old :test #'eq)
+            (return))
+          (let* ((new (remove thing old)))
+            (when (store-gvector-conditional target::symbol.vcell-cell
+                                           gvector-or-fixnum
+                                           old
+                                           new)
+              (return))))))
+    thing))
+  
 
-;;; (dotimes (i 5000) (declare (fixnum i)) (add-feature 'junk))
 
 
 
@@ -775,6 +807,7 @@ are running on, or NIL if we can't find any useful information."
   disassemble."
   (#+ppc-target ppc-xdisassemble
    #+x86-target x86-xdisassemble
+   #+arm-target arm-xdisassemble
    (require-type (function-for-disassembly thing) 'compiled-function)))
 
 (defun function-for-disassembly (thing)
@@ -890,33 +923,191 @@ are running on, or NIL if we can't find any useful information."
 ;;; found.  Report that information.
 
 (defun heap-utilization (&key (stream *debug-io*)
-                              (gc-first t))
+                              (gc-first t)
+                              (area nil)
+                              (unit nil)
+                              (sort :size)
+                              (classes nil)
+                              (start nil)
+                              (threshold (and classes 0.00005)))
+  "Show statistics about types of objects in the heap.
+   If :GC-FIRST is true (the default), do a full gc before scanning the heap.
+   If :START is non-nil, it should be an object returned by GET-ALLOCATION-SENTINEL, only
+     objects at higher address are scanned (i.e. roughly, only objects allocated after it).
+   :SORT can be one of :COUNT, :LOGICAL-SIZE, or :PHYSICAL-SIZE to sort by count or size.
+   :UNIT can be one of :KB :MB or :GB to show sizes in units other than bytes.
+   :AREA can be used to restrict the walk to one area or a list of areas.  Some possible
+   values are :DYNAMIC, :STATIC, :MANAGED-STATIC, :READONLY.  By default, all areas
+   (including stacks) are examined.
+   If :CLASSES is true, classifies by class rather than just typecode"
+  (let ((data (collect-heap-utilization :gc-first gc-first :start start :area area :classes classes)))
+    (report-heap-utilization data :stream stream :unit unit :sort sort :threshold threshold)))
+
+(defun collect-heap-utilization (&key (gc-first t) start area classes)
+  ;; returns list of (type-name count logical-sizes-total physical-sizes-total)
+  (when start
+    (unless (or (null area)
+                (eq (heap-area-code area) area-dynamic)
+                (and (consp area) (every (lambda (a) (eq (heap-area-code a) area-dynamic)) area)))
+      (error "~s ~s and ~s ~s are incompatible" :start start :area area))
+    (setq area area-dynamic))
+  (if classes
+    (collect-heap-utilization-by-class gc-first area start)
+    (collect-heap-utilization-by-typecode gc-first area start)))
+
+(defun collect-heap-utilization-by-typecode (gc-first area start)
   (let* ((nconses 0)
-         (nvectors (make-array 256))
-         (vector-sizes (make-array 256))
-         (vector-physical-sizes (make-array 256))
+         (counts (make-array 257))
+         (sizes (make-array 257))
+         (physical-sizes (make-array 257))
          (array-size-function (arch::target-array-data-size-function
                                (backend-target-arch *host-backend*))))
-    (declare (type (simple-vector 256) nvectors vector-sizes)
-             (dynamic-extent nvectors vector-sizes vector-physical-sizes))
-    (when gc-first (gc))
-    (%map-areas (lambda (thing)
-                  (if (listp thing)
-                    (incf nconses)
-                    (let* ((typecode (typecode thing))
-                           (logsize (funcall array-size-function typecode (uvsize thing))))
-                      (incf (aref nvectors typecode))
-                      (incf (aref vector-sizes typecode) logsize)
-                      (incf (aref vector-physical-sizes typecode)
-                            (logandc2 (+ logsize
-                                         #+64-bit-target (+ 8 15)
-                                         #+32-bit-target (+ 4 7))
-                                      #+64-bit-target 15
-                                      #+32-bit-target 7))))))
-                                         
-    (report-heap-utilization stream nconses nvectors vector-sizes vector-physical-sizes)
-    (values)))
+    (declare (type (simple-vector 257) counts sizes physical-sizes)
+             (fixnum nconses)
+             (dynamic-extent counts sizes physical-sizes))
+    (flet ((collect (thing)
+             (when (or (null start)
+                       (locally (declare (optimize (speed 3) (safety 0))) ;; lie
+                         (%i< start thing)))
+               (if (listp thing)
+                 (incf nconses)
+                 (let* ((typecode (typecode thing))
+                        (logsize (funcall array-size-function typecode (uvsize thing)))
+                        (physize (logandc2 (+ logsize
+                                              #+64-bit-target (+ 8 15)
+                                              #+32-bit-target (+ 4 7))
+                                           #+64-bit-target 15
+                                           #+32-bit-target 7)))
+                   (incf (aref counts typecode))
+                   (incf (aref sizes typecode) logsize)
+                   (incf (aref physical-sizes typecode) physize))))))
+      (declare (dynamic-extent #'collect))
+      (when gc-first (gc))
+      (%map-areas #'collect area))
+    (setf (aref counts 256) nconses)
+    (setf (aref sizes 256) (* nconses target::cons.size))
+    (setf (aref physical-sizes 256) (aref sizes 256))
+    (loop for i from 0 upto 256
+      when (plusp (aref counts i))
+      collect (list (if (eql i 256) 'cons (aref *heap-utilization-vector-type-names* i))
+                    (aref counts i)
+                    (aref sizes i)
+                    (aref physical-sizes i)))))
 
+(defun collect-heap-utilization-by-class (gc-first area start)
+  (let* ((nconses 0)
+         (max-classes (+ 100 (hash-table-count %find-classes%)))
+         (map (make-hash-table :shared nil
+                               :test 'eq
+                               :size max-classes))
+         (inst-counts (make-array max-classes :initial-element 0))
+         (slotv-counts (make-array max-classes :initial-element 0))
+         (inst-sizes (make-array max-classes :initial-element 0))
+         (slotv-sizes (make-array max-classes :initial-element 0))
+         (inst-psizes (make-array max-classes :initial-element 0))
+         (slotv-psizes (make-array max-classes :initial-element 0))
+         (overflow nil)
+         (array-size-function (arch::target-array-data-size-function
+                               (backend-target-arch *host-backend*))))
+    (declare (type simple-vector inst-counts slotv-counts inst-sizes slotv-sizes inst-psizes slotv-psizes))
+    (flet ((collect (thing)
+             (when (or (null start)
+                       (locally (declare (optimize (speed 3) (safety 0))) ;; lie
+                         (%i< start thing)))
+               (if (listp thing)
+                 (incf nconses)
+                 (unless (or (eq thing map)
+                             (eq thing (nhash.vector map))
+                             (eq thing inst-counts)
+                             (eq thing slotv-counts)
+                             (eq thing inst-sizes)
+                             (eq thing slotv-sizes)
+                             (eq thing inst-psizes)
+                             (eq thing slotv-psizes))
+                   (let* ((typecode (typecode thing))
+                          (logsize (funcall array-size-function typecode (uvsize thing)))
+                          (physize (logandc2 (+ logsize
+                                                #+64-bit-target (+ 8 15)
+                                                #+32-bit-target (+ 4 7))
+                                             #+64-bit-target 15
+                                             #+32-bit-target 7))
+                          (class (class-of (if (eql typecode target::subtag-slot-vector)
+                                             (uvref thing slot-vector.instance)
+                                             (if (eql typecode target::subtag-function)
+                                               (function-vector-to-function thing)
+                                               thing))))
+                          (index (or (gethash class map)
+                                     (let ((count (hash-table-count map)))
+                                       (if (eql count max-classes)
+                                         (setq overflow t count (1- max-classes))
+                                         (setf (gethash class map) count))))))
+                   
+                     (if (eql typecode target::subtag-slot-vector)
+                       (progn
+                         (incf (aref slotv-counts index))
+                         (incf (aref slotv-sizes index) logsize)
+                         (incf (aref slotv-psizes index) physize))
+                       (progn
+                         (incf (aref inst-counts index))
+                         (incf (aref inst-sizes index) logsize)
+                         (incf (aref inst-psizes index) physize)))))))))
+      (declare (dynamic-extent #'collect))
+      (when gc-first (gc))
+      (%map-areas #'collect area))
+    (let ((data ()))
+      (when (plusp nconses)
+        (push (list 'cons nconses (* nconses target::cons.size) (* nconses target::cons.size)) data))
+      (maphash (lambda (class index)
+                 (let* ((icount (aref inst-counts index))
+                        (scount (aref slotv-counts index))
+                        (name (if (and overflow (eql index (1- max-classes)))
+                                "All others"
+                                (or (%class-proper-name class) class))))
+                   (declare (fixnum icount) (fixnum scount))
+                   ;; When printing class names, the package matters.  report-heap-utilization
+                   ;; uses ~a, so print here.
+                   (when (plusp icount)
+                     (push (list (prin1-to-string name)
+                                 icount (aref inst-sizes index) (aref inst-psizes index)) data))
+                   (when (plusp scount)
+                     (push (list (format nil "~s slot vector" name)
+                                 scount (aref slotv-sizes index) (aref slotv-psizes index)) data))))
+               map)
+      data)))
+
+(defun collect-heap-ivector-utilization-by-typecode ()
+  (let* ((counts (make-array 256 :initial-element 0))
+	 (sizes (make-array 256 :initial-element 0))
+	 (physical-sizes (make-array 256 :initial-element 0))
+	 (array-size-function (arch::target-array-data-size-function
+                               (backend-target-arch *host-backend*)))
+	 (result ()))
+    (declare (dynamic-extent counts sizes))
+    (with-lock-grabbed (*heap-ivector-lock*)
+      (dolist (vector *heap-ivectors*)
+	(let* ((typecode (typecode vector))
+	       (logsize (funcall array-size-function typecode (uvsize vector)))
+	       (physsize (+ logsize
+			    ;; header, delta, round up
+			    #+32-bit-target (+ 4 2 7)
+			    #+64-bit-target (+ 8 2 15))))
+	  (incf (aref counts typecode))
+	  (incf (aref sizes typecode) logsize)
+	  (incf (aref physical-sizes typecode) physsize))))
+    (dotimes (i 256 result)
+      (when (plusp (aref counts i))
+	(push (list (aref *heap-utilization-vector-type-names* i)
+		    (aref counts i)
+		    (aref sizes i)
+		    (aref physical-sizes i))
+	      result)))))
+
+(defun heap-ivector-utilization (&key (stream *debug-io*)
+				      (unit nil)
+				      (sort :size))
+  (let* ((data (collect-heap-ivector-utilization-by-typecode)))
+    (report-heap-utilization data :stream stream :unit unit :sort sort)))
+  
 (defvar *heap-utilization-vector-type-names*
   (let* ((a (make-array 256)))
     #+x8664-target
@@ -949,7 +1140,7 @@ are running on, or NIL if we can't find any useful information."
                      (%svref *immheader-types* (ash i -2)))
                     ((= lowtag ppc64::lowtag-nodeheader)
                      (%svref *nodeheader-types* (ash i -2)))))))
-    #+(or ppc32-target x8632-target)
+    #+(or ppc32-target x8632-target arm-target)
     (dotimes (i 256)
       (let* ((fulltag (logand i target::fulltagmask)))
         (setf (%svref a i)
@@ -960,71 +1151,144 @@ are running on, or NIL if we can't find any useful information."
     a))
 
   
-    
-(defun report-heap-utilization (out nconses nvectors vector-sizes vector-physical-sizes)
-  (let* ((total-cons-size  (* nconses target::cons.size))
-         (total-vector-size 0)
-         (total-physical-vector-size 0)
-         (total-size 0))
-    (format out "~&Object type~42tCount~50tTotal Size in Bytes~72tTotal Size~82t % of Heap")
-    (dotimes( i (length nvectors))
-      (incf total-vector-size (aref vector-sizes i))
-      (incf total-physical-vector-size (aref vector-physical-sizes i)))
-    (setq total-size (+ total-cons-size total-physical-vector-size))
-    (unless (zerop nconses)
-      (format out "~&CONS~36t~12d~48t~16d~16d~8,2f%" nconses total-cons-size total-cons-size
-              (* 100 (/ total-cons-size total-size))))
-    (dotimes (i (length nvectors))
-      (let ((count (aref nvectors i))
-            (sizes (aref vector-sizes i))
-            (psizes (aref vector-physical-sizes i)))
-        (unless (zerop count)
-          (format out "~&~a~36t~12d~48t~16d~16d~8,2f%"
-                  (aref *heap-utilization-vector-type-names* i)
-                  count sizes psizes
-                  (* 100.0 (/ psizes total-size))))))
-    (format out "~&   Total sizes: ~49t~16d~16d" (+ total-cons-size total-vector-size) (+ total-cons-size total-physical-vector-size))))
+(defun report-heap-utilization (data &key stream unit sort threshold)
+  (check-type threshold (or null (real 0 1)))
+  (let* ((div (ecase unit
+                ((nil) 1)
+                (:kb 1024.0d0)
+                (:mb (* 1024.0d0 1024.0d0))
+                (:gb (* 1024.0d0 1024.0d0 1024.0d0))))
+         (sort-key (ecase sort
+                     (:count #'cadr)
+                     (:logical-size #'caddr)
+                     ((:physical-size :size) #'cadddr)
+                     ((:name nil) nil)))
+         (total-count 0)
+         (total-lsize 0)
+         (total-psize 0)
+         (max-name 0)
+         (others (list "All others" 0 0 0)))
 
-;; The number of words to allocate for static conses when the user requests
-;; one and we don't have any left over
-(defparameter *static-cons-chunk* 1048576)
+    (when (hash-table-p data)
+      (setq data
+            (let ((alist nil))
+              (maphash (lambda (type measures) (push (cons type measures) alist)) data)
+              alist)))
 
-(defun initialize-static-cons ()
-  "Activates collection of garbage conses in the static-conses
-   list and allocates initial static conses."
-  ; There might be a race here when multiple threads call this
-  ; function.  However, the discarded static conses will become
-  ; garbage and be added right back to the list.  No harm here
-  ; except for additional garbage collections.
-  (%set-kernel-global 'static-conses nil)
-  (allocate-static-conses))
+    (flet ((type-string (name)
+             (if (stringp name)
+               name
+               (if (symbolp name)
+                 (symbol-name name)
+                 (princ-to-string name)))))
+      (loop for (nil count lsize psize) in data
+            do (incf total-count count)
+            do (incf total-lsize lsize)
+            do (incf total-psize psize))
 
-(defun allocate-static-conses ()
-  "Allocates some memory, freezes it and lets it become garbage.
-   This will add the memory to the list of free static conses."
-  (let* ((nfullgc (full-gccount)))
-    (multiple-value-bind (head tail)
-        (%allocate-list 0 *static-cons-chunk*)
-      (if (eql (full-gccount) nfullgc)
-        (freeze)
-        (flash-freeze))
-      (%augment-static-conses head tail))))
+      (when (and data threshold)
+        (setq data (sort data #'< :key #'cadddr))
+        (loop while (< (/ (cadddr (car data)) total-psize) threshold)
+              do (destructuring-bind (type count lsize psize) (pop data)
+                   (declare (ignore type))
+                   (incf (cadr others) count)
+                   (incf (caddr others) lsize)
+                   (incf (cadddr others) psize))))
+
+      (setq data
+            (if sort-key
+              (sort data #'> :key sort-key)
+              (sort data #'string-lessp :key #'(lambda (s) (type-string (car s))))))
+
+      (when (> (cadr others) 0)
+        (setq data (nconc data (list others))))
+
+      (setq max-name (loop for (name) in data maximize (length (type-string name))))
+
+      (format stream "~&Object type~vtCount     Logical size   Physical size   % of Heap~%~vt ~a~vt ~2:*~a"
+              (+ max-name 7)
+              (+ max-name 15)
+              (ecase unit
+                ((nil) "  (in bytes)")
+                (:kb   "(in kilobytes)")
+                (:mb   "(in megabytes)")
+                (:gb   "(in gigabytes)"))
+              (+ max-name 31))
+      (loop for (type count logsize physsize) in data
+            do (if unit
+                 (format stream "~&~a~vt~11d~16,2f~16,2f~11,2f%"
+                         (type-string type)
+                         (1+ max-name)
+                         count
+                         (/ logsize div)
+                         (/ physsize div)
+                         (* 100.0 (/ physsize total-psize)))
+                 (format stream "~&~a~vt~11d~16d~16d~11,2f%"
+                         (type-string type)
+                         (1+ max-name)
+                         count
+                         logsize
+                         physsize
+                         (* 100.0 (/ physsize total-psize)))))
+      (if unit
+        (format stream "~&~a~vt~11d~16,2f~16,2f~11,2f%~%"
+                "Total"
+                (1+ max-name)
+                total-count
+                (/ total-lsize div)
+                (/ total-psize div)
+                100.0d0)
+        (format stream "~&~a~vt~11d~16d~16d~11,2f%~%"
+                "Total"
+                (1+ max-name)
+                total-count
+                total-lsize
+                total-psize
+                100.0d0))))
+  (values))
+
+(defun object-direct-size (thing)
+  "Returns the size of THING (in bytes), including any headers and
+   alignment overhead.  Does not descend an object's components."
+  (cond ((consp thing) #+64-bit-target 16 #+32-bit-target 8)
+        #+x8664-target ((symbolp thing)
+                        (object-direct-size (%symptr->symvector thing)))
+        #+x8664-target ((functionp thing)
+                        (object-direct-size (function-to-function-vector thing)))
+        ((uvectorp thing)
+         (let* ((typecode (ccl::typecode thing))
+                (element-count (ccl::uvsize thing))
+                (sizeof-content-in-octets
+                 ;; Call the architecture-specific backend function.
+                 (funcall (arch::target-array-data-size-function
+                           (backend-target-arch *host-backend*))
+                          typecode element-count)))
+           (logandc2 (+ sizeof-content-in-octets
+                           #+64-bit-target (+ 8 15)
+                           #+32-bit-target (+ 4 7))
+                     #+64-bit-target 15
+                     #+32-bit-target 7)))
+        (t 0)))
 
 (defun static-cons (car-value cdr-value)
   "Allocates a cons cell that doesn't move on garbage collection,
    and thus doesn't trigger re-hashing when used as a key in a hash
    table.  Usage is equivalent to regular CONS."
-  (when (eq (%get-kernel-global 'static-conses) 0)
-    (initialize-static-cons))
-  (let ((cell (%atomic-pop-static-cons)))
-    (if cell
-      (progn
-	(setf (car cell) car-value)
-	(setf (cdr cell) cdr-value)
-	cell)
-      (progn
-	(allocate-static-conses)
-	(static-cons car-value cdr-value)))))
+  (loop
+    (let ((cell (without-interrupts (%atomic-pop-static-cons))))
+      (if cell
+        (progn
+          (setf (car cell) car-value)
+          (setf (cdr cell) cdr-value)
+          (return cell))
+        (progn
+          (%ensure-static-conses))))))
+
+(defun free-static-conses ()
+  (%get-kernel-global free-static-conses))
+
+(defun reserved-static-conses ()
+  (%fixnum-ref-natural (%get-kernel-global static-cons-area) target::area.ndnodes))
 	
 
 (defparameter *weak-gc-method-names*
@@ -1055,7 +1319,7 @@ are running on, or NIL if we can't find any useful information."
 (defun all-watched-objects ()
   (let (result)
     (with-other-threads-suspended
-      (%map-areas #'(lambda (x) (push x result)) area-watched area-watched))
+      (%map-areas #'(lambda (x) (push x result)) area-watched))
     result))
 
 (defun primitive-watch (thing)
@@ -1083,4 +1347,131 @@ are running on, or NIL if we can't find any useful information."
 						(typecode thing))
 				   (cons nil nil))))
 			(return-from unwatch (%unwatch thing new)))))
-		area-watched area-watched)))
+                area-watched)))
+
+(defun %parse-unsigned-integer (vector start end)
+  (declare ((simple-array (unsigned-byte 8) (*)) vector)
+           (fixnum start end)
+           (optimize (speed 3) (safety 0)))
+  (let* ((count (- end start))
+         (msb 0))
+    (declare (fixnum count) ((unsigned-byte 8) msb))
+    (or
+     (do* ((i start (1+ i)))
+          ((>= i end) 0)
+       (declare (fixnum i))
+       (let* ((b (aref vector i)))
+         (declare ((unsigned-byte 8) b))
+         (cond ((zerop b) (incf start) (decf count))
+               (t (setq msb b) (return)))))
+     (cond
+       ((or (< count #+64-bit-target 8 #+32-bit-target 4)
+            (and (= count #+64-bit-target 8 #+32-bit-target 4)
+                 (< msb #+64-bit-target 16 #+32-bit-target 32)))
+        ;; Result will be a fixnum.
+        (do* ((result 0)
+              (shift 0 (+ shift 8))
+              (i (1- end) (1- i)))
+             ((< i start) result)
+          (declare (fixnum result shift i))
+          (setq result (logior result (the fixnum (%ilsl shift (aref vector i)))))))
+       (t
+        ;; Result will be a bignum.  If COUNT is a multiple of 4
+        ;; and the most significant bit is set, need to add an
+        ;; extra word of zero-extension.
+        (let* ((result (allocate-typed-vector :bignum
+                                              (if (and (logbitp 7 msb)
+                                                       (zerop (the fixnum (logand count 3))))
+                                                (the fixnum (1+ (the fixnum (ash count -2))))
+                                                (the fixnum (ash (the fixnum (+ count 3)) -2))))))
+          (declare ((simple-array (unsigned-byte 8) (*)) result)) ; lie
+          (dotimes (i count result)
+            (decf end)
+            (setf (aref result
+                        #+little-endian-target i
+                        #+big-endian-target (the fixnum (logxor i 3)))
+                  (aref vector end)))))))))
+
+  
+;;; Octets between START and END encode an unsigned integer in big-endian
+;;; byte order.
+(defun parse-unsigned-integer (vector &optional (start 0) end)
+  (setq end (check-sequence-bounds vector start end))
+  (locally (declare (fixnum start end))
+      (unless (typep vector '(simple-array (unsigned-byte 8) (*)))
+        (multiple-value-bind (data offset) (array-data-and-offset vector)
+          (declare (fixnum offset))
+          (unless (typep data '(simple-array (unsigned-byte 8) (*)))
+            (report-bad-arg vector '(vector (unsigned-byte 8))))
+          (incf start offset)
+          (incf end offset)
+          (setq vector data)))
+      (%parse-unsigned-integer vector start end)))
+
+(defun %parse-signed-integer (vector start end)
+  (declare ((simple-array (unsigned-byte 8) (*)) vector)
+           (fixnum start end)
+           (optimize (speed 3) (safety 0)))
+  (let* ((count (- end start)))
+    (declare (fixnum count))
+    (if (zerop count)
+      0
+      (let* ((sign-byte (aref vector start)))
+        (declare (fixnum sign-byte))
+        (if (< sign-byte 128)
+          (%parse-unsigned-integer vector start end)
+          (progn
+            (decf sign-byte 256)
+            (or
+             (do* ()
+                  ((= count 1) sign-byte)
+               (unless (= sign-byte -1)
+                 (return))
+               (let* ((next (1+ start))
+                      (nextb (aref vector next)))
+                 (declare (fixnum next nextb))
+                 (if (not (logbitp 7 nextb))
+                   (return))
+                 (setq sign-byte (- nextb 256)
+                       start next
+                       count (1- count))))
+             (cond ((or (< count #+64-bit-target 8 #+32-bit-target 4)
+                        (and (= count #+64-bit-target 8 #+32-bit-target 4)
+                             (>= sign-byte
+                                 #+64-bit-target -16
+                                 #+32-bit-target -32)))
+                    ;; Result will be a fixnum
+                    (do* ((result 0)
+                          (shift 0 (+ shift 8))
+                          (i (1- end) (1- i)))
+                         ((= i start) (logior result (the fixnum (%ilsl shift sign-byte))))
+                      (declare (fixnum result shift i))
+                      (setq result (logior result (the fixnum (%ilsl shift (aref vector i)))))))
+                   (t
+                    (let* ((result (allocate-typed-vector :bignum (the fixnum (ash (the fixnum (+ count 3)) -2)))))
+          (declare ((simple-array (unsigned-byte 8) (*)) result)) ; lie
+          (dotimes (i count (do* ((i count (1+ i)))
+                                 ((= 0 (the fixnum (logand i 3)))
+                                  result)
+                              (declare (fixnum i))
+                              (setf (aref result
+                                          #+little-endian-target i
+                                          #+big-endian-target (the fixnum (logxor i 3))) #xff)))
+            (decf end)
+            (setf (aref result
+                        #+little-endian-target i
+                        #+big-endian-target (the fixnum (logxor i 3)))
+                  (aref vector end)))))))))))))
+
+(defun parse-signed-integer (vector &optional (start 0) end)
+  (setq end (check-sequence-bounds vector start end))
+  (locally (declare (fixnum start end))
+    (unless (typep vector '(simple-array (unsigned-byte 8) (*)))
+      (multiple-value-bind (data offset) (array-data-and-offset vector)
+        (declare (fixnum offset))
+        (unless (typep data '(simple-array (unsigned-byte 8) (*)))
+          (report-bad-arg vector '(vector (unsigned-byte 8))))
+        (incf start offset)
+        (incf end offset)
+        (setq vector data)))
+    (%parse-signed-integer vector start end)))
