@@ -71,7 +71,7 @@
 (defvar *fasl-target-big-endian* *fasl-host-big-endian*)
 (defvar *fcomp-external-format* :default)
 
-(defvar *fasl-break-on-program-errors* #+ccl-0711 nil #-ccl-0711 :defer
+(defvar *fasl-break-on-program-errors* :defer
   "Controls what happens when the compiler detects PROGRAM-ERROR's during file compilation.
 
   If T, the compiler signals an error immediately when it detects the program-error.
@@ -123,7 +123,7 @@ Will differ from *compiling-file* during an INCLUDE")
                          (verbose *compile-verbose*)
                          (print *compile-print*)
                          load
-                         features
+                         (features nil features-p)
                          (target *fasl-target* target-p)
                          (save-local-symbols *fasl-save-local-symbols*)
                          (save-doc-strings *fasl-save-doc-strings*)
@@ -139,18 +139,28 @@ Will differ from *compiling-file* during an INCLUDE")
                                                     t  ;; really SLIME being interactive...
                                                     *fasl-break-on-program-errors*)))
   "Compile SRC, producing a corresponding fasl file and returning its filename."
-  (let* ((backend *target-backend*))
+  (let* ((backend *target-backend*)
+         (symbols ())
+         (values ()))
     (when (and target-p (not (setq backend (find-backend target))))
       (warn "Unknown :TARGET : ~S.  Reverting to ~s ..." target *fasl-target*)
       (setq target *fasl-target*  backend *target-backend*))
+    (if (eq *target-backend* *host-backend*)
+      (when features-p
+        (setq symbols '(*features*)
+              values (list (append (if (listp features) features (list features)) *features*))))
+      (setq symbols '(*features*)
+            values (list (setup-target-features *target-backend* *features*))
+            save-source-locations nil))
     (multiple-value-bind (output-file truename warnings-p serious-p)
         (loop
           (restart-case
-              (return (%compile-file src output-file verbose print features
-                                     save-local-symbols save-doc-strings save-definitions
-                                     save-source-locations break-on-program-errors
-                                     force backend external-format
-                                     compile-file-original-truename compile-file-original-buffer-offset))
+              (return (progv symbols values
+                        (%compile-file src output-file verbose print
+                                       save-local-symbols save-doc-strings save-definitions
+                                       save-source-locations break-on-program-errors
+                                       force backend external-format
+                                       compile-file-original-truename compile-file-original-buffer-offset)))
             (retry-compile-file ()
               :report (lambda (stream) (format stream "Retry compiling ~s" src))
               nil)
@@ -166,7 +176,7 @@ Will differ from *compiling-file* during an INCLUDE")
 
 (defvar *fasl-compile-time-env* nil)
 
-(defun %compile-file (src output-file verbose print features
+(defun %compile-file (src output-file verbose print
                           save-local-symbols save-doc-strings save-definitions
                           save-source-locations break-on-program-errors
                           force target-backend external-format
@@ -196,8 +206,7 @@ Will differ from *compiling-file* during an INCLUDE")
               output-file (pathname-type
                            (backend-target-fasl-pathname
                             *target-backend*))))
-    (let* ((*features* (append (if (listp features) features (list features)) (setup-target-features target-backend *features*)))
-           (*fasl-deferred-warnings* nil) ; !!! WITH-COMPILATION-UNIT ...
+    (let* ((*fasl-deferred-warnings* nil) ; !!! WITH-COMPILATION-UNIT ...
            (*fasl-save-local-symbols* save-local-symbols)
            (*save-source-locations* save-source-locations)
            (*fasl-save-doc-strings* save-doc-strings)
@@ -308,6 +317,7 @@ Will differ from *compiling-file* during an INCLUDE")
                               (signal c))))
       (funcall (compile-named-function
                 lambda
+                :compile-code-coverage nil
                 :source-notes *fcomp-source-note-map*
                 :env *fasl-compile-time-env*
                 :policy *compile-time-evaluation-policy*)))))
@@ -318,6 +328,7 @@ Will differ from *compiling-file* during an INCLUDE")
 
 ;;; Well, no usable methods by default.  How this is better than
 ;;; getting a NO-APPLICABLE-METHOD error frankly escapes me,
+;;; [Hint: this is called even when there is an applicable method]
 (defun no-make-load-form-for (object)
   (error "No ~S method is defined for ~s" 'make-load-form object))
 
@@ -497,9 +508,13 @@ Will differ from *compiling-file* during an INCLUDE")
       (when *compile-code-coverage*
 	(fcomp-compile-toplevel-forms env)
         (let* ((fns (fcomp-code-covered-functions))
-	       (v (nreverse (coerce fns 'vector))))
+	       (v (nreverse (coerce fns 'vector)))
+               (id (fcomp-file-checksum stream)))
 	  (map nil #'fcomp-digest-code-notes v)
-          (fcomp-random-toplevel-form `(register-code-covered-functions ',v) env)))
+          (fcomp-random-toplevel-form `(register-code-covered-functions ',v
+                                                                        ',*fcomp-external-format*
+                                                                        ,id)
+                                      env)))
       (while (setq form *fasl-eof-forms*)
         (setq *fasl-eof-forms* nil)
         (fcomp-form-list form env processing-mode))
@@ -507,6 +522,35 @@ Will differ from *compiling-file* during an INCLUDE")
         (fcomp-output-form $fasl-src env (namestring *compile-file-pathname*)))
       (fcomp-compile-toplevel-forms env))))
 
+(defvar *crc32-table* (let ((crc-table (make-array 256 :element-type '(unsigned-byte 32))))
+                        (loop for i from 0 below 255 as crc = i
+                              do (loop for j from 0 below 8
+                                       do (setq crc (ash crc -1))
+                                       do (when (oddp crc)
+                                            (setq crc (logxor crc  #xEDB88320))))
+                              do (setf (aref crc-table i) crc))
+                        crc-table))
+(declaim (type (simple-array (unsigned-byte 32) (256)) *crc32-table*))
+
+(defun fcomp-stream-checksum (stream)
+  ;; Could consider crc16 for 32-bit targets, but this is only used with code
+  ;; coverage so don't worry about efficiency anyway.
+  (file-position stream 0)
+  (let ((crc 0))
+    (declare (type (unsigned-byte 32) crc))
+    (loop for char base-char = (read-char stream nil) while char
+          do (setq crc (logxor
+                         (%ilogand (ash crc -8) #x00FFFFFF)
+                         (aref *crc32-table* (logand (logxor crc (char-code char)) #xFF)))))
+    (logior (ash (file-position stream) 32) crc)))
+
+(defun fcomp-file-checksum (filename &key (external-format *fcomp-external-format*))
+  (when (setq filename (probe-file filename))
+    (with-open-file (stream filename
+                            :element-type 'base-char
+                            :external-format external-format)
+      (fcomp-stream-checksum stream))))
+
 (defun fcomp-code-covered-functions ()
   (loop for op in *fcomp-output-list*
         when (consp op)
@@ -902,28 +946,36 @@ Will differ from *compiling-file* during an INCLUDE")
     (fcomp-random-toplevel-form form env)))
 
 (defun define-compile-time-structure (sd refnames predicate env)
-  (let ((defenv (definition-environment env)))
+  (let ((defenv (definition-environment env))
+        (class-name (and (not (sd-type sd)) (symbolp (sd-name sd)) (sd-name sd))))
     (when defenv
-      (when (non-nil-symbolp (sd-name sd))
-	(note-type-info (sd-name sd) 'class env)
-        (push (make-instance 'compile-time-class :name (sd-name sd))
+      (when class-name
+	(note-type-info class-name 'class env)
+        (push (make-instance 'compile-time-class :name class-name)
               (defenv.classes defenv)))
       (setf (defenv.structures defenv) (alist-adjoin (sd-name sd) sd (defenv.structures defenv)))
       (let* ((structrefs (defenv.structrefs defenv)))
-        (when (and (null (sd-type sd))
-                   predicate)
-          (setq structrefs (alist-adjoin predicate (sd-name sd) structrefs)))
+        (when (and predicate class-name)
+          (setq structrefs (alist-adjoin predicate class-name structrefs)))
         (dolist (slot (sd-slots sd))
-          (unless (fixnump (ssd-name slot))
-            (setq structrefs
-                (alist-adjoin (if refnames (pop refnames) (ssd-name slot))
-                              (ssd-type-and-refinfo slot)
-                              structrefs))))
+	  (cond
+	    ((fixnump (ssd-name slot)))	;skip
+	    ((ssd-inherited slot)
+	     (when refnames (pop refnames)))
+	    (t
+	     (setq structrefs
+		   ;; structrefs isn't a proper alist
+		   (alist-adjoin (if refnames (pop refnames) (ssd-name slot))
+				 (cons (ssd-type-and-refinfo slot) class-name)
+				 structrefs)))))
         (setf (defenv.structrefs defenv) structrefs)))))
 
 (defun fcomp-source-note (form &aux (notes *fcomp-source-note-map*))
   (and notes (gethash form notes)))
 
+(defun (setf fcomp-source-note) (note form &aux (notes *fcomp-source-note-map*))
+  (and notes (setf (gethash form notes) note)))
+
 (defun fcomp-note-source-transformation (original new)
   (let* ((*nx-source-note-map* *fcomp-source-note-map*))
     (nx-note-source-transformation original new)))
@@ -1002,19 +1054,22 @@ Will differ from *compiling-file* during an INCLUDE")
     (let* ((forms (nreverse *fcomp-toplevel-forms*))
            (*fcomp-stream-position* *fcomp-previous-position*)
 	   (*loading-toplevel-location* *fcomp-loading-toplevel-location*)
-           (lambda (if T ;; (null (cdr forms))
-                     `(lambda () ,@forms)
-                     `(lambda ()
-                        (macrolet ((load-time-value (value)
-                                     (declare (ignore value))
-                                     (compiler-function-overflow)))
-                          ,@forms)))))
+           (body (if T ;; (null (cdr forms))
+                   `(progn ,@forms)
+                   `(macrolet ((load-time-value (value)
+                                 (declare (ignore value))
+                                 (compiler-function-overflow)))
+                      ,@forms)))
+           (lambda `(lambda () ,body)))
+      ;; Don't assign a location to the lambda so it doesn't confuse acode printing, but
+      ;; arrange to assign it to any inner lambdas.
+      (setf (fcomp-source-note body) *loading-toplevel-location*)
       (setq *fcomp-toplevel-forms* nil)
       ;(format t "~& Random toplevel form: ~s" lambda)
       (handler-case (fcomp-output-form
                      $fasl-lfuncall
                      env
-                     (fcomp-named-function lambda nil env *loading-toplevel-location*))
+                     (fcomp-named-function lambda nil env #|*loading-toplevel-location*|#))
         (compiler-function-overflow ()
           (if (null (cdr forms))
             (error "Form ~s cannot be compiled - size exceeds compiler limitation"
@@ -1068,7 +1123,7 @@ Will differ from *compiling-file* during an INCLUDE")
 (defun fcomp-digest-code-notes (lfun &optional refs)
   (unless (memq lfun refs)
     (let* ((lfv (function-to-function-vector lfun))
-	   (start #+ppc-target 0 #+x86-target (%function-code-words lfun))
+	   (start #-x86-target 0 #+x86-target (%function-code-words lfun))
 	   (refs (cons lfun refs)))
       (declare (dynamic-extent refs))
       (loop for i from start below (uvsize lfv) as imm = (uvref lfv i)
@@ -1133,7 +1188,7 @@ Will differ from *compiling-file* during an INCLUDE")
 ;;;;
 ;These should be constants, but it's too much trouble when need to change 'em.
 (defparameter FASL-FILE-ID #xFF00)  ;Overall file format, shouldn't change much
-(defparameter FASL-VERSION #xFF5b)  ;Fasl block format. ($fasl-vers)
+(defparameter FASL-VERSION #xFF5e)  ;Fasl block format. ($fasl-vers)
 
 (defvar *fasdump-hash*)
 (defvar *fasdump-read-package*)
@@ -1203,7 +1258,8 @@ Will differ from *compiling-file* during an INCLUDE")
       (case type-code
         (#.target::tag-fixnum
          (fasl-scan-fixnum exp))
-        (#.target::fulltag-cons (fasl-scan-list exp))
+        (#+ppc64-target #.target::fulltag-cons
+         #-ppc64-target #.target::tag-list (fasl-scan-list exp))
         #+ppc32-target
         (#.ppc32::tag-imm)
         #+ppc64-target
@@ -1216,6 +1272,8 @@ Will differ from *compiling-file* during an INCLUDE")
         #+x8664-target
         ((#.x8664::fulltag-imm-0
           #.x8664::fulltag-imm-1))
+        #+arm-target
+        (#.arm::tag-imm)
         (t
          (if
            #+ppc32-target
@@ -1230,6 +1288,8 @@ Will differ from *compiling-file* during an INCLUDE")
                          (logior (ash 1 x8664::fulltag-immheader-0)
                                  (ash 1 x8664::fulltag-immheader-1)
                                  (ash 1 x8664::fulltag-immheader-2))))
+           #+arm-target
+           (= (the fixnum (logand type-code arm::fulltagmask)) arm::fulltag-immheader)
            (case type-code
              (#.target::subtag-dead-macptr (fasl-unknown exp))
              (#.target::subtag-macptr
@@ -1248,7 +1308,8 @@ Will differ from *compiling-file* during an INCLUDE")
              ((#.target::subtag-pool #.target::subtag-weak #.target::subtag-lock) (fasl-unknown exp))
              (#+ppc-target #.target::subtag-symbol
               #+x8632-target #.target::subtag-symbol
-              #+x8664-target #.target::tag-symbol (fasl-scan-symbol exp))
+              #+x8664-target #.target::tag-symbol
+              #+arm-target #.target::subtag-symbol (fasl-scan-symbol exp))
              ((#.target::subtag-instance #.target::subtag-struct)
               (fasl-scan-user-form exp))
              (#.target::subtag-package (fasl-scan-ref exp))
@@ -1419,8 +1480,6 @@ Will differ from *compiling-file* during an INCLUDE")
 
 (defun fasl-dump-block (gnames goffsets forms hash)
   (let ((etab-size (hash-table-count hash)))
-    (when (> etab-size 65535)
-      (error "Too many multiply-referenced objects in fasl file.~%Limit is ~d. Were ~d." 65535 etab-size))
     (fasl-out-word FASL-VERSION)          ; Word 0
     (fasl-out-long  0)
     (fasl-out-byte $fasl-vetab-alloc)
@@ -1500,10 +1559,13 @@ Will differ from *compiling-file* during an INCLUDE")
     (immediate (fasl-dump-t_imm exp))
     (double-float (fasl-dump-dfloat exp))
     (single-float (fasl-dump-sfloat exp))
-    (simple-string (let* ((n (length exp)))
-                     (fasl-out-opcode $fasl-nvstr exp)
-                     (fasl-out-count n)
-                     (fasl-out-simple-string exp 0 n)))
+    (simple-string
+     (let* ((nextra (utf-8-extra-bytes exp)))
+       (cond ((= 0 nextra)
+              (fasl-out-opcode $fasl-nvstr exp)
+              (fasl-out-nvstring exp))
+             (t (fasl-out-opcode $fasl-vstr exp)
+                (fasl-out-vstring exp nextra)))))
     (simple-bit-vector (fasl-dump-bit-vector exp))
     ((simple-array (unsigned-byte 8) (*))
      (fasl-dump-8-bit-ivector exp $fasl-u8-vector))
@@ -1662,7 +1724,16 @@ Will differ from *compiling-file* during an INCLUDE")
       (fasl-out-opcode $fasl-function f)
       (fasl-out-count n)
       (dotimes (i n)
-        (fasl-dump-form (%svref f i))))))
+        (if (= i 0)
+          (target-arch-case
+           (:arm
+	    (let ((arm-subprimitive-address
+		   (find-symbol "ARM-SUBPRIMITIVE-ADDRESS" "ARM")))
+	      (fasl-dump-form
+	       (ash (funcall arm-subprimitive-address '.SPfix-nfn-entrypoint)
+		    (- target::fixnumshift))))) ; host's fixnumshift
+           (t (fasl-dump-form (%svref f i))))
+          (fasl-dump-form (%svref f i)))))))
 
 #+x86-target
 (defun fasl-dump-function (f)
@@ -1672,17 +1743,24 @@ Will differ from *compiling-file* during an INCLUDE")
   (if (and (= (typecode f) target::subtag-xfunction)
            (= (typecode (uvref f 0)) target::subtag-u8-vector))
     (fasl-xdump-clfun f)
-    (let* ((code-size (%function-code-words f))
-           (function-vector (function-to-function-vector f))
-           (function-size (uvsize function-vector)))
-      (fasl-out-opcode $fasl-clfun f)
-      (fasl-out-count function-size)
-      (fasl-out-count code-size)
-      (fasl-out-ivect function-vector 0 (ash code-size target::word-shift))
-      (do* ((k code-size (1+ k)))
-           ((= k function-size))
-        (declare (fixnum k))
-        (fasl-dump-form (uvref function-vector k))))))
+    (if (= (typecode f) target::subtag-xfunction)
+      (let* ((n (uvsize f)))
+        (fasl-out-opcode $fasl-function f)
+        (fasl-out-count n)
+        (dotimes (i n)
+          (fasl-dump-form (%svref f i))))        
+
+      (let* ((code-size (%function-code-words f))
+             (function-vector (function-to-function-vector f))
+             (function-size (uvsize function-vector)))
+        (fasl-out-opcode $fasl-clfun f)
+        (fasl-out-count function-size)
+        (fasl-out-count code-size)
+        (fasl-out-ivect function-vector 0 (ash code-size target::word-shift))
+        (do* ((k code-size (1+ k)))
+             ((= k function-size))
+          (declare (fixnum k))
+          (fasl-dump-form (uvref function-vector k)))))))
         
 
   
@@ -1778,9 +1856,14 @@ Will differ from *compiling-file* during an INCLUDE")
 
 
 (defun fasl-dump-package (pkg)
-  (let ((name (package-name pkg)))
-    (fasl-out-opcode $fasl-nvpkg pkg)
-    (fasl-out-nvstring name)))
+  (let* ((name (package-name pkg))
+         (nextra (utf-8-extra-bytes name)))
+    (cond ((eql nextra 0)
+           (fasl-out-opcode $fasl-nvpkg pkg)
+           (fasl-out-nvstring name))
+          (t
+           (fasl-out-opcode $fasl-vpkg pkg)
+           (fasl-out-vstring name nextra)))))
 
 
 
@@ -1826,28 +1909,39 @@ Will differ from *compiling-file* during an INCLUDE")
 (defun fasl-dump-symbol (sym)
   (let* ((pkg (symbol-package sym))
          (name (symbol-name sym))
+         (nextra (utf-8-extra-bytes name))
+         (ascii (eql nextra 0))
          (idx (let* ((i (%svref (symptr->symvector (%symbol->symptr sym)) target::symbol.binding-index-cell)))
                 (declare (fixnum i))
                 (unless (zerop i) i))))
     (cond ((null pkg) 
            (progn 
-             (fasl-out-opcode (if idx $fasl-nvmksym-special $fasl-nvmksym) sym)
-             (fasl-out-nvstring name)))
+             (fasl-out-opcode (if idx
+                                (if ascii $fasl-nvmksym-special $fasl-vmksym-special)
+                                (if ascii $fasl-nvmksym $fasl-vmksym))
+                              sym)
+             (if ascii
+               (fasl-out-nvstring name)
+               (fasl-out-vstring name nextra))))
           (*fasdump-epush*
            (progn
              (fasl-out-byte (fasl-epush-op (if idx
-                                             $fasl-nvpkg-intern-special
-                                             $fasl-nvpkg-intern)))
+                                             (if ascii $fasl-nvpkg-intern-special $fasl-vpkg-intern-special)
+                                             (if ascii $fasl-nvpkg-intern $fasl-vpkg-intern))))
              (fasl-dump-form pkg)
              (fasl-dump-epush sym)
-             (fasl-out-nvstring name)))
+             (if ascii
+               (fasl-out-nvstring name)
+               (fasl-out-vstring name nextra))))
           (t
            (progn
              (fasl-out-byte (if idx
-                              $fasl-nvpkg-intern-special
-                              $fasl-nvpkg-intern))
+                              (if ascii $fasl-nvpkg-intern-special $fasl-vpkg-intern-special)
+                              (if ascii $fasl-nvpkg-intern $fasl-vpkg-intern)))
              (fasl-dump-form pkg)
-             (fasl-out-nvstring name))))))
+             (if ascii
+               (fasl-out-nvstring name)
+               (fasl-out-vstring name nextra)))))))
 
 
 (defun fasl-unknown (exp)
@@ -1858,12 +1952,59 @@ Will differ from *compiling-file* during an INCLUDE")
   (do* ((k start (1+ k)))
        ((= k end))
     (declare (fixnum k))
-    (fasl-out-count (char-code (schar str k)))))
+    (fasl-out-byte (char-code (schar str k)))))
 
 (defun fasl-out-nvstring (str)
   (fasl-out-count (length str))
   (fasl-out-simple-string str 0 (length str)))
 
+(defun utf-8-extra-bytes (string)
+  (declare (simple-string string))
+  (let* ((extra 0))
+    (declare (fixnum extra))
+    (dotimes (i (length string) extra)
+      (let* ((code (%scharcode string i)))
+        (declare ((mod #x110000) code))
+        (cond ((>= code #x10000) (incf extra 3))
+              ((>= code #x800) (incf extra 2))
+              ((>= code #x80) (incf extra 1)))))))
+
+(defun fasl-out-vstring (str nextra)
+  (declare (fixnum nextra))
+  (let* ((len (length str)))
+    (declare (fixnum len))
+    (fasl-out-count len)
+    (fasl-out-count nextra)
+    (dotimes (i len)
+      (let* ((code (%scharcode str i)))
+        (declare ((mod #x110000) code))
+        (cond ((< code #x80) (fasl-out-byte code))
+              ((< code #x800)
+               (let* ((y (ldb (byte 5 6) code))
+                      (z (ldb (byte 6 0) code)))
+                 (declare (fixnum y z))
+                 (fasl-out-byte (logior #xc0 y))
+                 (fasl-out-byte (logior #x80 z))))
+              ((< code #x10000)
+               (let* ((x (ldb (byte 4 12) code))
+                      (y (ldb (byte 6 6) code))
+                      (z (ldb (byte 6 0) code)))
+                 (declare (fixnum x y z))
+                 (fasl-out-byte (logior #xe0 x))
+                 (fasl-out-byte (logior #x80 y))
+                 (fasl-out-byte (logior #x80 z))))
+              (t
+                (let* ((w (ldb (byte 3 18) code))
+                       (x (ldb (byte 6 12) code))
+                       (y (ldb (byte 6 6) code))
+                       (z (ldb (byte 6 0) code)))
+                  (declare (fixnum w x y z))
+                  (fasl-out-byte (logior #xf0 w))
+                  (fasl-out-byte (logior #x80 x))
+                  (fasl-out-byte (logior #x80 y))
+                  (fasl-out-byte (logior #x80 z)))))))))
+
+
 (defun fasl-out-ivect (iv &optional 
                           (start 0) 
                           (nb 
