@@ -49,6 +49,7 @@
 
 (defconstant double-float-nan
   #.(let ((invalid (get-fpu-mode :invalid)))
+      (declare (notinline +))
       (unwind-protect
 	   (progn
 	     (set-fpu-mode :invalid nil)
@@ -346,22 +347,11 @@
 
 (defparameter a-short-float 1.0s0)
 
-#+32-bit-target
 (defmethod print-object ((rs random-state) stream)
-  (format stream "#.(~S ~S ~S)"         ;>> #.GAG!!!
-          'ccl::initialize-random-state
-          (random.seed-1 rs)
-          (random.seed-2 rs)))
-
-#+64-bit-target
-(defmethod print-object ((rs random-state) stream)
-  (let* ((s1 (random.seed-1 rs)))
-    (format stream "#.(~S ~S ~S)"       ;>> #.GAG!!!
-            'ccl::initialize-random-state
-            (ldb (byte 16 16) s1)
-            (ldb (byte 16 0) s1))))
-
-
+  (let* ((s1 (random.mrg31k3p-state rs)))
+    (format stream "#.(~s~{ ~s~})"       ;>> #.GAG!!!
+            'ccl::initialize-mrg31k3p-state
+	    (coerce s1 'list))))
 
 (defun float-radix (float)
   "Return (as an integer) the radix b of its floating-point argument."
