@@ -46,9 +46,14 @@
 
 ;; This used to be weak, but the keys are symbols-with-definitions, so why bother.
 ;; Set a high rehash threshold because space matters more than speed here.
+;; Do not use lock-free hash tables, because they optimize reads at the expense of
+;; writes/rehashes.  Writes/rehashes affect file-compilation speed, which matters.
 (defvar %source-files% (make-hash-table :test #'eq
-                                        :size 13000
-                                        :rehash-threshold .95))
+                                        :size 14000
+                                        :rehash-size 1.8 ;; compensate for high threshold
+                                        :rehash-threshold .95
+                                        :lock-free nil))
+
 
 
 (defvar *direct-methods-only* t
@@ -116,14 +121,9 @@ redefinition warnings.  The default method returns T.")
 (defmethod name-of ((thing definition-type))
   (definition-type-name thing))
 
-(defmacro define-definition-type (name supers &rest options)
+(defmacro define-definition-type (name supers)
   "Defines a class named name-DEFINITION-TYPE and registers it as the class of
 definition type NAME"
-  (loop with known-keys = '( ;; Backward compatibility
-                            #+ccl-0711 :default-name-function)
-        for (key . nil) in options
-        unless (memq key known-keys)
-          do (signal-program-error "Unknown option ~s" key))
   (let ((class-name (intern (%str-cat (symbol-name name) "-DEFINITION-TYPE"))))
     `(progn
        (defclass ,class-name ,(or supers '(definition-type))
@@ -418,7 +418,9 @@ definition type NAME"
       (when source
         (list (list* (cons *method-definition-type* (%method-function-method fn)) source nil)))))
   (:method ((m method))
-    (get-object-sources (method-function m))))
+    (get-object-sources (method-function m)))
+  (:method ((fn compiled-lexical-closure))
+    (get-object-sources (closure-function fn))))
 
 (defun find-definition-sources (name &optional (type t))
   "Returns a list of entries ((a-type . a-name) source . previous-sources), where
