@@ -46,16 +46,16 @@
 
           without-compiling-code-coverage))
 
-(defconstant $not-executed-style 2)
-(defconstant $totally-covered-style 5)
-(defconstant $partially-covered-style 6)
+(defconstant $no-style 0)
+(defconstant $not-executed-style 1)
+(defconstant $totally-covered-style 2)
+(defconstant $partially-covered-style 3)
 
 (defparameter *file-coverage* ())
 (defparameter *coverage-subnotes* (make-hash-table :test #'eq))
 (defparameter *emitted-code-notes* (make-hash-table :test #'eq))
 (defparameter *entry-code-notes* (make-hash-table :test #'eq))
 
-
 (defstruct (coverage-state (:conc-name "%COVERAGE-STATE-"))
   alist)
 
@@ -95,7 +95,8 @@
 
 (defun get-function-coverage (fn refs)
   (let ((entry (function-entry-code-note fn))
-	(refs (cons fn refs)))
+	(refs (cons fn refs))
+        (source (function-source-form-note fn)))
     (declare (dynamic-extent refs))
     (when entry
       (assert (eq fn (gethash entry *entry-code-notes* fn)))
@@ -106,26 +107,45 @@
        when (code-note-p imm)
        do (setf (gethash imm *emitted-code-notes*) t)
        when (and (functionp imm)
-                 (not (memq imm refs)))
+                 (not (memq imm refs))
+                 ;; Make sure this fn is in the source we're currently looking at.
+                 ;; It might not be, if it is referenced via (load-time-value (foo))
+                 ;; where (foo) returns an lfun from some different source entirely.
+                 ;; CL-PPCRE does that.
+                 (or (null source)
+                     (eq source (function-source-form-note imm))))
        nconc (get-function-coverage imm refs)))))
 
+(defun code-covered-info.file (data) (and (consp data) (car data)))
+(defun code-covered-info.fns (data) (and (consp data) (if (consp (cdr data)) (cadr data) (cdr data))))
+(defun code-covered-info.ef (data) (and (consp data) (consp (cdr data)) (caddr data)))
+(defun code-covered-info.id (data) (and (consp data) (consp (cdr data)) (cadddr data)))
+
+(defun code-covered-info-with-fns (data new-fns)
+  (assert (consp data))
+  (if (consp (cdr data))
+    (cons (car data) new-fns)
+    (let ((new (copy-list data)))
+      (setf (cadr new) new-fns)
+      new)))
+
 (defun get-coverage ()
   (setq *file-coverage* nil)
   (clrhash *coverage-subnotes*)
   (clrhash *emitted-code-notes*)
   (clrhash *entry-code-notes*)
   (loop for data in *code-covered-functions*
-	when (consp data)
-	do (destructuring-bind (file . toplevel-functions) data
-	     (push (list* file
-			  ;; Duplicates are possible if you have multiple instances of
-			  ;; (load-time-value (foo)) where (foo) returns an lfun.
-			  ;; CL-PPCRE does that.
-			  (delete-duplicates
-			   (loop for fn across toplevel-functions
-				nconc (get-function-coverage fn nil)))
-			  toplevel-functions)
-		   *file-coverage*)))
+	do (let* ((file (code-covered-info.file data))
+                  (toplevel-functions (code-covered-info.fns data)))
+             (when file
+               (let* ((all-functions (delete-duplicates
+                                      ;; Duplicates are possible if you have multiple instances of
+                                      ;; (load-time-value (foo)) where (foo) returns an lfun.
+                                      ;; CL-PPCRE does that.
+                                      (loop for fn across toplevel-functions
+                                            nconc (get-function-coverage fn nil))))
+                      (coverage (list* file all-functions toplevel-functions)))
+                 (push coverage *file-coverage*)))))
   ;; Now get subnotes, including un-emitted ones.
   (loop for note being the hash-key of *emitted-code-notes*
         do (loop for n = note then parent as parent = (code-note-parent-note n)
@@ -133,6 +153,25 @@
                  do (pushnew n (gethash parent *coverage-subnotes*))
                  until (emitted-code-note-p parent))))
 
+(defun file-coverage-acode-queue (coverage)
+  (loop with hash = (make-hash-table :test #'eq :shared nil)
+        for fn in (file-coverage-functions coverage)
+        as acode = (%function-acode-string fn)
+        as entry = (function-entry-code-note fn)
+        as sn = (entry-note-unambiguous-source entry)
+        as toplevel-sn = (function-source-form-note fn)
+        do (when sn
+             (assert toplevel-sn)
+             (let* ((pos (source-note-end-pos sn))
+                    (cell (assq acode (gethash toplevel-sn hash))))
+               (if cell
+                 (setf (cdr cell) (max (cdr cell) pos))
+                 (push (cons acode pos) (gethash toplevel-sn hash)))))
+        finally (return (sort (loop for sn being the hash-key of hash using (hash-value alist)
+                                    collect (cons (source-note-end-pos sn)
+                                                  (mapcar #'car (sort alist #'< :key #'cdr))))
+                              #'< :key #'car))))
+
 #+debug
 (defun show-notes (note)
   (when (functionp note)
@@ -144,6 +183,9 @@
 	       (format t " [Not Emitted]"))
 	     (when (entry-code-note-p note)
 	       (format t " (Entry to ~s)" (entry-code-note-p note)))
+             (when (code-note-acode-range note)
+               (multiple-value-bind (s e) (decode-file-range (code-note-acode-range note))
+                 (format t " [acode ~a:~a]" s e)))
 	     (format t "~%")
 	     (when (code-note-p note)
 	       (loop with subindent = (+ indent 3)
@@ -159,7 +201,7 @@
              alist)))
 
 (defun covered-functions-for-file (path)
-  (cdr (assoc-by-filename path *code-covered-functions*)))
+  (code-covered-info.fns (assoc-by-filename path *code-covered-functions*)))
 
 (defun clear-coverage ()
   "Clear all files from the coverage database. The files will be re-entered
@@ -176,8 +218,8 @@ image."
   "Reset all coverage data back to the `Not executed` state."
   (loop for data in *code-covered-functions*
         do (typecase data
-             (cons ;; (source-file . functions)
-		(loop for fn across (cdr data)
+             (cons
+		(loop for fn across (code-covered-info.fns data)
 		      do (reset-function-coverage fn)))
              (function (reset-function-coverage data)))))
 
@@ -194,10 +236,13 @@ image."
   (throw 'coverage-mismatch (cons why args)))
 
 (defmacro with-coverage-mismatch-catch ((saved-file) &body body)
-  `(let ((file ,saved-file)
-         (err (catch 'coverage-mismatch ,@body nil)))
-     (when err
-       (error "Mismatched coverage data for ~s, ~?" file (car err) (cdr err)))))
+  `(let ((file ,saved-file))
+     (with-simple-restart (ignore-file "Ignore ~s and continue" file)
+       (let ((err (catch 'coverage-mismatch 
+                    ,@body
+                    nil)))
+         (when err
+           (error "Mismatched coverage data for ~s, ~?" file (car err) (cdr err)))))))
 
 
 ;; (name . #(i1 i2 ...)) where in is either an index or (index . subfncoverage).
@@ -277,23 +322,31 @@ image."
   (make-coverage-state
    :alist (loop for data in *code-covered-functions*
                 when (consp data)
-                  collect (cons (car data)
-                                (map 'vector #'save-function-coverage (cdr data))))))
+                  collect (code-covered-info-with-fns
+                               data (map 'vector #'save-function-coverage (code-covered-info.fns data))))))
 
 (defun combine-coverage (coverage-states)
   (let ((result nil))
     (map nil
          (lambda (coverage-state)
-           (loop for (saved-file . saved-fns) in (coverage-state-alist coverage-state)
-                 for result-fns = (cdr (assoc-by-filename saved-file result))
+           (loop for saved-data in (coverage-state-alist coverage-state)
+                 as saved-file = (code-covered-info.file saved-data)
+                 as saved-fns = (code-covered-info.fns saved-data)
+                 as result-data = (assoc-by-filename saved-file result)
+                 as result-fns = (code-covered-info.fns result-data)
                  do (with-coverage-mismatch-catch (saved-file)
                       (cond ((null result-fns)
-                             (push (cons saved-file
-                                         (map 'vector #'copy-function-coverage saved-fns))
+                             (push (code-covered-info-with-fns
+                                    saved-data (map 'vector #'copy-function-coverage saved-fns))
                                    result))
                             ((not (eql (length result-fns) (length saved-fns)))
                              (coverage-mismatch "different function counts"))
-                            (t 
+                            (t
+                             (unless (equal (code-covered-info.id saved-data)
+                                            (code-covered-info.id result-data))
+                               (cerror "Ignore the mismatch"
+                                       "Combining different versions of file ~s (checksum mismatch)"
+                                       saved-file))
                              (loop for result-fn across result-fns
                                    for saved-fn across saved-fns
                                    do (add-function-coverage result-fn saved-fn)))))))
@@ -303,8 +356,11 @@ image."
 
 (defun restore-coverage (coverage-state)
   "Restore the code coverage data back to an earlier state produced by SAVE-COVERAGE."
-  (loop for (saved-file . saved-fns) in (coverage-state-alist coverage-state)
-        for fns = (covered-functions-for-file saved-file)
+  (loop for saved-data in (coverage-state-alist coverage-state)
+        for saved-file = (code-covered-info.file saved-data)
+        as saved-fns = (code-covered-info.fns saved-data)
+        for current-data = (assoc-by-filename saved-file *code-covered-functions*)
+        as fns = (and current-data (code-covered-info.fns current-data))
         do (with-coverage-mismatch-catch (saved-file)
              (cond ((null fns)
                     (warn "Couldn't restore saved coverage for ~s, no matching file present"
@@ -313,6 +369,11 @@ image."
                     (coverage-mismatch "had ~s functions, now have ~s"
                                        (length saved-fns) (length fns)))
                    (t 
+                    (unless (equal (code-covered-info.id saved-data)
+                                   (code-covered-info.id current-data))
+                      (cerror "Ignore the mismatch"
+                              "Restoring different version of file ~s (checksum mismatch)"
+                              saved-file))
                     (map nil #'restore-function-coverage fns saved-fns))))))
 
 (defvar *loading-coverage*)
@@ -351,7 +412,7 @@ image."
 	 (rev-dir ()))
     (loop for data in *code-covered-functions*
        when (consp data)
-       do (let ((file (probe-file (car data))))
+       do (let ((file (probe-file (code-covered-info.file data))))
 	    (when file
 	      (cond ((eq host :unknown)
 		     (setq host (pathname-host file)
@@ -389,7 +450,7 @@ image."
 	 (*coverage-subnotes* (make-hash-table :test #'eq :shared nil))
 	 (*emitted-code-notes* (make-hash-table :test #'eq :shared nil))
 	 (*entry-code-notes* (make-hash-table :test #'eq :shared nil)))
-    (get-coverage)
+    (get-coverage) 
     (loop for coverage in *file-coverage*
           as stats = (make-coverage-statistics :source-file (file-coverage-file coverage))
           do (map nil (lambda (fn)
@@ -443,21 +504,30 @@ written to the output directory.
     (get-coverage)
     (ensure-directories-exist directory)
     (loop for coverage in *file-coverage*
-      as file = (or (probe-file (file-coverage-file coverage))
+      as truename = (or (probe-file (file-coverage-file coverage))
 		    (progn (warn "Cannot find ~s, won't report coverage" (file-coverage-file coverage))
 			   nil))
-      do (when file
-           (let* ((src-name (enough-namestring file coverage-dir))
+      do (when truename
+           (let* ((src-name (enough-namestring truename coverage-dir))
                   (html-name (substitute
                               #\_ #\: (substitute
                                        #\_ #\. (substitute
-                                                #\_ #\/ (namestring-unquote src-name))))))
+                                                #\_ #\/ (namestring-unquote src-name)))))
+                  (file (file-coverage-file coverage)))
              (when html
-               (with-open-file (stream (make-pathname :name html-name :type "html" :defaults directory)
-                                       :direction :output
-                                       :if-exists :supersede
-                                       :if-does-not-exist :create)
-                 (report-file-coverage index-file coverage stream external-format)))
+               (with-coverage-mismatch-catch (file)
+                 (let* ((data (assoc-by-filename file *code-covered-functions*))
+                        (checksum (fcomp-file-checksum (code-covered-info.file data)
+                                                       :external-format (code-covered-info.ef data))))
+                   (unless (eql checksum (code-covered-info.id data))
+                     (cerror "Try coloring anyway"
+                             "File ~s has changed since coverage source location info was recorded."
+                             (code-covered-info.file data))))
+                 (with-open-file (stream (make-pathname :name html-name :type "html" :defaults directory)
+                                         :direction :output
+                                         :if-exists :supersede
+                                         :if-does-not-exist :create)
+                   (report-file-coverage index-file coverage stream external-format))))
              (push (list* src-name html-name coverage) paths))))
     (when (null paths)
       (error "No code coverage data available"))
@@ -530,14 +600,16 @@ written to the output directory.
 	(setf (code-note-code-coverage note) 'full)))))
 
 
+(defun style-for-coverage (coverage)
+  (case coverage
+    ((full) $totally-covered-style)
+    ((nil) $not-executed-style)
+    (t $partially-covered-style)))
+  
 (defun fill-with-text-style (coverage location-note styles)
-  (let ((style (case coverage
-		 ((full) $totally-covered-style)
-		 ((nil) $not-executed-style)
-		 (t $partially-covered-style))))
-    (fill styles style
-	  :start (source-note-start-pos location-note)
-	  :end (source-note-end-pos location-note))))
+  (fill styles (style-for-coverage coverage)
+        :start (source-note-start-pos location-note)
+        :end (source-note-end-pos location-note)))
 
 (defun update-text-styles (note styles)
   (let ((source (code-note-source-note note)))
@@ -599,29 +671,50 @@ written to the output directory.
 		   do (setq sn s))
 	     (return sn))))
 
-  
-(defun colorize-function (fn styles &optional refs)
+(defun colorize-acode (fn acode-styles)
+  (let* ((acode (%function-acode-string fn))
+         (note (function-entry-code-note fn))
+         (range (and note (code-note-acode-range note))))
+    (when (and acode range)
+      (let* ((cell (or (gethash acode acode-styles)
+                       (setf (gethash acode acode-styles)
+                             (let ((string (decode-string-from-octets acode :external-format :utf-8)))
+                               (cons string
+                                     (make-array (length string)
+                                                 :initial-element $no-style
+                                                 :element-type '(unsigned-byte 2)))))))
+             (styles (cdr cell)))
+        (iterate update ((note note))
+          (multiple-value-bind (start end) (decode-file-range (code-note-acode-range note))
+            (when (and start
+                       (setq start (position-if-not #'whitespacep acode :start start :end end)))
+              (fill styles (style-for-coverage (code-note-code-coverage note))
+                    :start start
+                    :end end)))
+          (loop for sub in (coverage-subnotes note)
+            unless (entry-code-note-p sub)
+            do (update sub)))))))
+
+(defun colorize-function (fn styles acode-styles &optional refs)
   (let* ((note (function-entry-code-note fn))
 	 (source (function-source-form-note fn))
 	 (refs (cons fn refs)))
     (declare (dynamic-extent refs))
     ;; Colorize the body of the function
     (when note
-      (colorize-source-note note styles))
+      (colorize-source-note note styles)
+      (colorize-acode fn acode-styles))
     ;; And now any subfunction references
     (lfunloop for imm in fn
 	      when (and (functionp imm)
 			(not (memq imm refs))
-			;; Make sure this fn is in the source we're currently looking at.
-			;; It might not be, if it is referenced via (load-time-value (foo))
-			;; where (foo) returns an lfun from some different source entirely.
-			;; CL-PPCRE does that.
+                        ;; See note in get-function-coverage
 			(or (null source)
 			    (eq source (function-source-form-note imm))
 			    #+debug (progn
 				      (warn "Ignoring ref to ~s from ~s" imm fn)
 				      nil)))
-	      do (colorize-function imm styles refs))))
+	      do (colorize-function imm styles acode-styles refs))))
 
 (defun report-file-coverage (index-file coverage html-stream external-format)
   "Print a code coverage report of FILE into the stream HTML-STREAM."
@@ -633,13 +726,15 @@ written to the output directory.
                      (read-sequence string s)
                      string)))
          (styles (make-array (length source)
-                             :initial-element 0
-                             :element-type '(unsigned-byte 2))))
-    (map nil #'(lambda (fn) (colorize-function fn styles)) (file-coverage-toplevel-functions coverage))
-    (print-file-coverage-report index-file html-stream coverage styles source)
+                             :initial-element $no-style
+                             :element-type '(unsigned-byte 2)))
+         (acode-styles (make-hash-table :test #'eq)))
+    (map nil #'(lambda (fn) (colorize-function fn styles acode-styles))
+         (file-coverage-toplevel-functions coverage))
+    (print-file-coverage-report index-file html-stream coverage styles acode-styles source)
     (format html-stream "</body></html>")))
 
-(defun print-file-coverage-report (index-file html-stream coverage styles source)
+(defun print-file-coverage-report (index-file html-stream coverage styles acode-styles source)
   (let ((*print-case* :downcase))
     (format html-stream "<h3><a href=~s>Coverage report</a>: ~a <br />~%</h3>~%"
             (native-translated-namestring (make-pathname :name (pathname-name index-file)
@@ -651,43 +746,75 @@ written to the output directory.
     (format html-stream "</table>")
 
     (format html-stream "<div class='key'><b>Key</b><br />~%")
-    (format html-stream "<div class='state-~a'>Fully covered - every single instruction executed</div>" $totally-covered-style)
-    (format html-stream "<div class='state-~a'>Partly covered - entered but some subforms not executed</div>" $partially-covered-style)
-    (format html-stream "<div class='state-~a'>Never entered - not a single instruction executed</div>" $not-executed-style)
-    (format html-stream "<p></p><div><code>~%")
+    (format html-stream "<div class='st~a'>Fully covered - every single instruction executed</div>" $totally-covered-style)
+    (format html-stream "<div class='st~a'>Partly covered - entered but some subforms not executed</div>" $partially-covered-style)
+    (format html-stream "<div class='st~a'>Never entered - not a single instruction executed</div>" $not-executed-style)
+    (format html-stream "</div><p></p>~%")
 
-    (flet ((line (line)
-             (unless (eql line 0)
-               (format html-stream "</span>"))
-             (incf line)
-             (format html-stream "</code></div></nobr>~%<nobr><div class='source'><div class='line-number'><code>~A</code></div><code>&#160;" line)
-             line))
-      (loop with line = (line 0) with col = 0
-        for last-style = nil then style
-        for char across source
-        for style across styles
-        do (unless (eq style last-style)
-             (when last-style
-               (format html-stream "</span>"))
-             (format html-stream "<span class='state-~a'>" style))
+    ;; Output source intertwined with acode
+    (iterate output ((start 0) (line 0) (queue (file-coverage-acode-queue coverage)))
+      (format html-stream "<div class='source'><code>")
+      (let ((next (car queue)))
+        (multiple-value-bind (end last-line)
+            (output-styled html-stream source styles
+                           :start start
+                           :line line
+                           :limit (car next))
+          (format html-stream "</code></div>~%")
+          (when (and next end (<= (car next) end))
+            (destructuring-bind (pos . strings) next
+              (format html-stream "<a href=javascript:swap('~d')><span class='toggle' id='p~:*~d'>Show expansion</span></a>~%~
+                                   <div class='acode' id='a~:*~d'><code>" pos)
+              (loop for acode in strings as (string . styles) = (gethash acode acode-styles)
+                    do (output-styled html-stream string styles)
+                    do (fresh-line html-stream))
+              (format html-stream "</code></div><hr/>~%")
+              (output (1+ end) last-line (cdr queue)))))))))
+
+(defun output-styled (html-stream source styles &key (start 0) line limit)
+  (let ((last-style $no-style)
+        (col 0)
+        (line line))
+    (labels ((outch (char)
+               (if (eql char #\Tab)
+                 (dotimes (i (- 8 (mod col 8)))
+                   (incf col)
+                   (write-string " " html-stream))
+                 (progn
+                   (incf col)
+                   (if (or (alphanumericp char) (find char "()+-:* ")) ;; common and safe
+                     (write-char char html-stream)
+                     (format html-stream "&#~D;" (char-code char))))))
+             (start-line ()
+               (when line
+                 (incf line)
+                 (format html-stream "<span class='line'>~A</span>" line))
+               (write-char #\space html-stream)
+               (setq col 0))
+             (set-style (new)
+               (unless (eq last-style new)
+                 (unless (eq last-style $no-style) (format html-stream "</span>"))
+                 (unless (eq new $no-style) (format html-stream "<span class='st~a'>" new))
+                 (setq last-style new)))
+             (end-line ()
+               (set-style $no-style)
+               (format html-stream "~%")))
+      (declare (inline outch start-line end-line))
+      (unless limit (setq limit (length source)))
+      (start-line)
+      (loop
+        for pos from start below (length source)
+        as char = (aref source pos) as style = (aref styles pos)
+        do (set-style style)
         do (case char
              ((#\Newline)
-              (setq style nil)
-              (setq col 0)
-              (setq line (line line)))
-             ((#\Space)
-              (incf col)
-              (write-string "&#160;" html-stream))
-             ((#\Tab)
-              (dotimes (i (- 8 (mod col 8)))
-                (incf col)
-                (write-string "&#160;" html-stream)))
+              (end-line)
+              (when (<= limit pos)
+                (return (values pos line)))
+              (start-line))
              (t
-              (incf col)
-              (if (alphanumericp char)
-                (write-char char html-stream)
-                (format html-stream "&#~D;" (char-code char))))))
-      (format html-stream "</code></div>"))))
+              (outch char)))
+        finally (end-line)))))
 
 
 (defun coverage-stats-head (html-stream stats-stream)
@@ -832,15 +959,25 @@ written to the output directory.
 
 (defun write-coverage-styles (html-stream)
   (format html-stream "<style type='text/css'>
-*.state-~a { background-color: #ffaaaa }
-*.state-~a { background-color: #aaffaa }
-*.state-~a { background-color: #44dd44 }
-div.key { margin: 20px; width: 88ex }
-div.source { width: 98ex; background-color: #eeeeee; padding-left: 5px;
+*.st~a { background-color: #ffaaaa }
+*.st~a { background-color: #aaffaa }
+*.st~a { background-color: #44dd44 }
+*.key { margin: 20px; width: 88ex }
+*.source { width: 120ex; background-color: #eeeeee; padding-left: 5px;
              /* border-style: solid none none none; border-width: 1px;
-             border-color: #dddddd */ }
+             border-color: #dddddd */
+             white-space: pre; }
 
-*.line-number { color: #666666; float: left; width: 6ex; text-align: right; margin-right: 1ex; }
+*.acode { border-left: 1px dashed #c0c0c0;
+         margin-top: 1ex;
+         margin-left: 6ex;
+         margin-bottom: 2ex;
+         white-space: pre;
+         display: none; }
+
+*.line { color: #666666; float: left; width: 6ex; text-align: right; margin-right: 1ex; }
+
+*.toggle { font-size: small; }
 
 table.summary tr.head-row { background-color: #aaaaff }
 table.summary tr td.text-cell { text-align: left }
@@ -849,8 +986,25 @@ table.summary tr td { text-align: right }
 table.summary tr.even { background-color: #eeeeff }
 table.summary tr.subheading { background-color: #aaaaff}
 table.summary tr.subheading td { text-align: left; font-weight: bold; padding-left: 5ex; }
-</style>"
+
+</style>
+
+<script type='text/javascript'>
+function swap (id) {
+  var acode = document.getElementById('a' + id);
+  var prompt = document.getElementById('p' + id);
+  if (acode.style.display == 'block') {
+      acode.style.display = 'none';
+      prompt.innerHTML = 'Show expansion';
+  } else {
+    acode.style.display = 'block';
+    prompt.innerHTML = 'Hide expansion';
+  }
+}
+</script>
+"
           $not-executed-style
           $partially-covered-style
           $totally-covered-style
           ))
+
