@@ -18,6 +18,11 @@
   (use-interface-dir :elf))
 
 
+(defloadvar *readonly-area*
+    (do-consing-areas (a)
+      (when (eql (%fixnum-ref a target::area.code)
+                 ccl::area-readonly)
+        (return a))))
 
 ;;; String tables: used both for symbol names and for section names.
 (defstruct elf-string-table
@@ -163,41 +168,58 @@
   (let* ((name (format nil "~s" f)))
     (subseq (nsubstitute #\0 #\# (nsubstitute #\. #\Space name)) 1)))
 
-#+x8664-target
-(defx86lapfunction dynamic-dnode ((x arg_z))
-  (movq (% x) (% imm0))
-  (ref-global x86::heap-start arg_y)
-  (subq (% arg_y) (% imm0))
-  (shrq ($ x8664::dnode-shift) (% imm0))
-  (box-fixnum imm0 arg_z)
-  (single-value-return))
-
-#+x8632-target
-(defx8632lapfunction dynamic-dnode ((x arg_z))
-  (movl (% x) (% imm0))
-  (ref-global x86::heap-start arg_y)
-  (subl (% arg_y) (% imm0))
-  (shrl ($ x8632::dnode-shift) (% imm0))
-  (box-fixnum imm0 arg_z)
-  (single-value-return))
 
+#+x86-target
 (defun collect-elf-static-functions ()
   (collect ((functions))
-    (freeze)
+    (purify)
     (block walk
-      (let* ((frozen-dnodes (frozen-space-dnodes)))
-        (%map-areas (lambda (o)
-                      (when (>= (dynamic-dnode o) frozen-dnodes)
-                        (return-from walk nil))
-                      (when (typep o
-                                   #+x8664-target 'function-vector
-                                   #-x8664-target 'function)
-                        (functions (function-vector-to-function o))))
-                    ccl::area-dynamic
-                    ccl::area-dynamic
-                    )))
+      (%map-areas (lambda (o)
+                    (when (typep o
+                                 #+x8664-target 'function-vector
+                                 #-x8664-target 'function)
+                      (functions (function-vector-to-function o))))
+                  :readonly
+                  ))
     (functions)))
 
+#+(or arm-target ppc-target)
+(defun collect-elf-static-functions ()
+  (ccl::purify)
+  (multiple-value-bind (pure-low pure-high)
+      (ccl::do-gc-areas (a)
+        (when (eql(ccl::%fixnum-ref a target::area.code)
+                  ccl::area-readonly)
+          (return
+            (values (ash (ccl::%fixnum-ref a target::area.low) target::fixnumshift)
+                    (ash (ccl::%fixnum-ref a target::area.active) target::fixnumshift)))))
+    (let* ((hash (make-hash-table :test #'eq))
+           (code-vector-index #+ppc-target 0 #+arm-target 1))
+      (ccl::%map-lfuns #'(lambda (f)
+                           (let* ((code-vector  (ccl:uvref f code-vector-index))
+                                  (startaddr (+ (ccl::%address-of code-vector)
+                                                target::misc-data-offset)))
+                             (when (and (>= startaddr pure-low)
+                                        (< startaddr pure-high))
+                               (push f (gethash code-vector hash))))))
+      (let* ((n 0))
+        (declare (fixnum n))
+        (maphash #'(lambda (k v)
+                     (declare (ignore k))
+                     (if (null (cdr v))
+                       (incf n)))
+                 hash)
+        (let* ((functions ()))
+          (maphash #'(lambda (k v)
+                       (declare (ignore k))
+                       (when (null (cdr v))
+                         (push (car v) functions)))
+                   hash)
+          (sort functions
+                #'(lambda (x y)
+                    (< (ccl::%address-of (uvref x code-vector-index))
+                       (ccl::%address-of (uvref y code-vector-index))))))))))
+
 (defun register-elf-functions (section-number)
   (let* ((functions (collect-elf-static-functions))
          (n (length functions))
@@ -230,10 +252,17 @@
                   #+32-bit-target :<E>lf32_<S>ym.st_shndx) section-number
             (pref p
                   #+64-bit-target :<E>lf64_<S>ym.st_value
-                  #+32-bit-target :<E>lf32_<S>ym.st_value) (%address-of f)
+                  #+32-bit-target :<E>lf32_<S>ym.st_value)
+            #+x86-target (%address-of f)
+            #+ppc-target (- (%address-of (uvref f 0)) (- ppc::fulltag-misc ppc::node-size))
+            #+arm-target (- (%address-of (uvref f 1)) (- arm::fulltag-misc arm::node-size))
             (pref p
                   #+64-bit-target :<E>lf64_<S>ym.st_size
-                  #+32-bit-target :<E>lf32_<S>ym.st_size) (1+ (ash (1- (%function-code-words f)) target::word-shift))))))
+                  #+32-bit-target :<E>lf32_<S>ym.st_size)
+            #+x86-target (1+ (ash (1- (%function-code-words f)) target::word-shift))
+            #+ppc-target (ash (uvsize (uvref f 0)) ppc::word-shift)
+            #+arm-target (ash (uvsize (uvref f 1)) arm::word-shift)
+            ))))
 
 (defun elf-section-index (section)
   (#_elf_ndxscn section))
@@ -312,9 +341,17 @@
       (fd-lseek fd pos #$SEEK_SET)
       (fd-read fd shdr (record-length #+64-bit-target :<E>lf64_<S>hdr
                                       #+32-bit-target :<E>lf32_<S>hdr))
+      ;; On 64-bit platforms, the section data precedes the image
+      ;; header; on 32-bit platforms, the image header and image
+      ;; section table precede the image data for the first (static)
+      ;; section.  With alignment, the header/section headers are
+      ;; one 4K page, and the static section size is 8K ...
       (setf (pref shdr #+64-bit-target :<E>lf64_<S>hdr.sh_offset
                   #+32-bit-target :<E>lf32_<S>hdr.sh_offset)
-            (+ #x2000 (logandc2 (+ eof 4095) 4095))) ; #x2000 for nilreg-area
+            (+ #+32-bit-target #x1000 #+64-bit-target 0  #x2000 (logandc2 (+ eof 4095) 4095))) 
+      (setf (pref shdr #+64-bit-target :<E>lf64_<S>hdr.sh_type
+                  #+32-bit-target :<E>lf32_<S>hdr.sh_type)
+            #$SHT_PROGBITS)
       (fd-lseek fd pos #$SEEK_SET)
       (fd-write fd shdr (record-length #+64-bit-target :<E>lf64_<S>hdr
                                        #+32-bit-target :<E>lf32_<S>hdr))
@@ -330,6 +367,7 @@
                                            #+x8632-target #$EM_386
                                            #+ppc32-target #$EM_PPC
                                            #+ppc64-target #$EM_PPC64
+                                           #+arm-target #$EM_ARM
                                            ))
          (program-header (new-elf-program-header object))
          (lisp-section (new-elf-section object))
@@ -353,9 +391,9 @@
           (pref lisp-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_flags
                 #+32-bit-target :<E>lf32_<S>hdr.sh_flags) (logior #$SHF_WRITE #$SHF_ALLOC #$SHF_EXECINSTR)
           (pref lisp-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_addr
-                #+32-bit-target :<E>lf32_<S>hdr.sh_addr) (ash (%get-kernel-global heap-start) target::fixnumshift)
+                #+32-bit-target :<E>lf32_<S>hdr.sh_addr) (ash (%fixnum-ref *readonly-area* target::area.low) target::fixnumshift)
           (pref lisp-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_size
-                #+32-bit-target :<E>lf32_<S>hdr.sh_size) (ash (frozen-space-dnodes) target::dnode-shift)
+                #+32-bit-target :<E>lf32_<S>hdr.sh_size) (ash (- (%fixnum-ref *readonly-area* target::area.active) (%fixnum-ref *readonly-area* target::area.low) )target::fixnumshift)
           (pref lisp-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_offset
                 #+32-bit-target :<E>lf32_<S>hdr.sh_offset) 0
           (pref lisp-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_addralign
@@ -381,7 +419,7 @@
                 #+32-bit-target :<E>lf32_<S>hdr.sh_type) #$SHT_STRTAB
           (pref shstrtab-section-header #+64-bit-target :<E>lf64_<S>hdr.sh_flags
                 #+32-bit-target :<E>lf32_<S>hdr.sh_flags) (logior #$SHF_STRINGS #$SHF_ALLOC))
-    (elf-make-empty-data-for-section object lisp-section (ash (frozen-space-dnodes) target::dnode-shift))
+    (elf-make-empty-data-for-section object lisp-section (ash (- (%fixnum-ref *readonly-area* target::area.active) (%fixnum-ref *readonly-area* target::area.low) )target::fixnumshift))
     (elf-init-section-data-from-string-table object strings-section (elf-symbol-table-strings symbols))
     (elf-init-section-data-from-string-table object shstrtab-section section-names)
     (elf-init-symbol-section-from-symbol-table object symbols-section symbols)
