@@ -823,8 +823,7 @@
 
 (def-accessors (random-state) %svref
   ()
-  random.seed-1
-  random.seed-2)
+  random.mrg31k3p-state)
 
 ;;; IEEE-floating-point constants.
 
@@ -1285,6 +1284,7 @@
   tstack                                ; (dynamic-extent) temp stack
   readonly                              ; readonly section
   watched				; static area containing a single object
+  static-cons                           ; static cons cells
   managed-static                        ; growable static area
   static                                ; static data in application
   dynamic                               ; dynmaic (heap) data in application
