@@ -20,7 +20,7 @@
 #include "bits.h"
 #include "gc.h"
 #include "area.h"
-#include "Threads.h"
+#include "threads.h"
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
@@ -97,6 +97,23 @@ LispObj GCephemeral_low = 0;
 natural GCn_ephemeral_dnodes = 0;
 natural GCstack_limit = 0;
 
+void
+check_static_cons_freelist(char *phase)
+{
+  LispObj 
+    n,
+    base = (LispObj)static_cons_area->low, 
+    limit = static_cons_area->ndnodes;
+  natural i=0;
+
+  for (n=lisp_global(STATIC_CONSES);n!=lisp_nil;n=((cons *)untag(n))->cdr, i++) {
+    if ((fulltag_of(n) != fulltag_cons) ||
+        (area_dnode(n,base) >= limit)) {
+      Bug(NULL, "%s: static cons freelist has invalid element 0x" LISP "\n",
+          phase, i);
+    }
+  }
+}
 
 void
 reapweakv(LispObj weakv)
@@ -195,10 +212,13 @@ reapweakv(LispObj weakv)
 
   if (terminatablep) {
     deref(weakv,1+3) = termination_list;
-    if (termination_list != lisp_nil) {
-      deref(weakv,1) = GCweakvll;
-      GCweakvll = weakv;
-    }
+  }
+  if (termination_list != lisp_nil) {
+    deref(weakv,1) = GCweakvll;
+    GCweakvll = untag(weakv);
+  } else {
+    deref(weakv,1) = lisp_global(WEAKVLL);
+    lisp_global(WEAKVLL) = untag(weakv);
   }
 }
 
@@ -212,23 +232,51 @@ reaphashv(LispObj hashv)
   hash_table_vector_header
     *hashp = (hash_table_vector_header *) ptr_from_lispobj(untag(hashv));
   natural
-    dnode,
+    dnode;
+  signed_natural
     npairs = (header_element_count(hashp->header) - 
               (hash_table_vector_header_count -1)) >> 1;
   LispObj *pairp = (LispObj*) (hashp+1), weakelement;
-  Boolean 
-    weak_on_value = ((hashp->flags & nhash_weak_value_mask) != 0);
+  int weak_index = (((hashp->flags & nhash_weak_value_mask) == 0) ? 0 : 1);
   Boolean
     keys_frozen = ((hashp->flags & nhash_keys_frozen_mask) != 0);
   bitvector markbits = GCmarkbits;
   int tag;
 
-  while (npairs--) {
-    if (weak_on_value) {
-      weakelement = pairp[1];
-    } else {
-      weakelement = pairp[0];
+  natural *tenured_low = (LispObj *)tenured_area->low;
+  natural tenured_dnodes = area_dnode(GCarealow, tenured_low);
+  natural memo_dnode = area_dnode(ptr_to_lispobj(pairp+weak_index), tenured_low);
+  Boolean
+    hashv_tenured = (memo_dnode < tenured_dnodes);
+  natural bits, bitidx, *bitsp;
+
+  if (hashv_tenured) {
+    set_bitidx_vars(tenured_area->refbits, memo_dnode, bitsp, bits, bitidx);
+  }
+
+  while (true) {
+    if (hashv_tenured) {
+      while (bits == 0) {
+        int skip = nbits_in_word - bitidx;
+        npairs -= skip;
+        if (npairs <= 0) break;
+        pairp += (skip+skip);
+        bitidx = 0;
+        bits = *++bitsp;
+      }
+      if (bits != 0) {
+        int skip = (count_leading_zeros(bits) - bitidx);
+        if (skip != 0) {
+          npairs -= skip;
+          pairp += (skip+skip);
+          bitidx += skip;
+        }
+      }
     }
+
+    if (npairs <= 0) break;
+
+    weakelement = pairp[weak_index];
     tag = fulltag_of(weakelement);
     if (is_node_fulltag(tag)) {
       dnode = gc_area_dnode(weakelement);
@@ -247,7 +295,10 @@ reaphashv(LispObj hashv)
       }
     }
     pairp += 2;
+    --npairs;
   }
+  deref(hashv, 1) = lisp_global(WEAKVLL);
+  lisp_global(WEAKVLL) = untag(hashv);
 }
 
 void
@@ -255,18 +306,16 @@ traditional_dws_mark_htabv(LispObj htabv)
 {
   /* Do nothing, just add htabv to GCweakvll */
   LispObj *base = (LispObj *) ptr_from_lispobj(untag(htabv));
-  
-  deref(base,1) = GCweakvll;
-  GCweakvll = htabv;
+
+  base[1] = GCweakvll;
+  GCweakvll = ptr_to_lispobj(base);
 }
 
 void
 ncircle_dws_mark_htabv(LispObj htabv)
 {
   /* Do nothing, just add htabv to GCdwsweakvll */
-  LispObj *base = (LispObj *) ptr_from_lispobj(untag(htabv));
-  
-  deref(base,1) = GCdwsweakvll;
+  deref(htabv,1) = GCdwsweakvll;
   GCdwsweakvll = htabv;
 }
 
@@ -274,13 +323,13 @@ void
 traditional_mark_weak_htabv(LispObj htabv)
 {
   int i, skip = hash_table_vector_header_count;;
+  LispObj *base = (LispObj *) ptr_from_lispobj(untag(htabv));
 
   for (i = 2; i <= skip; i++) {
-    rmark(deref(htabv,i));
+    rmark(base[i]);
   }
-
-  deref(htabv,1) = GCweakvll;
-  GCweakvll = htabv;
+  base[1] = GCweakvll;
+  GCweakvll = ptr_to_lispobj(base);
 }
 
 void
@@ -308,30 +357,40 @@ ncircle_mark_weak_htabv(LispObj htabv)
     rmark(*pairp);
     pairp += 2;
   }
-
-  deref(htabv,1) = GCweakvll;
-  GCweakvll = htabv;
+  deref(htabv,1)  = GCweakvll;
+  GCweakvll = (LispObj)untag(htabv);
 }
 
 
 Boolean
 mark_weak_hash_vector(hash_table_vector_header *hashp, natural elements)
 {
-  natural flags = hashp->flags, key_dnode, val_dnode;
+  natural flags = hashp->flags, weak_dnode, nonweak_dnode;
   Boolean 
     marked_new = false, 
-    key_marked,
-    val_marked,
-    weak_value = ((flags & nhash_weak_value_mask) != 0);
+    weak_marked;
+  int non_weak_index = (((flags & nhash_weak_value_mask) != 0) ? 0 : 1);
   int 
     skip = hash_table_vector_header_count-1,
-    key_tag,
-    val_tag,
+    weak_tag,
+    nonweak_tag,
     i;
+  signed_natural
+    npairs = (elements - skip) >> 1;
   LispObj 
     *pairp = (LispObj*) (hashp+1),
-    key,
-    val;
+    weak,
+    nonweak;
+
+  natural *tenured_low = (LispObj *)tenured_area->low;
+  natural tenured_dnodes = area_dnode(GCarealow, tenured_low);
+  natural memo_dnode = area_dnode(ptr_to_lispobj(pairp+non_weak_index), tenured_low);
+  Boolean hashv_tenured = (memo_dnode < tenured_dnodes);
+  natural bits, bitidx, *bitsp;
+
+  if (hashv_tenured) {
+    set_bitidx_vars(tenured_area->refbits, memo_dnode, bitsp, bits, bitidx);
+  }
 
   /* Mark everything in the header */
   
@@ -339,40 +398,53 @@ mark_weak_hash_vector(hash_table_vector_header *hashp, natural elements)
     mark_root(deref(ptr_to_lispobj(hashp),i));
   }
 
-  elements -= skip;
-
-  for (i = 0; i<elements; i+=2, pairp+=2) {
-    key = pairp[0];
-    val = pairp[1];
-    key_marked = val_marked = true;
-    key_tag = fulltag_of(key);
-    val_tag = fulltag_of(val);
-    if (is_node_fulltag(key_tag)) {
-      key_dnode = gc_area_dnode(key);
-      if ((key_dnode < GCndnodes_in_area) &&
-          ! ref_bit(GCmarkbits,key_dnode)) {
-        key_marked = false;
+  while (true) {
+    if (hashv_tenured) {
+      while (bits == 0) {
+        int skip = nbits_in_word - bitidx;
+        npairs -= skip;
+        if (npairs <= 0) break;
+        pairp += (skip+skip);
+        bitidx = 0;
+        bits = *++bitsp;
+      }
+      if (bits != 0) {
+        int skip = count_leading_zeros(bits) - bitidx;
+        if (skip != 0) {
+          npairs -= skip;
+          pairp += (skip+skip);
+          bitidx += skip;
+        }
       }
     }
-    if (is_node_fulltag(val_tag)) {
-      val_dnode = gc_area_dnode(val);
-      if ((val_dnode < GCndnodes_in_area) &&
-          ! ref_bit(GCmarkbits,val_dnode)) {
-        val_marked = false;
+    if (npairs <= 0) break;
+
+    nonweak = pairp[non_weak_index];
+    weak = pairp[1-non_weak_index];
+
+    nonweak_tag = fulltag_of(nonweak);
+    if (is_node_fulltag(nonweak_tag)) {
+      nonweak_dnode = gc_area_dnode(nonweak);
+      if ((nonweak_dnode < GCndnodes_in_area) &&
+          ! ref_bit(GCmarkbits,nonweak_dnode)) {
+        weak_marked = true;
+        weak_tag = fulltag_of(weak);
+        if (is_node_fulltag(weak_tag)) {
+          weak_dnode = gc_area_dnode(weak);
+          if ((weak_dnode < GCndnodes_in_area) &&
+              ! ref_bit(GCmarkbits, weak_dnode)) {
+            weak_marked = false;
+          }
+        }
+        if (weak_marked) {
+          mark_root(nonweak);
+          marked_new = true;
+        }
       }
     }
 
-    if (weak_value) {
-      if (val_marked & !key_marked) {
-        mark_root(key);
-        marked_new = true;
-      }
-    } else {
-      if (key_marked & !val_marked) {
-        mark_root(val);
-        marked_new = true;
-      }
-    }
+    pairp+=2;
+    --npairs;
   }
   return marked_new;
 }
@@ -425,10 +497,35 @@ mark_weak_alist(LispObj weak_alist, int weak_type)
   return marked_new;
 }
   
+void
+mark_termination_lists()
+{
+  /* 
+     Mark the termination lists in all terminatable weak vectors, which
+     are now linked together on GCweakvll, and add them to WEAKVLL,
+     which already contains all other weak vectors.
+  */
+  LispObj pending = GCweakvll,
+          *base = (LispObj *)NULL;
+
+  while (pending) {
+    base = ptr_from_lispobj(pending);
+    pending = base[1];
+
+    mark_root(base[1+3]);
+  }
+  if (base) {
+    base[1] = lisp_global(WEAKVLL);
+    lisp_global(WEAKVLL) = GCweakvll;
+  }
+
+}
+
+
 void
 traditional_markhtabvs()
 {
-  LispObj this, header, pending;
+  LispObj *base, this, header, pending;
   int subtag;
   hash_table_vector_header *hashp;
   Boolean marked_new;
@@ -438,16 +535,17 @@ traditional_markhtabvs()
     marked_new = false;
     
     while (GCweakvll) {
-      this = GCweakvll;
-      GCweakvll = deref(this,1);
+      base = ptr_from_lispobj(GCweakvll);
+      GCweakvll = base[1];
       
-      header = header_of(this);
+      header = base[0];
       subtag = header_subtag(header);
       
       if (subtag == subtag_weak) {
-        natural weak_type = deref(this,2);
-        deref(this,1) = pending;
-        pending = this;
+        natural weak_type = base[2];
+        this = ptr_to_lispobj(base) + fulltag_misc;
+        base[1] = pending;
+        pending = ptr_to_lispobj(base);
         if ((weak_type & population_type_mask) == population_weak_alist) {
           if (mark_weak_alist(this, weak_type)) {
             marked_new = true;
@@ -456,16 +554,16 @@ traditional_markhtabvs()
       } else if (subtag == subtag_hash_vector) {
         natural elements = header_element_count(header);
 
-        hashp = (hash_table_vector_header *) ptr_from_lispobj(untag(this));
+        hashp = (hash_table_vector_header *) base;
         if (hashp->flags & nhash_weak_mask) {
-          deref(this,1) = pending;
-          pending = this;
+          base[1] = pending;
+          pending = ptr_to_lispobj(base);
           if (mark_weak_hash_vector(hashp, elements)) {
             marked_new = true;
           }
         } 
       } else {
-        Bug(NULL, "Strange object on weak vector linked list: 0x~08x\n", this);
+        Bug(NULL, "Strange object on weak vector linked list: " LISP "\n", base);
       }
     }
 
@@ -480,40 +578,27 @@ traditional_markhtabvs()
      */
 
   while (pending) {
-    this = pending;
-    pending = deref(this,1);
-    deref(this,1) = (LispObj)NULL;
+    base = ptr_from_lispobj(pending);
+    pending = base[1];
+    base[1] = (LispObj)NULL;
 
-    subtag = header_subtag(header_of(this));
+    this = ptr_to_lispobj(base) + fulltag_misc;
+
+    subtag = header_subtag(base[0]);
     if (subtag == subtag_weak) {
       reapweakv(this);
     } else {
       reaphashv(this);
     }
   }
-
-  /* Finally, mark the termination lists in all terminatable weak vectors
-     They are now linked together on GCweakvll.
-     This is where to store  lisp_global(TERMINATION_LIST) if we decide to do that,
-     but it will force terminatable popualations to hold on to each other
-     (set TERMINATION_LIST before clearing GCweakvll, and don't clear deref(this,1)).
-     */
-  pending = GCweakvll;
-  GCweakvll = (LispObj)NULL;
-  while (pending) {
-    this = pending;
-    pending = deref(this,1);
-    deref(this,1) = (LispObj)NULL;
-    mark_root(deref(this,1+3));
-  }
+  mark_termination_lists();
 }
 
 void
 ncircle_markhtabvs()
 {
-  LispObj this, header, pending = 0;
+  LispObj *base, this, header, pending = 0;
   int subtag;
-  Boolean marked_new;
 
   /* First, process any weak hash tables that may have
      been encountered by the link-inverting marker; we
@@ -526,20 +611,21 @@ ncircle_markhtabvs()
   }
 
   while (GCweakvll) {
-    this = GCweakvll;
-    GCweakvll = deref(this,1);
-      
-    header = header_of(this);
+    base = ptr_from_lispobj(GCweakvll);
+    GCweakvll = base[1];
+    base[1] = (LispObj)NULL;
+
+    this = ptr_to_lispobj(base) + fulltag_misc;
+
+    header = base[0];
     subtag = header_subtag(header);
       
     if (subtag == subtag_weak) {
-      natural weak_type = deref(this,2);
-      deref(this,1) = pending;
-      pending = this;
+      natural weak_type = base[2];
+      base[1] = pending;
+      pending = ptr_to_lispobj(base);
       if ((weak_type & population_type_mask) == population_weak_alist) {
-        if (mark_weak_alist(this, weak_type)) {
-          marked_new = true;
-          }
+        mark_weak_alist(this, weak_type);
       }
     } else if (subtag == subtag_hash_vector) {
       reaphashv(this);
@@ -552,11 +638,13 @@ ncircle_markhtabvs()
      */
 
   while (pending) {
-    this = pending;
-    pending = deref(this,1);
-    deref(this,1) = (LispObj)NULL;
+    base = ptr_from_lispobj(pending);
+    pending = base[1];
+    base[1] = (LispObj)NULL;
 
-    subtag = header_subtag(header_of(this));
+    this = ptr_to_lispobj(base) + fulltag_misc;
+
+    subtag = header_subtag(base[0]);
     if (subtag == subtag_weak) {
       reapweakv(this);
     } else {
@@ -564,20 +652,7 @@ ncircle_markhtabvs()
     }
   }
 
-  /* Finally, mark the termination lists in all terminatable weak vectors
-     They are now linked together on GCweakvll.
-     This is where to store  lisp_global(TERMINATION_LIST) if we decide to do that,
-     but it will force terminatable popualations to hold on to each other
-     (set TERMINATION_LIST before clearing GCweakvll, and don't clear deref(this,1)).
-     */
-  pending = GCweakvll;
-  GCweakvll = (LispObj)NULL;
-  while (pending) {
-    this = pending;
-    pending = deref(this,1);
-    deref(this,1) = (LispObj)NULL;
-    mark_root(deref(this,1+3));
-  }
+  mark_termination_lists();
 }
 
 void
@@ -666,7 +741,7 @@ freeGCptrs()
 {
   void *p, *next, *addr;
   struct xmacptr *x, *xnext;
-  int i, flags;
+  int flags;
   xmacptr_dispose_fn dfn;
 
   for (p = postGCptrs; p; p = next) {
@@ -676,7 +751,7 @@ freeGCptrs()
   postGCptrs = NULL;
   
   for (x = user_postGC_macptrs; x; x = xnext) {
-    xnext = (xmacptr *) (x->class);;
+    xnext = (xmacptr *) (x->class);
     flags = x->flags - xmacptr_flag_user_first;
     dfn = xmacptr_dispose_functions[flags];
     addr = (void *) x->address;
@@ -845,6 +920,127 @@ install_weak_mark_functions(natural set) {
   }
 }
 
+void
+init_weakvll ()
+{
+  LispObj this = lisp_global(WEAKVLL); /* all weak vectors as of last gc */
+
+  GCweakvll = (LispObj)NULL;
+  lisp_global(WEAKVLL) = (LispObj)NULL;
+
+  if (GCn_ephemeral_dnodes) {
+    /* For egc case, initialize GCweakvll with weak vectors not in the
+       GC area.  Weak vectors in the GC area will be added during marking.
+    */
+
+    LispObj *tenured_low = (LispObj *)tenured_area->low;
+    natural tenured_dnodes = area_dnode(GCarealow, tenured_low);
+    bitvector refbits = tenured_area->refbits;
+
+    while (this) {
+      LispObj *base = ptr_from_lispobj(this);
+      LispObj next = base[1];
+      natural dnode = gc_dynamic_area_dnode(this);
+      if (dnode < GCndynamic_dnodes_in_area) {
+        base[1] = (LispObj)NULL; /* drop it, might be garbage */
+      } else {
+        base[1] = GCweakvll;
+        GCweakvll = ptr_to_lispobj(base);
+        if (header_subtag(base[0]) == subtag_weak) {
+          dnode = area_dnode(&base[3], tenured_low);
+          if (dnode < tenured_dnodes) {
+            clr_bit(refbits, dnode); /* Don't treat population.data as root */
+          }
+        } else {
+          if (header_subtag(base[0]) != subtag_hash_vector)
+            Bug(NULL, "Unexpected entry " LISP " -> " LISP " on WEAKVLL", base, base[0]);
+          dnode = area_dnode(base, tenured_low);
+          if ((dnode < tenured_dnodes) && !ref_bit(refbits, dnode)) {
+            Boolean drop = true;
+            /* hash vectors get marked headers if they have any ephemeral keys */
+            /* but not if they have ephemeral values. */
+            if (((hash_table_vector_header *)base)->flags & nhash_weak_value_mask) {
+              signed_natural count = (header_element_count(base[0]) + 2) >> 1;
+              natural bits, bitidx, *bitsp;
+              set_bitidx_vars(refbits, dnode, bitsp, bits, bitidx);
+              while ((0 < count) && (bits == 0)) {
+                int skip = nbits_in_word - bitidx;
+                count -= skip;
+                bits = *++bitsp;
+                bitidx = 0;
+              }
+              count -=  (count_leading_zeros(bits) - bitidx);
+
+              if (0 < count) {
+                set_bit(refbits, dnode); /* has ephemeral values, mark header */
+                drop = false;
+              }
+            }
+            if (drop) { /* if nothing ephemeral, drop it from GCweakvll. */
+              GCweakvll = base[1];
+              base[1] = lisp_global(WEAKVLL);
+              lisp_global(WEAKVLL) = ptr_to_lispobj(base);
+            }
+          }
+        }
+      }
+      this = next;
+    }
+  }
+}
+
+  
+void
+preforward_weakvll ()
+{
+  /* reset population refbits for forwarding */
+  if (GCn_ephemeral_dnodes) {
+    LispObj this = lisp_global(WEAKVLL);
+    LispObj *tenured_low = (LispObj *)tenured_area->low;
+    natural tenured_dnodes = area_dnode(GCarealow, tenured_low);
+    bitvector refbits = tenured_area->refbits;
+
+    while (this) {
+      LispObj *base = ptr_from_lispobj(this);
+      if (header_subtag(base[0]) == subtag_weak) {
+        natural dnode = area_dnode(&base[3], tenured_low);
+        if (base[3] >= GCarealow) {
+          if (dnode < tenured_dnodes) {
+            set_bit(refbits, dnode);
+          }
+        }
+        /* might have set termination list to a new pointer */
+        if ((base[2] >> population_termination_bit) && (base[4] >= GCarealow)) {
+          if ((dnode + 1) < tenured_dnodes) {
+            set_bit(refbits, dnode+1);
+          }
+        }
+      }
+      this = base[1];
+    }
+  }
+}
+
+
+void
+forward_weakvll_links()
+{
+  LispObj *ptr = &(lisp_global(WEAKVLL)), this, new, old;
+
+  while ((this = *ptr)) {
+    old = this + fulltag_misc;
+    new = node_forwarding_address(old);
+    if (old != new) {
+      *ptr = untag(new);
+    }
+    ptr = &(deref(new,1));
+  }
+}
+
+
+
+
+
 LispObj
 node_forwarding_address(LispObj node)
 {
@@ -904,7 +1100,7 @@ forward_gcable_ptrs()
     prev = &(((xmacptr *)ptr_from_lispobj(untag(next)))->link);
   }
   xprev = &user_postGC_macptrs;
-  while (xnext = *xprev) {
+  while ((xnext = *xprev)) {
     xnew = (struct xmacptr *)locative_forwarding_address((LispObj)xnext);
     if (xnew != xnext) {
       *xprev = xnew;
@@ -918,78 +1114,95 @@ forward_memoized_area(area *a, natural num_memo_dnodes)
 {
   bitvector refbits = a->refbits;
   LispObj *p = (LispObj *) a->low, x1, x2, new;
+#ifdef ARM
+  LispObj *p0 = p;
+#endif
   natural bits, bitidx, *bitsp, nextbit, diff, memo_dnode = 0, hash_dnode_limit = 0;
   int tag_x1;
   hash_table_vector_header *hashp = NULL;
   Boolean header_p;
 
-  if (GCDebug) {
-    check_refmap_consistency(p, p+(num_memo_dnodes << 1), refbits);
-  }
+  if (num_memo_dnodes) {
+    if (GCDebug) {
+      check_refmap_consistency(p, p+(num_memo_dnodes << 1), refbits);
+    }
 
-  /* This is pretty straightforward, but we have to note
-     when we move a key in a hash table vector that wants
-     us to tell it about that. */
+    /* This is pretty straightforward, but we have to note
+       when we move a key in a hash table vector that wants
+       us to tell it about that. */
 
-  set_bitidx_vars(refbits, 0, bitsp, bits, bitidx);
-  while (memo_dnode < num_memo_dnodes) {
-    if (bits == 0) {
-      int remain = nbits_in_word - bitidx;
-      memo_dnode += remain;
-      p += (remain+remain);
-      bits = *++bitsp;
-      bitidx = 0;
-    } else {
-      nextbit = count_leading_zeros(bits);
-      if ((diff = (nextbit - bitidx)) != 0) {
-        memo_dnode += diff;
-        bitidx = nextbit;
-        p += (diff+diff);
-      }
-      x1 = p[0];
-      x2 = p[1];
-      tag_x1 = fulltag_of(x1);
-      bits &= ~(BIT0_MASK >> bitidx);
-      header_p = (nodeheader_tag_p(tag_x1));
+    set_bitidx_vars(refbits, 0, bitsp, bits, bitidx);
+    while (memo_dnode < num_memo_dnodes) {
+      if (bits == 0) {
+        int remain = nbits_in_word - bitidx;
+        memo_dnode += remain;
+        p += (remain+remain);
+        if (memo_dnode < num_memo_dnodes) {
+          bits = *++bitsp;
+        }
+        bitidx = 0;
+      } else {
+        nextbit = count_leading_zeros(bits);
+        if ((diff = (nextbit - bitidx)) != 0) {
+          memo_dnode += diff;
+          bitidx = nextbit;
+          p += (diff+diff);
+        }
+        x1 = p[0];
+        x2 = p[1];
+        tag_x1 = fulltag_of(x1);
+        bits &= ~(BIT0_MASK >> bitidx);
+        header_p = (nodeheader_tag_p(tag_x1));
 
-      if (header_p &&
-          (header_subtag(x1) == subtag_hash_vector)) {
-        hashp = (hash_table_vector_header *) p;
-        if (hashp->flags & nhash_track_keys_mask) {
-          hash_dnode_limit = memo_dnode + ((header_element_count(x1)+2)>>1);
-        } else {
-          hashp = NULL;
+        if (header_p &&
+            (header_subtag(x1) == subtag_hash_vector)) {
+          hashp = (hash_table_vector_header *) p;
+          if (hashp->flags & nhash_track_keys_mask) {
+            hash_dnode_limit = memo_dnode + ((header_element_count(x1)+2)>>1);
+          } else {
+            hashp = NULL;
+          }
         }
-      }
 
 
-      if (! header_p) {
-        new = node_forwarding_address(x1);
-        if (new != x1) {
+        if (! header_p) {
+          new = node_forwarding_address(x1);
+          if (new != x1) {
+            *p = new;
+#ifdef ARM
+            if (p != p0) {
+              if(header_subtag(p[-2]) == subtag_function) {
+                /* Just updated the code vector; fix the entrypoint */
+                if (p[-1] == (untag(x1)+fulltag_odd_fixnum)) {
+                  p[-1] = (untag(new)+fulltag_odd_fixnum);
+                }
+              }
+            }
+#endif
+          }
+        }
+        p++;
+
+        new = node_forwarding_address(x2);
+        if (new != x2) {
           *p = new;
+          if (memo_dnode < hash_dnode_limit) {
+            /* If this code is reached, 'hashp' is non-NULL and pointing
+               at the header of a hash_table_vector, and 'memo_dnode' identifies
+               a pair of words inside the hash_table_vector.  It may be
+               hard for program analysis tools to recognize that, but I
+               believe that warnings about 'hashp' being NULL here can
+               be safely ignored. */
+            hashp->flags |= nhash_key_moved_mask;
+            hash_dnode_limit = 0;
+            hashp = NULL;
+          }
         }
-      }
-      p++;
+        p++;
+        memo_dnode++;
+        bitidx++;
 
-      new = node_forwarding_address(x2);
-      if (new != x2) {
-        *p = new;
-        if (memo_dnode < hash_dnode_limit) {
-          /* If this code is reached, 'hashp' is non-NULL and pointing
-             at the header of a hash_table_vector, and 'memo_dnode' identifies
-             a pair of words inside the hash_table_vector.  It may be
-             hard for program analysis tools to recognize that, but I
-             believe that warnings about 'hashp' being NULL here can
-             be safely ignored. */
-          hashp->flags |= nhash_key_moved_mask;
-          hash_dnode_limit = 0;
-          hashp = NULL;
-        }
       }
-      p++;
-      memo_dnode++;
-      bitidx++;
-
     }
   }
 }
@@ -1015,42 +1228,37 @@ forward_tcr_tlb(TCR *tcr)
 void
 reclaim_static_dnodes()
 {
-  natural nstatic = tenured_area->static_dnodes, i, bits, bitnum;
+  natural nstatic = tenured_area->static_dnodes, 
+    i, 
+    bits, 
+    bitnum,
+    nfree = 0,
+    nstatic_conses = area_dnode(static_cons_area->high, static_cons_area->low);
   cons *c = (cons *)tenured_area->low, *d;
   bitvector bitsp = GCmarkbits;
   LispObj head = lisp_global(STATIC_CONSES);
 
-  if (nstatic) {
-    if (head) {
-      for (i = 0; i < nstatic; i+= nbits_in_word, c+= nbits_in_word) {
-        bits = *bitsp++;
-        if (bits != ALL_ONES) {
-          for (bitnum = 0; bitnum < nbits_in_word; bitnum++) {
-            if (! (bits & (BIT0_MASK>>bitnum))) {
-              d = c + bitnum;
-              d->car = 0;
-              d->cdr = head;
-              head = ((LispObj)d)+fulltag_cons;
-            }
-          }
-        }
-      }
-      lisp_global(STATIC_CONSES) = head;
-    } else {
-      for (i = 0; i < nstatic; i+= nbits_in_word, c+= nbits_in_word) {
-        bits = *bitsp++;
-        if (bits != ALL_ONES) {
-          for (bitnum = 0; bitnum < nbits_in_word; bitnum++) {
-            if (! (bits & (BIT0_MASK>>bitnum))) {
-              d = c + bitnum;
-              d->car = 0;
-              d->cdr = 0;
-            }
+  for (i = 0; i < nstatic; i+= nbits_in_word, c+= nbits_in_word) {
+    bits = *bitsp++;
+    if (bits != ALL_ONES) {
+      for (bitnum = 0; bitnum < nbits_in_word; bitnum++) {
+        if (! (bits & (BIT0_MASK>>bitnum))) {
+          d = c + bitnum;
+          if (i < nstatic_conses) {                
+            d->car = unbound;
+            d->cdr = head;
+            head = ((LispObj)d)+fulltag_cons;
+            nfree++;
+          } else {
+            d->car = 0;
+            d->cdr = 0;
           }
         }
       }
     }
   }
+  lisp_global(STATIC_CONSES) = head;
+  lisp_global(FREE_STATIC_CONSES)+=(nfree<<fixnumshift);
 }
 
 Boolean
@@ -1086,6 +1294,83 @@ Boolean just_purified_p = false;
 #endif
 
 
+Boolean
+mark_static_ref(LispObj n, BytePtr dynamic_start, natural ndynamic_dnodes)
+{
+  int tag_n = fulltag_of(n);
+  natural dyn_dnode;
+
+  if (nodeheader_tag_p(tag_n)) {
+    return (header_subtag(n) == subtag_hash_vector);
+  }
+ 
+  if (is_node_fulltag (tag_n)) {
+    dyn_dnode = area_dnode(n, dynamic_start);
+    if (dyn_dnode < ndynamic_dnodes) {
+      mark_root(n);             /* May or may not mark it */
+      return true;              /* but return true 'cause it's a dynamic node */
+    }
+  }
+  return false;                 /* Not a heap pointer or not dynamic */
+}
+
+void
+mark_managed_static_refs(area *a, BytePtr low_dynamic_address, natural ndynamic_dnodes)
+{
+  bitvector refbits = a->refbits;
+  LispObj *p = (LispObj *) a->low, x1, x2;
+  natural inbits, outbits, bits, bitidx, *bitsp, nextbit, diff, memo_dnode = 0,
+    num_memo_dnodes = a->ndnodes;
+  Boolean keep_x1, keep_x2;
+
+  if (num_memo_dnodes) {
+    if (GCDebug) {
+      check_refmap_consistency(p, p+(num_memo_dnodes << 1), refbits);
+    }
+
+ 
+    set_bitidx_vars(refbits, 0, bitsp, bits, bitidx);
+    inbits = outbits = bits;
+    while (memo_dnode < num_memo_dnodes) {
+      if (bits == 0) {
+        int remain = nbits_in_word - bitidx;
+        memo_dnode += remain;
+        p += (remain+remain);
+        if (outbits != inbits) {
+          *bitsp = outbits;
+        }
+        if (memo_dnode < num_memo_dnodes) {
+          bits = *++bitsp;
+        }
+        inbits = outbits = bits;
+        bitidx = 0;
+      } else {
+        nextbit = count_leading_zeros(bits);
+        if ((diff = (nextbit - bitidx)) != 0) {
+          memo_dnode += diff;
+          bitidx = nextbit;
+          p += (diff+diff);
+        }
+        x1 = *p++;
+        x2 = *p++;
+        bits &= ~(BIT0_MASK >> bitidx);
+        keep_x1 = mark_static_ref(x1, low_dynamic_address, ndynamic_dnodes);
+        keep_x2 = mark_static_ref(x2, low_dynamic_address, ndynamic_dnodes);
+        if ((keep_x1 == false) && 
+            (keep_x2 == false)) {
+          outbits &= ~(BIT0_MASK >> bitidx);
+        }
+        memo_dnode++;
+        bitidx++;
+      }
+    }
+    if (GCDebug) {
+      p = (LispObj *) a->low;
+      check_refmap_consistency(p, p+(num_memo_dnodes << 1), refbits);
+    }
+  }
+}
+
 void 
 gc(TCR *tcr, signed_natural param)
 {
@@ -1096,12 +1381,11 @@ gc(TCR *tcr, signed_natural param)
   LispObj
     pkg = 0,
     itabvec = 0;
-  BytePtr oldfree = a->active;
+  BytePtr oldfree = a->active, last_zeroed_addr;
   TCR *other_tcr;
   natural static_dnodes;
+  natural weak_method = lisp_global(WEAK_GC_METHOD) >> fixnumshift;
 
-  install_weak_mark_functions(lisp_global(WEAK_GC_METHOD) >> fixnumshift);
-  
 #ifndef FORCE_DWS_MARK
   if ((natural) (tcr->cs_limit) == CS_OVERFLOW_FORCE_LIMIT) {
     GCstack_limit = CS_OVERFLOW_FORCE_LIMIT;
@@ -1145,6 +1429,8 @@ gc(TCR *tcr, signed_natural param)
     note = tenured_area;
   }
 
+  install_weak_mark_functions(weak_method);
+  
   if (GCverbose) {
     char buf[16];
 
@@ -1174,6 +1460,7 @@ gc(TCR *tcr, signed_natural param)
     GCDebug = ((nrs_GC_EVENT_STATUS_BITS.vcell & gc_integrity_check_bit) != 0);
     if (GCDebug) {
       check_all_areas(tcr);
+      check_static_cons_freelist("in pre-gc static-cons check");
     }
   }
 
@@ -1192,18 +1479,24 @@ gc(TCR *tcr, signed_natural param)
       GCmarkbits + ((GCndnodes_in_area-GCndynamic_dnodes_in_area)>>bitmap_shift);
 
     zero_bits(GCmarkbits, GCndnodes_in_area);
-    GCweakvll = (LispObj)NULL;
+
+    init_weakvll();
 
     if (GCn_ephemeral_dnodes == 0) {
       /* For GCTWA, mark the internal package hash table vector of
        *PACKAGE*, but don't mark its contents. */
       {
         LispObj
-          itab;
+          itab,
+          pkgidx = nrs_PACKAGE.binding_index;
         natural
           dnode, ndnodes;
       
-        pkg = nrs_PACKAGE.vcell;
+        if ((pkgidx >= tcr->tlb_limit) ||
+            ((pkg = tcr->tlb_pointer[pkgidx>>fixnumshift]) == 
+             no_thread_local_binding_marker)) {
+          pkg = nrs_PACKAGE.vcell;
+        }
         if ((fulltag_of(pkg) == fulltag_misc) &&
             (header_subtag(header_of(pkg)) == subtag_package)) {
           itab = ((package *)ptr_from_lispobj(untag(pkg)))->itab;
@@ -1257,10 +1550,12 @@ gc(TCR *tcr, signed_natural param)
       }
     }
   
-    if (lisp_global(OLDEST_EPHEMERAL)) {
+    if (GCephemeral_low) {
       mark_memoized_area(tenured_area, area_dnode(a->low,tenured_area->low));
     }
 
+    mark_managed_static_refs(managed_static_area,low_markable_address,area_dnode(a->active,low_markable_address));
+    
     other_tcr = tcr;
     do {
       mark_tcr_xframes(other_tcr);
@@ -1334,6 +1629,8 @@ gc(TCR *tcr, signed_natural param)
   
     reap_gcable_ptrs();
 
+    preforward_weakvll();
+
     GCrelocptr = global_reloctab;
     GCfirstunmarked = calculate_relocation();
 
@@ -1393,16 +1690,26 @@ gc(TCR *tcr, signed_natural param)
       forward_memoized_area(tenured_area, area_dnode(a->low, tenured_area->low));
     }
   
+    forward_memoized_area(managed_static_area,area_dnode(managed_static_area->active,managed_static_area->low));
     a->active = (BytePtr) ptr_from_lispobj(compact_dynamic_heap());
+
+    forward_weakvll_links();
+
     if (to) {
       tenure_to_area(to);
     }
 
-    zero_memory_range(a->active, oldfree);
 
     resize_dynamic_heap(a->active,
                         (GCephemeral_low == 0) ? lisp_heap_gc_threshold : 0);
 
+    if (oldfree < a->high) {
+      last_zeroed_addr = oldfree;
+    } else {
+      last_zeroed_addr = a->high;
+    }
+    zero_memory_range(a->active, last_zeroed_addr);
+
     /*
       If the EGC is enabled: If there's no room for the youngest
       generation, untenure everything.  If this was a full GC and
@@ -1428,6 +1735,7 @@ gc(TCR *tcr, signed_natural param)
 
   if (GCDebug) {
     check_all_areas(tcr);
+    check_static_cons_freelist("in post-gc static-cons check");
   }
 
   
