@@ -19,7 +19,7 @@
 #include "lisp-exceptions.h"
 #include "lisp_globals.h"
 #include "area.h"
-#include "Threads.h"
+#include "threads.h"
 #include <ctype.h>
 #include <stdio.h>
 #include <stddef.h>
@@ -45,6 +45,15 @@ typedef enum {
   debug_kill
 } debug_command_return;
 
+#ifdef SVN_REVISION
+#define xstr(s) str(s)
+#define str(s) #s
+char *kernel_svn_revision = xstr(SVN_REVISION);
+#undef xstr
+#undef str
+#else
+char *kernel_svn_revision = "unknown";
+#endif
 
 Boolean
 open_debug_output(int fd)
@@ -134,12 +143,14 @@ foreign_name_and_offset(natural addr, int *delta)
     *delta = 0;
   }
 #ifndef WINDOWS
+#ifndef ANDROID
   if (dladdr((void *)addr, &info)) {
     ret = (char *)info.dli_sname;
     if (delta) {
       *delta = ((natural)addr - (natural)info.dli_saddr);
     }
   }
+#endif
 #endif
   return ret;
 }
@@ -284,9 +295,60 @@ show_lisp_register(ExceptionInformation *xp, char *label, int r)
     fprintf(dbgout, "%%%s (%s) = %s\n", Iregnames[r], label, s);
   }
 #endif
-
+#ifdef ARM
+  fprintf(dbgout, "r%02d (%s) = %s\n", r, label, print_lisp_object(val));
+#endif
 }
 
+void
+describe_siginfo(siginfo_t *info)
+{
+#if defined(WINDOWS) || defined(FREEBSD) || defined(DARWIN)
+  /*
+   * It's not surprising that Windows doesn't have this signal stuff.
+   * It is somewhat surprising that FreeBSD 6.x lacks the si_code
+   * constants.  (Subsequent FreeBSD versions define them, though.)
+   *
+   * On Darwin, recall that we handle exceptions at the Mach level,
+   * and build a "fake" signal context ourselves.  We don't try very
+   * hard to translate the Mach exception information to Unix-style
+   * information, so avoid printing out possibly-misleading garbage.
+   * (bsd/dev/i386/unix_signal.c from the xnu sources is where that
+   * happens for Mac OS X's own Mach-exception-to-Unix-signal
+   * translation.
+   */
+#else
+  if (info->si_code > 0) {
+    if (info->si_signo == SIGSEGV) {
+      switch (info->si_code) {
+      case SEGV_MAPERR:
+	fprintf(dbgout, "address not mapped to object\n");
+	break;
+      case SEGV_ACCERR:
+	fprintf(dbgout, "invalid permissions for mapped object\n");
+	break;
+      default:
+	fprintf(dbgout, "unexpected si_code value: %d\n", info->si_code);
+	break;
+      }
+    } else if (info->si_signo == SIGBUS) {
+      switch (info->si_code) {
+      case BUS_ADRALN:
+	fprintf(dbgout, "invalid address alignment\n");
+	break;
+      case BUS_ADRERR:
+	fprintf(dbgout, "non-existent physical address");
+	break;
+      case BUS_OBJERR:
+	fprintf(dbgout, "object-specific hardware error");
+	break;
+      default:
+	fprintf(dbgout, "unexpected si_code value: %d\n", info->si_code);
+      }
+    }
+  }
+#endif
+}
 
 void
 describe_memfault(ExceptionInformation *xp, siginfo_t *info)
@@ -299,6 +361,12 @@ describe_memfault(ExceptionInformation *xp, siginfo_t *info)
 	  dsisr & (1<<25) ? "Write" : "Read",
 	  dsisr & (1<<27) ? "protected" : "unmapped",
 	  addr);
+#elif !defined(WINDOWS)
+  if (info) {
+    fprintf(dbgout, "received signal %d; faulting address: %p\n",
+            info->si_signo, info->si_addr);
+    describe_siginfo(info);
+  }
 #endif
 }
 
@@ -504,6 +572,95 @@ describe_ppc_trap(ExceptionInformation *xp)
 }
 #endif
 
+#ifdef ARM
+void
+describe_arm_uuo(ExceptionInformation *xp)
+{
+  pc program_counter = xpPC(xp);
+  opcode instruction = *program_counter;
+
+  if (IS_UUO(instruction)) {
+    unsigned format = UUO_FORMAT(instruction);
+
+    switch(format) {
+    case uuo_format_nullary:
+    case uuo_format_nullary_error:
+      switch UUOA_field(instruction) {
+      case 0:
+        fprintf(dbgout,"alloc_trap\n");
+        break;
+      case 1:
+        fprintf(dbgout,"wrong number of args (%d) to %s\n",xpGPR(xp,nargs)>>node_shift,
+                print_lisp_object(xpGPR(xp,nfn)));
+        break;
+      case 2:
+        fprintf(dbgout,"gc trap\n");
+        break;
+      case 3:
+        fprintf(dbgout,"debug trap\n");
+        break;
+      case 4:
+        fprintf(dbgout,"deferred interrupt\n");
+        break;
+      case 5:
+        fprintf(dbgout,"deferred suspend\n");
+        break;
+      default:
+        break;
+      }
+      break;
+
+    case uuo_format_unary_error:
+      switch (UUO_UNARY_field(instruction)) {
+      case 0:
+      case 1:
+        fprintf(dbgout,"%s is unbound\n", print_lisp_object(xpGPR(xp,UUOA_field(instruction))));
+        break;
+
+      default:
+        break;
+      }
+    default:
+      break;
+    }
+  }
+}
+#endif
+
+char *
+area_code_name(int code)
+{
+  switch (code) {
+    case AREA_VOID: return "void";
+    case AREA_CSTACK: return "cstack";
+    case AREA_VSTACK: return "vstack";
+    case AREA_TSTACK: return "tstack";
+    case AREA_READONLY: return "readonly";
+    case AREA_WATCHED: return "watched";
+    case AREA_STATIC_CONS: return "static cons";
+    case AREA_MANAGED_STATIC: return "managed static";
+    case AREA_STATIC: return "static";
+    case AREA_DYNAMIC: return "dynamic";
+    default: return "unknown";
+  }
+}
+
+debug_command_return
+debug_memory_areas(ExceptionInformation *xp, siginfo_t *info, int arg)
+{
+  area *a, *header = all_areas;
+  char label[100];
+
+  fprintf(dbgout, "Lisp memory areas:\n");
+  fprintf(dbgout, "%20s %20s %20s\n", "code", "low", "high");
+  for (a = header->succ; a != header; a = a->succ) {
+    snprintf(label, sizeof(label), "%s (%d)", area_code_name(a->code),
+	     a->code >> fixnumshift);
+    fprintf(dbgout, "%20s %20p %20p\n", label, a->low, a->high);
+  }
+  return debug_continue;
+}
+
 debug_command_return
 debug_lisp_registers(ExceptionInformation *xp, siginfo_t *info, int arg)
 {
@@ -569,13 +726,30 @@ debug_lisp_registers(ExceptionInformation *xp, siginfo_t *info, int arg)
   if (tag_of(xpGPR(xp,Inargs)) == tag_fixnum) {
     fprintf(dbgout,"%%edx (nargs) = %d (maybe)\n", unbox_fixnum(xpGPR(xp,Inargs)));
   }
+#endif
+#ifdef ARM
+    TCR *xpcontext = (TCR *)ptr_from_lispobj(xpGPR(xp, rcontext));
+
+    fprintf(dbgout, "rcontext = 0x%lX ", xpcontext);
+    if (!active_tcr_p(xpcontext)) {
+      fprintf(dbgout, "(INVALID)\n");
+    } else {
+      fprintf(dbgout, "\nnargs = %d\n", xpGPR(xp, nargs) >> fixnumshift);
+      show_lisp_register(xp, "fn", Rfn);
+      show_lisp_register(xp, "arg_z", arg_z);
+      show_lisp_register(xp, "arg_y", arg_y);
+      show_lisp_register(xp, "arg_x", arg_x);
+      show_lisp_register(xp, "temp0", temp0);
+      show_lisp_register(xp, "temp1/fname/next_method_context", temp1);
+      show_lisp_register(xp, "temp2/nfn", temp2);
+    }
 #endif
   }
   
   return debug_continue;
 }
 
-#ifdef PPC
+#ifndef X86
 debug_command_return
 debug_advance_pc(ExceptionInformation *xp, siginfo_t *info, int arg)
 {
@@ -587,11 +761,13 @@ debug_advance_pc(ExceptionInformation *xp, siginfo_t *info, int arg)
 debug_command_return
 debug_identify_exception(ExceptionInformation *xp, siginfo_t *info, int arg)
 {
-#ifdef PPC
+#ifndef X86
   pc program_counter = xpPC(xp);
   opcode instruction = 0;
+#endif
 
   switch (arg) {
+#ifdef PPC
   case SIGILL:
   case SIGTRAP:
     instruction = *program_counter;
@@ -602,6 +778,16 @@ debug_identify_exception(ExceptionInformation *xp, siginfo_t *info, int arg)
       describe_ppc_illegal(xp);
     }
     break;
+#endif
+
+#ifdef ARM  
+  case SIGILL:
+    instruction = *program_counter;
+    if (IS_UUO(instruction)) {
+      describe_arm_uuo(xp);
+    }
+    break;
+#endif
   case SIGSEGV:
   case SIGBUS:
     describe_memfault(xp, info);
@@ -609,7 +795,6 @@ debug_identify_exception(ExceptionInformation *xp, siginfo_t *info, int arg)
   default:
     break;
   }
-#endif
   return debug_continue;
 }
 
@@ -636,8 +821,7 @@ debug_get_string_value(char *prompt)
 natural
 debug_get_natural_value(char *prompt)
 {
-  char s[32], *res;
-  int n;
+  char s[32], *res, *endptr;
   natural val;
 
   do {
@@ -645,8 +829,8 @@ debug_get_natural_value(char *prompt)
     fprintf(dbgout, "\n  %s :", prompt);
     s[0]=0;
     res = fgets(s, 24, stdin);
-    n = sscanf(s, "%lu", &val);
-  } while (n != 1);
+    val = strtoul(res,&endptr,0);
+  } while (*endptr);
   return val;
 }
 
@@ -678,6 +862,17 @@ debug_show_symbol(ExceptionInformation *xp, siginfo_t *info, int arg)
   return debug_continue;
 }
 
+debug_command_return
+debug_show_lisp_version(ExceptionInformation *xp, siginfo_t *info, int arg)
+{
+  extern void *plsym(ExceptionInformation *,char*);
+
+  fprintf(dbgout, "Lisp kernel svn revision: %s\n", kernel_svn_revision);
+  if (xp)
+    plsym(xp, "*OPENMCL-VERSION*");
+  return debug_continue;
+}
+
 debug_command_return
 debug_thread_info(ExceptionInformation *xp, siginfo_t *info, int arg)
 {
@@ -691,14 +886,19 @@ debug_thread_info(ExceptionInformation *xp, siginfo_t *info, int arg)
             (cs_area->low), (cs_area->high));
     fprintf(dbgout, "Value (lisp) stack area: low = 0x" LISP ", high = 0x" LISP "\n",
             (u64_t)(natural)(vs_area->low), (u64_t)(natural)vs_area->high);
-    fprintf(dbgout, "Exception stack pointer = 0x" LISP "\n",
+    if (xp) {
+      fprintf(dbgout, "Exception stack pointer = 0x" LISP "\n",
 #ifdef PPC
-            (u64_t) (natural)(xpGPR(xp,1))
+              (u64_t) (natural)(xpGPR(xp,1))
 #endif
 #ifdef X86
-            (u64_t) (natural)(xpGPR(xp,Isp))
+              (u64_t) (natural)(xpGPR(xp,Isp))
+#endif           
+#ifdef ARM
+              (u64_t) (natural)(xpGPR(xp,Rsp))
 #endif
-            );
+              );
+    }
   }
   return debug_continue;
 }
@@ -838,6 +1038,15 @@ debug_show_registers(ExceptionInformation *xp, siginfo_t *info, int arg)
 
 #endif
 
+#endif
+
+#ifdef ARM
+  int a, b;
+  for (a = 0, b = 8; a < 8; a++, b++) {
+    fprintf(dbgout,"r%02d = 0x%08lX    r%02d = 0x%08lX\n",
+	    a, xpGPR(xp, a),
+	    b, xpGPR(xp, b));
+  }
 #endif
 
   return debug_continue;
@@ -859,7 +1068,7 @@ debug_show_fpu(ExceptionInformation *xp, siginfo_t *info, int arg)
 #endif
 #ifdef X8664
 #ifdef LINUX
-  struct _libc_xmmreg * xmmp = &(xp->uc_mcontext.fpregs->_xmm[0]);
+  struct _libc_xmmreg * xmmp = NULL;
 #endif
 #ifdef DARWIN
   struct xmm {
@@ -881,6 +1090,13 @@ debug_show_fpu(ExceptionInformation *xp, siginfo_t *info, int arg)
 #endif
   float *sp;
 
+#ifdef LINUX
+  if (xp->uc_mcontext.fpregs)
+    xmmp = &(xp->uc_mcontext.fpregs->_xmm[0]);
+  else
+    /* no fp state, apparently */
+    return debug_continue;
+#endif
 
   for (i = 0; i < 16; i++, xmmp++) {
     sp = (float *) xmmp;
@@ -967,8 +1183,10 @@ debug_backtrace(ExceptionInformation *xp, siginfo_t *info, int arg)
 
   if (xp) {
     plbt(xp);
+#ifndef X86
   } else {
     plbt_sp(current_stack_pointer());
+#endif
   }
   return debug_continue;
 }
@@ -988,7 +1206,7 @@ debug_command_entry debug_command_entries[] =
    DEBUG_COMMAND_FLAG_AUX_REGNO,
    "GPR to set (0-31) ?",
    'G'},
-#ifdef PPC
+#ifndef X86
   {debug_advance_pc,
    "Advance the program counter by one instruction (use with caution!)",
    DEBUG_COMMAND_FLAG_REQUIRE_XP | DEBUG_COMMAND_FLAG_EXCEPTION_ENTRY_ONLY,
@@ -1031,6 +1249,11 @@ debug_command_entry debug_command_entries[] =
    0,
    NULL,
    'T'},
+  {debug_memory_areas,
+   "Show memory areas",
+   0,
+   NULL,
+   'M'},
   {debug_win,
    "Exit from this debugger, asserting that any exception was handled",
    0,
@@ -1055,6 +1278,11 @@ debug_command_entry debug_command_entries[] =
    0,
    NULL,
    'K'},
+  {debug_show_lisp_version,
+   "Show Subversion revision information",
+   0,
+   NULL,
+   'V'},
   {debug_help,
    "Show this help",
    0,
@@ -1101,6 +1329,7 @@ apply_debug_command(ExceptionInformation *xp, int c, siginfo_t *info, int why)
   }
 }
 
+void
 debug_identify_function(ExceptionInformation *xp, siginfo_t *info) 
 {
 #ifdef PPC
@@ -1190,7 +1419,10 @@ lisp_Debugger(ExceptionInformation *xp,
       debug_lisp_registers(xp, info, 0);
       debug_show_fpu(xp, info, 0);
     }
+    debug_memory_areas(xp, info, 0);
     debug_backtrace(xp, info, 0);
+    debug_show_lisp_version(xp, info, 0);
+
     abort();
   }
 
@@ -1245,7 +1477,6 @@ FBug(ExceptionInformation *xp, const char *format, ...)
   vsnprintf(s, sizeof(s),format, args);
   va_end(args);
   lisp_Debugger(xp, NULL, debug_entry_bug, true, s);
-
 }
 
 void
