@@ -18,7 +18,7 @@
 #include "lisp.h"
 #include "lisp-exceptions.h"
 #include "lisp_globals.h"
-#include "Threads.h"
+#include "threads.h"
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,8 +30,6 @@
 #include <unistd.h>
 #ifdef LINUX
 #include <strings.h>
-#include <fpu_control.h>
-#include <linux/prctl.h>
 #endif
 
 #ifndef WINDOWS
@@ -48,35 +46,16 @@ allocation_failure(Boolean pointerp, natural size)
   Fatal(":   Kernel memory allocation failure.  ", buf);
 }
 
-void
-fatal_oserr(StringPtr param, OSErr err)
+void *
+lisp_malloc(size_t size)
 {
-  char buf[64];
-  sprintf(buf," - operating system error %d.", err);
-  Fatal(param, buf);
-}
-
-
-Ptr
-allocate(natural size)
-{
-  return (Ptr) malloc(size);
+  return malloc(size);
 }
 
 void
-deallocate(Ptr p)
+lisp_free(void *p)
 {
-  free((void *)p);
-}
-
-Ptr
-zalloc(natural size)
-{
-  Ptr p = allocate(size);
-  if (p != NULL) {
-    memset(p, 0, size);
-  }
-  return p;
+  free(p);
 }
 
 #ifdef DARWIN
@@ -128,6 +107,7 @@ address_unmapped_p(char *addr, natural len)
 #endif
 
 
+
   /*
     Through trial and error, we've found that IMAGE_BASE_ADDRESS is
     likely to reside near the beginning of an unmapped block of memory
@@ -218,11 +198,12 @@ ReserveMemoryForHeap(LogicalAddress want, natural totalsize)
 int
 CommitMemory (LogicalAddress start, natural len) 
 {
-  LogicalAddress rc;
 #if DEBUG_MEMORY
   fprintf(dbgout, "Committing memory at 0x" LISP ", size 0x" LISP "\n", start, len);
 #endif
 #ifdef WINDOWS
+  LogicalAddress rc;
+
   if ((start < ((LogicalAddress)nil_value)) &&
       (((LogicalAddress)nil_value) < (start+len))) {
     /* nil area is in the executable on Windows; ensure range is
@@ -240,7 +221,7 @@ CommitMemory (LogicalAddress start, natural len)
   }
   return true;
 #else
-  int i, err;
+  int i;
   void *addr;
 
   for (i = 0; i < 3; i++) {
@@ -308,10 +289,14 @@ MapMemoryForStack(natural nbytes)
 #ifdef WINDOWS
   return VirtualAlloc(0, nbytes, MEM_RESERVE|MEM_COMMIT, MEMPROTECT_RWX);
 #else
-  return mmap(NULL, nbytes, MEMPROTECT_RWX, MAP_PRIVATE|MAP_ANON|MAP_GROWSDOWN, -1, 0);
+  return mmap(NULL, nbytes, MEMPROTECT_RWX, MAP_PRIVATE|MAP_ANON, -1, 0);
 #endif
 }
 
+
+/* Cause the mapped memory region at ADDR to become completely unmapped.
+   ADDR should be an address returned by MapMemoryForStack() or MapMemory(),
+   and NBYTES should be the size of the mapped region at that address. */
 int
 UnMapMemory(LogicalAddress addr, natural nbytes)
 {
@@ -319,8 +304,7 @@ UnMapMemory(LogicalAddress addr, natural nbytes)
   fprintf(dbgout, "Unmapping memory at 0x" LISP ", size 0x" LISP "\n", addr, nbytes);
 #endif
 #ifdef WINDOWS
-  /* Can't MEM_RELEASE here because we only want to free a chunk */
-  return VirtualFree(addr, nbytes, MEM_DECOMMIT);
+  return !VirtualFree(addr, 0, MEM_RELEASE);
 #else
   return munmap(addr, nbytes);
 #endif
@@ -346,6 +330,13 @@ ProtectMemory(LogicalAddress addr, natural nbytes)
   
   if (status) {
     status = errno;
+    
+    if (status == ENOMEM) {
+      void *mapaddr = mmap(addr,nbytes, PROT_READ | PROT_EXEC, MAP_ANON|MAP_PRIVATE|MAP_FIXED,-1,0);
+      if (mapaddr != MAP_FAILED) {
+        return 0;
+      }
+    }
     Bug(NULL, "couldn't protect " DECIMAL " bytes at " LISP ", errno = %d", nbytes, addr, status);
   }
   return status;
@@ -438,7 +429,7 @@ unprotect_area(protected_area_ptr p)
 protected_area_ptr
 new_protected_area(BytePtr start, BytePtr end, lisp_protection_kind reason, natural protsize, Boolean now)
 {
-  protected_area_ptr p = (protected_area_ptr) allocate(sizeof(protected_area));
+  protected_area_ptr p = malloc(sizeof(protected_area));
   
   if (p == NULL) return NULL;
   p->protsize = protsize;
@@ -456,32 +447,8 @@ new_protected_area(BytePtr start, BytePtr end, lisp_protection_kind reason, natu
   return p;
 }
 
-/*
-  Un-protect the first nbytes bytes in specified area.
-  Note that this may cause the area to be empty.
-*/
-void
-unprotect_area_prefix(protected_area_ptr area, size_t delta)
-{
-  unprotect_area(area);
-  area->start += delta;
-  if ((area->start + area->protsize) <= area->end) {
-    protect_area(area);
-  }
-}
 
 
-/*
-  Extend the protected area, causing the preceding nbytes bytes
-  to be included and protected.
-*/
-void
-protect_area_prefix(protected_area_ptr area, size_t delta)
-{
-  unprotect_area(area);
-  area->start -= delta;
-  protect_area(area);
-}
 
 protected_area_ptr
 AllProtectedAreas = NULL;
@@ -530,10 +497,9 @@ Boolean
 resize_dynamic_heap(BytePtr newfree, 
 		    natural free_space_size)
 {
-  extern int page_size;
   area *a = active_dynamic_area;
-  BytePtr newlimit, protptr, zptr;
-  int psize = page_size;
+  BytePtr newlimit;
+
   if (free_space_size) {
     BytePtr lowptr = a->active;
     newlimit = lowptr + align_to_power_of_2(newfree-lowptr+free_space_size,
@@ -545,6 +511,7 @@ resize_dynamic_heap(BytePtr newfree,
       return true;
     }
   }
+  return false;
 }
 
 void
@@ -555,6 +522,9 @@ protect_area(protected_area_ptr p)
 
   if (n && ! p->nprot) {
     ProtectMemory(start, n);
+#ifdef WINDOWS
+    VirtualAlloc(start+n-page_size,page_size,MEM_COMMIT,PAGE_READWRITE|PAGE_GUARD);
+#endif
     p->nprot = n;
   }
 }
@@ -578,7 +548,7 @@ zero_page(BytePtr start)
 area *
 new_area(BytePtr lowaddr, BytePtr highaddr, area_code code)
 {
-  area *a = (area *) (zalloc(sizeof(area)));
+  area *a = calloc(1, sizeof(area));
   if (a) {
     natural ndnodes = area_dnode(highaddr, lowaddr);
     a->low = lowaddr;
@@ -741,11 +711,8 @@ tenure_to_area(area *target)
   area *a = active_dynamic_area, *child;
   BytePtr 
     curfree = a->active,
-    target_low = target->low,
-    tenured_low = tenured_area->low;
-  natural 
-    dynamic_dnodes = area_dnode(curfree, a->low),
-    new_tenured_dnodes = area_dnode(curfree, tenured_area->low);
+    target_low = target->low;
+  natural new_tenured_dnodes = area_dnode(curfree, tenured_area->low);
   bitvector 
     refbits = tenured_area->refbits,
     markbits = a->markbits,
@@ -773,7 +740,7 @@ tenure_to_area(area *target)
   }
    
   a->markbits = new_markbits;
-  lisp_global(OLDSPACE_DNODE_COUNT) = area_dnode(curfree, lisp_global(HEAP_START));
+  lisp_global(OLDSPACE_DNODE_COUNT) = area_dnode(curfree, lisp_global(REF_BASE));
 }
 
 
@@ -808,7 +775,7 @@ untenure_from_area(area *from)
     if (from == tenured_area) {
       /* Everything's in the dynamic area */
       lisp_global(OLDEST_EPHEMERAL) = 0;
-      lisp_global(OLDSPACE_DNODE_COUNT) = 0;
+      lisp_global(OLDSPACE_DNODE_COUNT) = area_dnode(managed_static_area->active,managed_static_area->low);
 
     }
   }
@@ -874,7 +841,7 @@ delete_protected_area(protected_area_ptr p)
     }
   }
 
-  deallocate((Ptr)p);
+  free(p);
 }
 
 
@@ -905,7 +872,7 @@ condemn_area_holding_area_lock(area *a)
   if (p) delete_protected_area(p);
 
   if (h) free_stack(h);
-  deallocate((Ptr)a);
+  free(a);
 }
 
 
@@ -943,15 +910,6 @@ condemn_area_chain(area *a, TCR *tcr)
   UNLOCK(lisp_global(TCR_AREA_LOCK),tcr);
 }
 
-void
-release_readonly_area()
-{
-  area *a = readonly_area;
-  UnMapMemory(a->low,align_to_power_of_2(a->active-a->low, log2_page_size));
-  a->active = a->low;
-  a->ndnodes = 0;
-  pure_space_active = pure_space_start;
-}
 
 void
 protect_watched_areas()
