@@ -15,13 +15,6 @@
    http://opensource.franz.com/preamble.html
 */
 
-#ifdef DARWIN
-/*	dyld.h included here because something in "lisp.h" causes
-    a conflict (actually I think the problem is in "constants.h")
-*/
-#include <mach-o/dyld.h>
-
-#endif
 #include "lisp.h"
 #include "lisp_globals.h"
 #include "gc.h"
@@ -43,7 +36,9 @@
 #endif
 
 #ifdef LINUX
+#ifndef ANDROID
 #include <mcheck.h>
+#endif
 #include <dirent.h>
 #include <dlfcn.h>
 #include <sys/time.h>
@@ -90,6 +85,8 @@ Boolean use_mach_exception_handling =
 #include <mach/vm_region.h>
 #include <mach/port.h>
 #include <sys/sysctl.h>
+#undef undefined
+#include <mach-o/dyld.h>
 #include <dlfcn.h>
 #endif
 
@@ -105,9 +102,11 @@ Boolean use_mach_exception_handling =
 #ifndef WINDOWS
 #include <sys/select.h>
 #endif
-#include "Threads.h"
+#include "threads.h"
 
+#if !(defined(DARWIN) && defined(ARM))
 #include <fenv.h>
+#endif
 #include <sys/stat.h>
 
 #ifndef MAP_NORESERVE
@@ -137,7 +136,7 @@ wperror(char* message)
 #endif
 
 LispObj lisp_nil = (LispObj) 0;
-bitvector global_mark_ref_bits = NULL;
+bitvector global_mark_ref_bits = NULL, dynamic_mark_ref_bits = NULL, relocatable_mark_ref_bits = NULL;
 
 
 /* These are all "persistent" : they're initialized when
@@ -240,7 +239,7 @@ allocate_lisp_stack(natural useable,
     if (hprotp == NULL) {
       if (base_p) *base_p = NULL;
       if (h_p) *h_p = NULL;
-      deallocate(h);
+      free(h);
       return NULL;
     }
     if (hardp) *hardp = hprotp;
@@ -313,6 +312,10 @@ register_cstack_holding_area_lock(BytePtr bottom, natural size)
   a->softlimit = a->hardlimit+CSTACK_SOFTPROT;
 #ifdef USE_SIGALTSTACK
   setup_sigaltstack(a);
+#endif
+#ifdef PROTECT_CSTACK
+  a->softprot=new_protected_area(a->hardlimit,a->softlimit,kSPsoftguard,CSTACK_SOFTPROT,true);
+  a->hardprot=new_protected_area(lowlimit,a->hardlimit,kSPhardguard,CSTACK_HARDPROT,true);
 #endif
   add_area_holding_area_lock(a);
   return a;
@@ -363,50 +366,6 @@ unsigned unsigned_max(unsigned x, unsigned y)
   }
 }
 
-#if WORD_SIZE == 64
-#ifdef DARWIN
-#define MAXIMUM_MAPPABLE_MEMORY (512L<<30L)
-#endif
-#ifdef FREEBSD
-#define MAXIMUM_MAPPABLE_MEMORY (512L<<30L)
-#endif
-#ifdef SOLARIS
-#define MAXIMUM_MAPPABLE_MEMORY (128L<<30L)
-#endif
-#ifdef LINUX
-#ifdef X8664
-#define MAXIMUM_MAPPABLE_MEMORY (512L<<30L)
-#endif
-#ifdef PPC
-#define MAXIMUM_MAPPABLE_MEMORY (128L<<30L)
-#endif
-#endif
-#ifdef WINDOWS
-/* Supposedly, the high-end version of Vista allow 128GB of pageable memory */
-#define MAXIMUM_MAPPABLE_MEMORY (120LL<<30LL)
-#endif
-#else
-#ifdef DARWIN
-#define MAXIMUM_MAPPABLE_MEMORY ((1U<<31)-2*heap_segment_size)
-#endif
-#ifdef LINUX
-#ifdef X86
-#define MAXIMUM_MAPPABLE_MEMORY (9U<<28)
-#else
-#define MAXIMUM_MAPPABLE_MEMORY (1U<<30)
-#endif
-#endif
-#ifdef WINDOWS
-#define MAXIMUM_MAPPABLE_MEMORY (1U<<30)
-#endif
-#ifdef FREEBSD
-#define MAXIMUM_MAPPABLE_MEMORY (1U<<30)
-#endif
-#ifdef SOLARIS
-#define MAXIMUM_MAPPABLE_MEMORY (1U<<30)
-#endif
-#endif
-
 natural
 reserved_area_size = MAXIMUM_MAPPABLE_MEMORY;
 
@@ -416,6 +375,7 @@ area
   *g2_area=NULL, 
   *g1_area=NULL,
   *managed_static_area=NULL,
+  *static_cons_area=NULL,
   *readonly_area=NULL;
 
 area *all_areas=NULL;
@@ -445,6 +405,9 @@ int cache_block_size=32;
 natural
 lisp_heap_gc_threshold = DEFAULT_LISP_HEAP_GC_THRESHOLD;
 
+natural
+lisp_heap_notify_threshold = 0;
+
 natural 
 initial_stack_size = DEFAULT_INITIAL_STACK_SIZE;
 
@@ -487,10 +450,11 @@ Boolean
 commit_pages(void *start, size_t len)
 {
   if (len != 0) {
-    if (CommitMemory(start, len)) {
-      if (touch_all_pages(start, len)) {
-	return true;
-      }
+    if (!CommitMemory(start, len)) {
+      return false;
+    }
+    if (!touch_all_pages(start, len)) {
+      return false;
     }
   }
   return true;
@@ -510,7 +474,7 @@ find_readonly_area()
 }
 
 area *
-extend_readonly_area(unsigned more)
+extend_readonly_area(natural more)
 {
   area *a;
   unsigned mask;
@@ -589,8 +553,8 @@ create_reserved_area(natural totalsize)
   static_space_start = static_space_active = (BytePtr)STATIC_BASE_ADDRESS;
   static_space_limit = static_space_start + STATIC_RESERVE;
   pure_space_start = pure_space_active = start;
-  pure_space_limit = start + PURESPACE_RESERVE;
-  start = pure_space_limit;
+  pure_space_limit = start + PURESPACE_SIZE;
+  start += PURESPACE_RESERVE;
 
   /*
     Allocate mark bits here.  They need to be 1/64 the size of the
@@ -606,7 +570,6 @@ create_reserved_area(natural totalsize)
   /* The root of all evil is initially linked to itself. */
   reserved->pred = reserved->succ = reserved;
   all_areas = reserved;
-  reserved->markbits = global_mark_ref_bits;
   return reserved;
 }
 
@@ -631,18 +594,76 @@ allocate_from_reserved_area(natural size)
 
 
 BytePtr reloctab_limit = NULL, markbits_limit = NULL;
+BytePtr low_relocatable_address = NULL, high_relocatable_address = NULL,
+  low_markable_address = NULL, high_markable_address = NULL;
+
+void
+map_initial_reloctab(BytePtr low, BytePtr high)  
+{
+  natural ndnodes, reloctab_size;
+
+  low_relocatable_address = low; /* will never change */
+  high_relocatable_address = high;
+  ndnodes = area_dnode(high,low);
+  reloctab_size = (sizeof(LispObj)*(((ndnodes+((1<<bitmap_shift)-1))>>bitmap_shift)+1));
+  
+  reloctab_limit = (BytePtr)align_to_power_of_2(((natural)global_reloctab)+reloctab_size,log2_page_size);
+  CommitMemory(global_reloctab,reloctab_limit-(BytePtr)global_reloctab);
+}
+
+void
+map_initial_markbits(BytePtr low, BytePtr high)
+{
+  natural
+    prefix_dnodes = area_dnode(low, pure_space_limit),
+    ndnodes = area_dnode(high, low),
+    prefix_size = (prefix_dnodes+7)>>3,
+    markbits_size = (3*sizeof(LispObj))+((ndnodes+7)>>3),
+    n;
+  low_markable_address = low;
+  high_markable_address = high;
+  dynamic_mark_ref_bits = (bitvector)(((BytePtr)global_mark_ref_bits)+prefix_size);
+  relocatable_mark_ref_bits = dynamic_mark_ref_bits;
+  n = align_to_power_of_2(markbits_size,log2_page_size);
+  markbits_limit = ((BytePtr)dynamic_mark_ref_bits)+n;
+  CommitMemory(dynamic_mark_ref_bits,n);
+}
+    
+void
+lower_heap_start(BytePtr new_low, area *a)
+{
+  natural new_dnodes = area_dnode(low_markable_address,new_low);
+
+  if (new_dnodes) {
+    natural n = (new_dnodes+7)>>3;
+
+    BytePtr old_markbits = (BytePtr)dynamic_mark_ref_bits,
+      new_markbits = old_markbits-n;
+    CommitMemory(new_markbits,n);
+    dynamic_mark_ref_bits = (bitvector)new_markbits;
+    if (a->refbits) {
+      a->refbits= dynamic_mark_ref_bits;
+    }
+    a->static_dnodes += new_dnodes;
+    a->ndnodes += new_dnodes;
+    a->low = new_low;
+    low_markable_address = new_low;
+    lisp_global(HEAP_START) = (LispObj)new_low;
+    static_cons_area->ndnodes = area_dnode(static_cons_area->high,new_low);
+  }
+}
 
 void
 ensure_gc_structures_writable()
 {
   natural 
-    ndnodes = area_dnode(lisp_global(HEAP_END),lisp_global(HEAP_START)),
+    ndnodes = area_dnode(lisp_global(HEAP_END),low_relocatable_address),
     markbits_size = (3*sizeof(LispObj))+((ndnodes+7)>>3),
     reloctab_size = (sizeof(LispObj)*(((ndnodes+((1<<bitmap_shift)-1))>>bitmap_shift)+1)),
     n;
   BytePtr 
     new_reloctab_limit = (BytePtr)align_to_power_of_2(((natural)global_reloctab)+reloctab_size,log2_page_size),
-    new_markbits_limit = (BytePtr)align_to_power_of_2(((natural)global_mark_ref_bits)+markbits_size,log2_page_size);
+    new_markbits_limit = (BytePtr)align_to_power_of_2(((natural)relocatable_mark_ref_bits)+markbits_size,log2_page_size);
 
   if (new_reloctab_limit > reloctab_limit) {
     n = new_reloctab_limit - reloctab_limit;
@@ -676,17 +697,14 @@ allocate_dynamic_area(natural initsize)
   a = new_area(start, end, AREA_DYNAMIC);
   a->active = start+initsize;
   add_area_holding_area_lock(a);
-  a->markbits = reserved_area->markbits;
-  reserved_area->markbits = NULL;
   CommitMemory(start, end-start);
   a->h = start;
   a->softprot = NULL;
   a->hardprot = NULL;
+  map_initial_reloctab(a->low, a->high);
+  map_initial_markbits(a->low, a->high);
   lisp_global(HEAP_START) = ptr_to_lispobj(a->low);
   lisp_global(HEAP_END) = ptr_to_lispobj(a->high);
-  markbits_limit = (BytePtr)global_mark_ref_bits;
-  reloctab_limit = (BytePtr)global_reloctab;
-  ensure_gc_structures_writable();
   return a;
  }
 
@@ -741,12 +759,22 @@ shrink_dynamic_area(natural delta)
   return true;
 }
 
+#ifndef WINDOWS
+natural user_signal_semaphores[NSIG];
+sigset_t user_signals_reserved;
+#endif
+
 
 #ifndef WINDOWS
 void
 user_signal_handler (int signum, siginfo_t *info, ExceptionInformation *context)
 {
-  if (signum == SIGINT) {
+  SEMAPHORE s = (SEMAPHORE)user_signal_semaphores[signum];
+
+  if (s != 0) {
+    signal_semaphore(s);
+  }
+  else if (signum == SIGINT) {
     lisp_global(INTFLAG) = (((signum<<8) + 1) << fixnumshift);
   }
   else if (signum == SIGTERM) {
@@ -762,6 +790,7 @@ user_signal_handler (int signum, siginfo_t *info, ExceptionInformation *context)
 
 #endif
 
+
 void
 register_user_signal_handler()
 {
@@ -772,28 +801,43 @@ register_user_signal_handler()
 
   SetConsoleCtrlHandler(ControlEventHandler,TRUE);
 #else
-  install_signal_handler(SIGINT, (void *)user_signal_handler);
-  install_signal_handler(SIGTERM, (void *)user_signal_handler);
+  install_signal_handler(SIGINT, (void *)user_signal_handler, false, false);
+  install_signal_handler(SIGTERM, (void *)user_signal_handler, false, false);
+  install_signal_handler(SIGQUIT, (void *)user_signal_handler, false, false);
 #endif
 }
 
-
+int
+wait_for_signal(int signo, int seconds, int milliseconds)
+{
+#ifdef WINDOWS
+  return EINVAL;
+#else
+  if ((signo <= 0) || (signo >= NSIG)) {
+    return EINVAL;
+  }
+  if (sigismember(&user_signals_reserved,signo)) {
+    return EINVAL;
+  }
+  if (user_signal_semaphores[signo] == 0) {
+    user_signal_semaphores[signo] = (natural)new_semaphore(0);
+    install_signal_handler(signo,(void *)user_signal_handler, false, false);
+  }
+  return wait_on_semaphore((void *)user_signal_semaphores[signo],seconds,milliseconds);
+#endif
+}
 
 BytePtr
 initial_stack_bottom()
 {
-#ifndef WINDOWS
-  extern char **environ;
-  char *p = *environ;
-  while (*p) {
-    p += (1+strlen(p));
-  }
-  return (BytePtr)((((natural) p) +4095) & ~4095);
-#endif
-#ifdef WINDOWS
-  return (BytePtr)((current_stack_pointer() + 4095) & ~ 4095);
-#endif
+  extern void os_get_current_thread_stack_bounds(void **, natural*);
+  void *stack_bottom;
+  natural stack_size;
+  
+  os_get_current_thread_stack_bounds(&stack_bottom, &stack_size);
+  return (BytePtr)stack_bottom;
 }
+
 
 
   
@@ -805,13 +849,21 @@ Fatal(StringPtr param0, StringPtr param1)
 {
 
   if (fatal_spare_ptr) {
-    deallocate(fatal_spare_ptr);
+    free(fatal_spare_ptr);
     fatal_spare_ptr = NULL;
   }
   fprintf(dbgout, "Fatal error: %s\n%s\n", param0, param1);
   _exit(-1);
 }
 
+void
+fatal_oserr(StringPtr param, OSErr err)
+{
+  char buf[64];
+  sprintf(buf," - operating system error %d.", err);
+  Fatal(param, buf);
+}
+
 OSErr application_load_err = noErr;
 
 area *
@@ -875,33 +927,6 @@ path_by_appending_image(char *path)
 }
 #endif
 
-char *
-case_inverted_path(char *path)
-{
-  char *copy = strdup(path), *base = copy, *work = copy, c;
-  if (copy == NULL) {
-    return NULL;
-  }
-  while(*work) {
-    if (*work++ == '/') {
-      base = work;
-    }
-  }
-  work = base;
-  while ((c = *work) != '\0') {
-    if (islower(c)) {
-      *work++ = toupper(c);
-    } else {
-      *work++ = tolower(c);
-    }
-  }
-  return copy;
-}
-/* 
-   The underlying file system may be case-insensitive (e.g., HFS),
-   so we can't just case-invert the kernel's name.
-   Tack ".image" onto the end of the kernel's name.  Much better ...
-*/
 #ifdef WINDOWS
 wchar_t *
 default_image_name(wchar_t *orig)
@@ -914,20 +939,8 @@ default_image_name(wchar_t *orig)
 char *
 default_image_name(char *orig)
 {
-#ifdef WINDOWS
-  char *path = chop_exe_suffix(orig);
-#else
   char *path = orig;
-#endif
   char *image_name = path_by_appending_image(path);
-#if !defined(WINDOWS) && !defined(DARWIN)
-  if (!probe_file(image_name)) {
-    char *legacy = case_inverted_path(path);
-    if (probe_file(legacy)) {
-      image_name = legacy;
-    }
-  }
-#endif
   return image_name;
 }
 #endif
@@ -966,8 +979,8 @@ determine_executable_name(char *argv0)
 #ifdef DARWIN
   uint32_t len = 1024;
   char exepath[1024], *p = NULL;
-
-  if (_NSGetExecutablePath(exepath, (void *)&len) == 0) {
+    
+  if (_NSGetExecutablePath(exepath, &len) == 0) {
     p = malloc(len+1);
     memmove(p, exepath, len);
     p[len]=0;
@@ -1327,6 +1340,9 @@ terminate_lisp()
 #ifdef X86
 #define min_os_version "2.6"
 #endif
+#ifdef ARM
+#define min_os_version "2.6"
+#endif
 #endif
 #ifdef FREEBSD
 #define min_os_version "6.0"
@@ -1437,20 +1453,17 @@ check_os_version(char *progname)
   struct utsname uts;
   long got, want;
   char *got_end,*want_end;
-#ifdef X8632
-  extern Boolean rcontext_readonly;
-#endif
 
   want = strtoul(min_os_version,&want_end,10);
 
   uname(&uts);
   got = strtoul(uts.release,&got_end,10);
-#ifdef X8632
-#ifdef FREEBSD
+#if defined(X8632) && defined(FREEBSD)
   if (!strcmp(uts.machine,"amd64")) {
+    extern Boolean rcontext_readonly;
+
     rcontext_readonly = true;
   }
-#endif
 #endif
   while (got == want) {
     if (*want_end == '.') {
@@ -1528,8 +1541,10 @@ lazarus()
     LOCK(lisp_global(TCR_AREA_LOCK),tcr);
     tcr->vs_area->active = tcr->vs_area->high - node_size;
     tcr->save_vsp = (LispObj *)(tcr->vs_area->active);
+#ifndef ARM
     tcr->ts_area->active = tcr->ts_area->high;
     tcr->save_tsp = (LispObj *)(tcr->ts_area->active);
+#endif
     tcr->catch_top = 0;
     tcr->db_link = 0;
     tcr->xframe = 0;
@@ -1639,7 +1654,7 @@ wide_argv_to_utf_8(wchar_t *wide_argv[], int argc)
 
 int
 main(int argc, char *argv[]
-#ifndef WINDOWS
+#if defined(PPC) && defined(LINUX)
 , char *envp[], void *aux
 #endif
 )
@@ -1817,7 +1832,9 @@ main(int argc, char *argv[]
   gc_init();
 
   set_nil(load_image(image_name));
+  lisp_heap_notify_threshold = GC_NOTIFY_THRESHOLD;
   lisp_heap_threshold_from_image = lisp_global(LISP_HEAP_THRESHOLD);
+  
   if (lisp_heap_threshold_from_image) {
     if ((!lisp_heap_threshold_set_from_command_line) &&
         (lisp_heap_threshold_from_image != lisp_heap_gc_threshold)) {
@@ -1836,9 +1853,14 @@ main(int argc, char *argv[]
 
 #ifdef X86
   lisp_global(SUBPRIMS_BASE) = (LispObj)((1<<16)+(5<<10));
-#else
+#endif
+#ifdef PPC
   lisp_global(SUBPRIMS_BASE) = (LispObj)(5<<10);
 #endif
+#ifdef ARM
+  lisp_global(SUBPRIMS_BASE) = (LispObj)(9<<12);
+#endif
+
   lisp_global(RET1VALN) = (LispObj)&ret1valn;
   lisp_global(LEXPR_RETURN) = (LispObj)&nvalret;
   lisp_global(LEXPR_RETURN1V) = (LispObj)&popj;
@@ -1892,13 +1914,16 @@ main(int argc, char *argv[]
     g2_area->younger = g1_area;
     g2_area->older = tenured_area;
     tenured_area->younger = g2_area;
-    tenured_area->refbits = a->markbits;
+    tenured_area->refbits = dynamic_mark_ref_bits;
+    managed_static_area->refbits = global_mark_ref_bits;
+    a->markbits = dynamic_mark_ref_bits;
     tenured_area->static_dnodes = a->static_dnodes;
     a->static_dnodes = 0;
     tenured_area->static_used = a->static_used;
     a->static_used = 0;
     lisp_global(TENURED_AREA) = ptr_to_lispobj(tenured_area);
-    lisp_global(REFBITS) = ptr_to_lispobj(tenured_area->refbits);
+    lisp_global(STATIC_CONS_AREA) = ptr_to_lispobj(static_cons_area);
+    lisp_global(REFBITS) = ptr_to_lispobj(global_mark_ref_bits);
     g2_area->threshold = default_g2_threshold;
     g1_area->threshold = default_g1_threshold;
     a->threshold = default_g0_threshold;
@@ -1909,6 +1934,10 @@ main(int argc, char *argv[]
   init_threads((void *)(stack_base), tcr);
   thread_init_tcr(tcr, current_sp, current_sp-stack_base);
 
+  if (lisp_global(STATIC_CONSES) == 0) {
+    lisp_global(STATIC_CONSES) = lisp_nil;
+  }
+
   lisp_global(EXCEPTION_LOCK) = ptr_to_lispobj(new_recursive_lock());
   enable_fp_exceptions();
   register_user_signal_handler();
@@ -1931,6 +1960,8 @@ main(int argc, char *argv[]
 #endif
   if (egc_enabled) {
     egc_control(true, NULL);
+  } else {
+    lisp_global(OLDSPACE_DNODE_COUNT) = area_dnode(managed_static_area->active,managed_static_area->low);
   }
   atexit(lazarus);
   start_lisp(TCR_TO_TSD(tcr), 0);
@@ -1952,7 +1983,7 @@ set_nil(LispObj r)
 void
 xMakeDataExecutable(void *start, unsigned long nbytes)
 {
-#ifndef X86
+#ifdef PPC
   extern void flush_cache_lines();
   natural ustart = (natural) start, base, end;
   
@@ -1960,6 +1991,10 @@ xMakeDataExecutable(void *start, unsigned long nbytes)
   end = (ustart + nbytes + cache_block_size - 1) & ~(cache_block_size-1);
   flush_cache_lines(base, (end-base)/cache_block_size, cache_block_size);
 #endif
+#ifdef ARM
+  extern void flush_cache_lines(void *, size_t);
+  flush_cache_lines(start,nbytes);
+#endif
 }
 
 natural
@@ -1988,95 +2023,8 @@ xGetSharedLibrary(char *path, int mode)
 void *
 xGetSharedLibrary(char *path, int *resultType)
 {
-#if 0
-  NSObjectFileImageReturnCode code;
-  NSObjectFileImage	         moduleImage;
-  NSModule		         module;
-  const struct mach_header *     header;
-  const char *                   error;
-  void *                         result;
-  /* not thread safe */
-  /*
-  static struct {
-    const struct mach_header  *header;
-    NSModule	              *module;
-    const char                *error;
-  } results;	
-  */
-  result = NULL;
-  error = NULL;
-
-  /* first try to open this as a bundle */
-  code = NSCreateObjectFileImageFromFile(path,&moduleImage);
-  if (code != NSObjectFileImageSuccess &&
-      code != NSObjectFileImageInappropriateFile &&
-      code != NSObjectFileImageAccess)
-    {
-      /* compute error strings */
-      switch (code)
-	{
-	case NSObjectFileImageFailure:
-	  error = "NSObjectFileImageFailure";
-	  break;
-	case NSObjectFileImageArch:
-	  error = "NSObjectFileImageArch";
-	  break;
-	case NSObjectFileImageFormat:
-	  error = "NSObjectFileImageFormat";
-	  break;
-	case NSObjectFileImageAccess:
-	  /* can't find the file */
-	  error = "NSObjectFileImageAccess";
-	  break;
-	default:
-	  error = "unknown error";
-	}
-      *resultType = 0;
-      return (void *)error;
-    }
-  if (code == NSObjectFileImageInappropriateFile ||
-      code == NSObjectFileImageAccess ) {
-    /* the pathname might be a partial pathane (hence the access error)
-       or it might be something other than a bundle, if so perhaps
-       it is a .dylib so now try to open it as a .dylib */
-
-    /* protect against redundant loads, Gary Byers noticed possible
-       heap corruption if this isn't done */
-    header = NSAddImage(path, NSADDIMAGE_OPTION_RETURN_ON_ERROR |
-			NSADDIMAGE_OPTION_WITH_SEARCHING |
-			NSADDIMAGE_OPTION_RETURN_ONLY_IF_LOADED);
-    if (!header)
-      header = NSAddImage(path, NSADDIMAGE_OPTION_RETURN_ON_ERROR |
-			  NSADDIMAGE_OPTION_WITH_SEARCHING);
-    result = (void *)header;
-    *resultType = 1;
-  }
-  else if (code == NSObjectFileImageSuccess) {
-    /* we have a sucessful module image
-       try to link it, don't bind symbols privately */
-
-    module = NSLinkModule(moduleImage, path,
-			  NSLINKMODULE_OPTION_RETURN_ON_ERROR | NSLINKMODULE_OPTION_BINDNOW);
-    NSDestroyObjectFileImage(moduleImage);	
-    result = (void *)module;
-    *resultType = 2;
-  }
-  if (!result)
-    {
-      /* compute error string */
-      NSLinkEditErrors ler;
-      int lerno;
-      const char* file;
-      NSLinkEditError(&ler,&lerno,&file,&error);
-      if (error) {
-	result = (void *)error;
-	*resultType = 0;
-      }
-    }
-  return result;
-#else
-  const char *                   error;
-  void *                         result;
+  const char *error;
+  void *result;
 
   result = dlopen(path, RTLD_NOW | RTLD_GLOBAL);
   
@@ -2087,7 +2035,6 @@ xGetSharedLibrary(char *path, int *resultType)
   }
   *resultType = 1;
   return result;
-#endif
 }
 #endif
 
@@ -2196,10 +2143,17 @@ load_image(
     err = errno;
   }
   if (image_nil == 0) {
+#ifdef WINDOWS
+    char *fmt = "Couldn't load lisp heap image from %ls";
+#else
+    char *fmt = "Couldn't load lisp heap image from %s";
+#endif
+
+    fprintf(dbgout, fmt, path);
     if (err == 0) {
-      fprintf(dbgout, "Couldn't load lisp heap image from %s\n", path);
+      fprintf(dbgout, "\n");
     } else {
-      fprintf(dbgout, "Couldn't load lisp heap image from %s:\n%s\n", path, strerror(err));
+      fprintf(dbgout, ": %s\n", strerror(err));
     }
     exit(-1);
   }
@@ -2213,29 +2167,6 @@ set_errno(int val)
   return -1;
 }
 
-/* A horrible hack to allow us to initialize a JVM instance from lisp.
-   On Darwin, creating a JVM instance clobbers the thread's existing
-   Mach exception infrastructure, so we save and restore it here.
-*/
-
-typedef int (*jvm_initfunc)(void*,void*,void*);
-
-int
-jvm_init(jvm_initfunc f,void*arg0,void*arg1,void*arg2)
-{
-  int result = -1;
-  TCR *tcr = get_tcr(1);
-#ifdef DARWIN
-  extern kern_return_t tcr_establish_lisp_exception_port(TCR *);
-#endif
-  
-  result = f(arg0,arg1,arg2);
-#ifdef DARWIN
-  tcr_establish_lisp_exception_port(tcr);
-#endif
-  return result;
-}
-  
 
 
 
@@ -2246,7 +2177,6 @@ xFindSymbol(void* handle, char *name)
   return dlsym(handle, name);
 #endif
 #ifdef DARWIN
-#if 1
   void *result;
 
   if ((handle == NULL) || (handle == ((void *) -1))) {
@@ -2257,19 +2187,6 @@ xFindSymbol(void* handle, char *name)
     result = dlsym(handle, name+1);
   }
   return result;
-#else
-  natural address = 0;
-
-  if ((handle == NULL) ||
-      (handle == (void *)-1) ||
-      (handle == (void *)-2)){
-    if (NSIsSymbolNameDefined(name)) { /* Keep dyld_lookup from crashing */
-      _dyld_lookup_and_bind(name, (void *) &address, (void*) NULL);
-    }
-    return (void *)address;
-  }
-  Bug(NULL, "How did this happen ?");
-#endif
 #endif
 #ifdef WINDOWS
   extern void *windows_find_symbol(void *, char *);
@@ -2350,3 +2267,95 @@ report_paging_info_delta(FILE *out, paging_info *start, paging_info *stop)
 
 #endif
 #endif
+
+void
+allocate_static_conses(natural n)
+{
+  BytePtr old_low = static_cons_area->low,
+    new_low = old_low - (n<<dnode_shift);
+  cons *c;
+  natural i;
+  LispObj prev;
+
+  CommitMemory(new_low,old_low-new_low);
+
+  static_cons_area->low = new_low;
+  lower_heap_start(new_low, tenured_area);
+  /* what a mess this is ... */
+  if (active_dynamic_area->low == old_low) {
+    active_dynamic_area->low = new_low;
+  }
+  if (!active_dynamic_area->older) {
+    active_dynamic_area->markbits = tenured_area->refbits;
+  }
+  if (g1_area->low == old_low) {
+    g1_area->low = new_low;
+  }
+  if (g1_area->high == old_low) {
+    g1_area->high = new_low;
+  }
+  if (g2_area->low == old_low) {
+    g2_area->low = new_low;
+  }
+  if (g2_area->high == old_low) {
+    g2_area->high = new_low;
+  }
+  for (i=0, prev=lisp_global(STATIC_CONSES), c=(cons *)new_low;
+       i < n;
+       i++, c++) {
+    c->car = unbound;
+    c->cdr = prev;
+    prev = ((LispObj)c)+fulltag_cons;
+  }
+  lisp_global(STATIC_CONSES)=prev;
+  lisp_global(FREE_STATIC_CONSES)+=(n<<fixnumshift);
+}
+
+#ifdef X86
+#define USE_GC_NOTIFICATION 1
+#else
+#undef USE_GC_NOTIFICATION
+#endif
+
+void
+ensure_static_conses(ExceptionInformation *xp, TCR *tcr, natural nconses)
+{
+  area *a = active_dynamic_area;
+  natural nbytes = nconses>>dnode_shift, have;
+  BytePtr p = a->high-nbytes;
+#ifdef USE_GC_NOTIFICATION
+  Boolean crossed_notify_threshold = false;
+  LispObj before_shrink, after_shrink;
+#endif
+
+  if (p < a->active) {
+    untenure_from_area(tenured_area);
+    gc_from_xp(xp, 0L);
+#ifdef USE_GC_NOTIFICATION
+    did_gc_notification_since_last_full_gc = false;
+#endif
+  }
+
+  have = unbox_fixnum(lisp_global(FREE_STATIC_CONSES));
+  if (have < nconses) {
+#ifdef USE_GC_NOTIFICATION
+    before_shrink = a->high-a->active;
+    if (before_shrink>nbytes) {
+      shrink_dynamic_area(nbytes);
+      after_shrink = a->high-a->active; 
+      if ((before_shrink >= lisp_heap_notify_threshold) &&
+          (after_shrink < lisp_heap_notify_threshold)) {
+        crossed_notify_threshold = true;
+      }
+    }
+#endif
+    allocate_static_conses(nconses);
+    tcr->bytes_allocated += nbytes;
+  }
+#ifdef USE_GC_NOTIFICATION
+  if (crossed_notify_threshold && !did_gc_notification_since_last_full_gc) {
+    callback_for_gc_notification(xp,tcr);
+  }
+#endif
+}
+      
