@@ -45,7 +45,7 @@ extern void pseudo_sigreturn(ExceptionInformation *);
 #endif
 
 
-#include "Threads.h"
+#include "threads.h"
 
 #define MSR_FE0_MASK (((unsigned)0x80000000)>>20)
 #define MSR_FE1_MASK (((unsigned)0x80000000)>>23)
@@ -469,6 +469,10 @@ handle_gc_trap(ExceptionInformation *xp, TCR *tcr)
     xpGPR(xp, imm0) = lisp_heap_gc_threshold;
     break;
 
+  case GC_TRAP_FUNCTION_ENSURE_STATIC_CONSES:
+    ensure_static_conses(xp,tcr,32768);
+    break;
+
   case GC_TRAP_FUNCTION_FLASH_FREEZE:
     untenure_from_area(tenured_area);
     gc_like_from_xp(xp,flash_freeze,0);
@@ -506,11 +510,12 @@ handle_gc_trap(ExceptionInformation *xp, TCR *tcr)
       if (selector & GC_TRAP_FUNCTION_IMPURIFY) {
         impurify_from_xp(xp, 0L);
         /*        nrs_GC_EVENT_STATUS_BITS.vcell |= gc_integrity_check_bit; */
+        lisp_global(OLDSPACE_DNODE_COUNT) = 0;
         gc_from_xp(xp, 0L);
-        release_readonly_area();
       }
       if (selector & GC_TRAP_FUNCTION_PURIFY) {
         purify_from_xp(xp, 0L);
+        lisp_global(OLDSPACE_DNODE_COUNT) = area_dnode(managed_static_area->active, managed_static_area->low);
         gc_from_xp(xp, 0L);
       }
       if (selector & GC_TRAP_FUNCTION_SAVE_APPLICATION) {
@@ -527,9 +532,7 @@ handle_gc_trap(ExceptionInformation *xp, TCR *tcr)
         fatal_oserr(": save_application", err);
       }
       switch (selector) {
-      case GC_TRAP_FUNCTION_SET_HONS_AREA_SIZE:
-        xpGPR(xp, imm0) = 0;
-        break;
+
 
       case GC_TRAP_FUNCTION_FREEZE:
         a->active = (BytePtr) align_to_power_of_2(a->active, log2_page_size);
@@ -1537,6 +1540,9 @@ callback_to_lisp (LispObj callback_macptr, ExceptionInformation *xp,
   tcr->save_vsp = (LispObj*) ptr_from_lispobj(xpGPR(xp, vsp));
   tcr->save_tsp = (LispObj*) ptr_from_lispobj(xpGPR(xp, tsp));
 
+#ifdef DARWIN
+  enable_fp_exceptions();
+#endif
 
 
   /* Call back.
@@ -1984,12 +1990,12 @@ pc_luser_xp(ExceptionInformation *xp, TCR *tcr, signed_natural *alloc_disp)
       *ea = val;
     }
     if (need_check_memo) {
-      natural  bitnumber = area_dnode(ea, lisp_global(HEAP_START));
+      natural  bitnumber = area_dnode(ea, lisp_global(REF_BASE));
       if ((bitnumber < lisp_global(OLDSPACE_DNODE_COUNT)) &&
           ((LispObj)ea < val)) {
         atomic_set_bit(refbits, bitnumber);
         if (need_memoize_root) {
-          bitnumber = area_dnode(root, lisp_global(HEAP_START));
+          bitnumber = area_dnode(root, lisp_global(REF_BASE));
           atomic_set_bit(refbits, bitnumber);
         }
       }
@@ -2205,7 +2211,7 @@ interrupt_handler (int signum, siginfo_t *info, ExceptionInformation *context)
 
 
 void
-install_signal_handler(int signo, void *handler)
+install_signal_handler(int signo, void *handler, Boolean system, Boolean on_altstack)
 {
   struct sigaction sa;
   
@@ -2222,6 +2228,11 @@ install_signal_handler(int signo, void *handler)
     ;
 
   sigaction(signo, &sa, NULL);
+  if (system) {
+    extern sigset_t user_signals_reserved;
+    sigaddset(&user_signals_reserved, signo);
+  }
+
 }
 
 void
@@ -2240,17 +2251,17 @@ install_pmcl_exception_handlers()
     ;
   if (install_signal_handlers_for_exceptions) {
     extern int no_sigtrap;
-    install_signal_handler(SIGILL, (void *)signal_handler);
+    install_signal_handler(SIGILL, (void *)signal_handler, true, false);
     if (no_sigtrap != 1) {
-      install_signal_handler(SIGTRAP, (void *)signal_handler);
+      install_signal_handler(SIGTRAP, (void *)signal_handler, true, false);
     }
-    install_signal_handler(SIGBUS,  (void *)signal_handler);
-    install_signal_handler(SIGSEGV, (void *)signal_handler);
-    install_signal_handler(SIGFPE, (void *)signal_handler);
+    install_signal_handler(SIGBUS,  (void *)signal_handler, true, false);
+    install_signal_handler(SIGSEGV, (void *)signal_handler, true, false);
+    install_signal_handler(SIGFPE, (void *)signal_handler, true, false);
   }
   
   install_signal_handler(SIGNAL_FOR_PROCESS_INTERRUPT,
-			 (void *)interrupt_handler);
+			 (void *)interrupt_handler, true, false);
   signal(SIGPIPE, SIG_IGN);
 }
 
@@ -2289,8 +2300,8 @@ thread_signal_setup()
   thread_suspend_signal = SIG_SUSPEND_THREAD;
   thread_kill_signal = SIG_KILL_THREAD;
 
-  install_signal_handler(thread_suspend_signal, (void *) suspend_resume_handler);
-  install_signal_handler(thread_kill_signal, (void *)thread_kill_handler);
+  install_signal_handler(thread_suspend_signal, (void *) suspend_resume_handler, true, false);
+  install_signal_handler(thread_kill_signal, (void *)thread_kill_handler, true, false);
 }
 
 
@@ -2472,7 +2483,7 @@ restore_mach_thread_state(mach_port_t thread, ExceptionInformation *pseudosigcon
   /* Set the thread's FP state from the pseudosigcontext */
   kret = thread_set_state(thread,
                           PPC_FLOAT_STATE,
-                          (thread_state_t)&(mc->fs),
+                          (thread_state_t)&(mc->__fs),
                           PPC_FLOAT_STATE_COUNT);
 
   MACH_CHECK_ERROR("setting thread FP state", kret);
@@ -2481,12 +2492,12 @@ restore_mach_thread_state(mach_port_t thread, ExceptionInformation *pseudosigcon
 #ifdef PPC64
   kret = thread_set_state(thread,
                           PPC_THREAD_STATE64,
-                          (thread_state_t)&(mc->ss),
+                          (thread_state_t)&(mc->__ss),
                           PPC_THREAD_STATE64_COUNT);
 #else
   kret = thread_set_state(thread, 
                           MACHINE_THREAD_STATE,
-                          (thread_state_t)&(mc->ss),
+                          (thread_state_t)&(mc->__ss),
                           MACHINE_THREAD_STATE_COUNT);
 #endif
   MACH_CHECK_ERROR("setting thread state", kret);
@@ -2558,7 +2569,7 @@ create_thread_context_frame(mach_port_t thread,
     get_tcr(true);
     Bug(NULL, "Exception thread can't obtain thread state, Mach result = %d", result);
   }
-  stackp = ts.r1;
+  stackp = ts.__r1;
   backlink = stackp;
   stackp = TRUNC_DOWN(stackp, C_REDZONE_LEN, C_STK_ALIGN);
   stackp -= sizeof(*pseudosigcontext);
@@ -2566,12 +2577,12 @@ create_thread_context_frame(mach_port_t thread,
 
   stackp = TRUNC_DOWN(stackp, sizeof(*mc), C_STK_ALIGN);
   mc = (MCONTEXT_T) ptr_from_lispobj(stackp);
-  memmove(&(mc->ss),&ts,sizeof(ts));
+  memmove(&(mc->__ss),&ts,sizeof(ts));
 
   thread_state_count = PPC_FLOAT_STATE_COUNT;
   thread_get_state(thread,
 		   PPC_FLOAT_STATE,
-		   (thread_state_t)&(mc->fs),
+		   (thread_state_t)&(mc->__fs),
 		   &thread_state_count);
 
 
@@ -2586,7 +2597,7 @@ create_thread_context_frame(mach_port_t thread,
 #else
 		   PPC_EXCEPTION_STATE,
 #endif
-		   (thread_state_t)&(mc->es),
+		   (thread_state_t)&(mc->__es),
 		   &thread_state_count);
 
 
@@ -2649,18 +2660,18 @@ setup_signal_frame(mach_port_t thread,
      args) when the thread's resumed.
   */
 
-  ts.srr0 = (natural) handler_address;
-  ts.srr1 = (int) xpMSR(pseudosigcontext) & ~MSR_FE0_FE1_MASK;
-  ts.r1 = stackp;
-  ts.r3 = signum;
-  ts.r4 = (natural)pseudosigcontext;
-  ts.r5 = (natural)tcr;
-  ts.r6 = (natural)old_valence;
-  ts.lr = (natural)pseudo_sigreturn;
+  ts.__srr0 = (natural) handler_address;
+  ts.__srr1 = (int) xpMSR(pseudosigcontext) & ~MSR_FE0_FE1_MASK;
+  ts.__r1 = stackp;
+  ts.__r3 = signum;
+  ts.__r4 = (natural)pseudosigcontext;
+  ts.__r5 = (natural)tcr;
+  ts.__r6 = (natural)old_valence;
+  ts.__lr = (natural)pseudo_sigreturn;
 
 
 #ifdef PPC64
-  ts.r13 = xpGPR(pseudosigcontext,13);
+  ts.__r13 = xpGPR(pseudosigcontext,13);
   thread_set_state(thread,
                    PPC_THREAD_STATE64,
                    (thread_state_t)&ts,
@@ -2709,36 +2720,12 @@ thread_set_fp_exceptions_enabled(mach_port_t thread, Boolean enabled)
 		   (thread_state_t)&ts,
 		   &thread_state_count);
   if (enabled) {
-    ts.srr1 |= MSR_FE0_FE1_MASK;
+    ts.__srr1 |= MSR_FE0_FE1_MASK;
   } else {
-    ts.srr1 &= ~MSR_FE0_FE1_MASK;
-  }
-  /* 
-     Hack-o-rama warning (isn't it about time for such a warning?):
-     pthread_kill() seems to want to lose the MSR's FE0/FE1 bits.
-     Our handler for lisp's use of pthread_kill() pushes a phony
-     lisp frame on the stack and force the context to resume at
-     the UUO in enable_fp_exceptions(); the "saveLR" field of that
-     lisp frame contains the -real- address that process_interrupt
-     should have returned to, and the fact that it's in a lisp
-     frame should convince the GC to notice that address if it
-     runs in the tiny time window between returning from our
-     interrupt handler and ... here.
-     If the top frame on the stack is a lisp frame, discard it
-     and set ts.srr0 to the saveLR field in that frame.  Otherwise,
-     just adjust ts.srr0 to skip over the UUO.
-  */
-  {
-    lisp_frame *tos = (lisp_frame *)ts.r1,
-      *next_frame = tos->backlink;
-    
-    if (tos == (next_frame -1)) {
-      ts.srr0 = tos->savelr;
-      ts.r1 = (LispObj) next_frame;
-    } else {
-      ts.srr0 += 4;
-    }
+    ts.__srr1 &= ~MSR_FE0_FE1_MASK;
   }
+  
+  ts.__srr0 += 4;
   thread_set_state(thread, 
 #ifdef PPC64
 		   PPC_THREAD_STATE64,	/* GPRs, some SPRs  */
@@ -3073,8 +3060,6 @@ darwin_exception_init(TCR *tcr)
     fprintf(dbgout, "Couldn't setup exception handler - error = %d\n", kret);
     terminate_lisp();
   }
-  lisp_global(LISP_EXIT_HOOK) = (LispObj) restore_foreign_exception_ports;
-  lisp_global(LISP_RETURN_HOOK) = (LispObj) tcr_establish_lisp_exception_port;
 }
 
 /*
