@@ -16,7 +16,7 @@
 */
 
 
-#include "Threads.h"
+#include "threads.h"
 
 
 typedef struct {
@@ -99,6 +99,8 @@ raise_thread_interrupt(TCR *target)
       (target->unwinding != 0) ||
       (!((where < (pc)lisp_global(HEAP_END)) &&
          (where >= (pc)lisp_global(HEAP_START))) &&
+       (!((where < (pc)(managed_static_area->active)) &&
+	 (where >= (pc)(readonly_area->low)))) &&
        !((where < spentry_end) && (where >= spentry_start)) &&
        !((where < subprims_end) && (where >= subprims_start)) &&
        !((where < (pc) 0x16000) &&
@@ -1332,7 +1334,6 @@ new_tcr(natural vstack_size, natural tstack_size)
 #if (WORD_SIZE == 64)
   tcr->single_float_convert.tag = subtag_single_float;
 #endif
-  lisp_global(TCR_COUNT) += (1<<fixnumshift);
   tcr->suspend = new_semaphore(0);
   tcr->resume = new_semaphore(0);
   tcr->reset_completion = new_semaphore(0);
@@ -1342,11 +1343,15 @@ new_tcr(natural vstack_size, natural tstack_size)
   tcr->vs_area = a;
   a->owner = tcr;
   tcr->save_vsp = (LispObj *) a->active;  
+#ifndef ARM
   a = allocate_tstack_holding_area_lock(tstack_size);
+#endif
   UNLOCK(lisp_global(TCR_AREA_LOCK),tcr);
+#ifndef ARM
   tcr->ts_area = a;
   a->owner = tcr;
   tcr->save_tsp = (LispObj *) a->active;
+#endif
 #ifdef X86
   tcr->next_tsp = tcr->save_tsp;
 #endif
@@ -1363,6 +1368,12 @@ new_tcr(natural vstack_size, natural tstack_size)
     (1 << MXCSR_UM_BIT) | 
 #endif
     (1 << MXCSR_PM_BIT);
+#endif
+#ifdef ARM
+  tcr->lisp_fpscr = 
+    (1 << FPSCR_IOE_BIT) | 
+    (1 << FPSCR_DZE_BIT) |
+    (1 << FPSCR_OFE_BIT);
 #endif
   tcr->save_allocbase = tcr->save_allocptr = (void *) VOID_ALLOCPTR;
   tcr->tlb_limit = 2048<<fixnumshift;
@@ -1405,19 +1416,39 @@ shutdown_thread_tcr(void *arg)
     LOCK(lisp_global(TCR_AREA_LOCK),current);
     vs = tcr->vs_area;
     tcr->vs_area = NULL;
+#ifndef ARM
     ts = tcr->ts_area;
     tcr->ts_area = NULL;
+#endif
     cs = tcr->cs_area;
     tcr->cs_area = NULL;
     if (vs) {
       condemn_area_holding_area_lock(vs);
     }
+#ifndef ARM
     if (ts) {
       condemn_area_holding_area_lock(ts);
     }
+#endif
     if (cs) {
       condemn_area_holding_area_lock(cs);
     }
+    /* On some platforms - currently just linuxarm - we have to
+       allocate a separate alternate signal stack (rather than just
+       using a few pages of the thread's main stack.)  Disable and
+       free that alternate stack here.
+    */
+#ifdef ARM
+#if defined(LINUX)
+    {
+      stack_t new, current;
+      new.ss_flags = SS_DISABLE;
+      if (sigaltstack(&new, &current) == 0) {
+        munmap(current.ss_sp, current.ss_size);
+      }
+    }
+#endif
+#endif
     destroy_semaphore(&tcr->suspend);
     destroy_semaphore(&tcr->resume);
     destroy_semaphore(&tcr->reset_completion);
@@ -1425,6 +1456,11 @@ shutdown_thread_tcr(void *arg)
     tcr->tlb_limit = 0;
     free(tcr->tlb_pointer);
     tcr->tlb_pointer = NULL;
+#ifdef WINDOWS
+    if (tcr->osid != 0) {
+      CloseHandle((HANDLE)(tcr->osid));
+    }
+#endif
     tcr->osid = 0;
     tcr->interrupt_pending = 0;
     tcr->termination_semaphore = NULL;
@@ -1456,10 +1492,12 @@ tcr_cleanup(void *arg)
   if (a) {
     a->active = a->high;
   }
+#ifndef ARM
   a = tcr->ts_area;
   if (a) {
     a->active = a->high;
   }
+#endif
   a = tcr->cs_area;
   if (a) {
     a->active = a->high;
@@ -1509,6 +1547,9 @@ thread_init_tcr(TCR *tcr, void *stack_base, natural stack_size)
   UNLOCK(lisp_global(TCR_AREA_LOCK),tcr);
   tcr->cs_area = a;
   a->owner = tcr;
+#ifdef ARM
+  tcr->last_lisp_frame = (natural)(a->high);
+#endif
   if (!(tcr->flags & (1<<TCR_FLAG_BIT_FOREIGN))) {
     tcr->cs_limit = (LispObj)ptr_to_lispobj(a->softlimit);
   }
@@ -1556,9 +1597,6 @@ register_thread_tcr(TCR *tcr)
 
   
   
-#ifndef MAP_GROWSDOWN
-#define MAP_GROWSDOWN 0
-#endif
 
 Ptr
 create_stack(natural size)
@@ -1605,25 +1643,11 @@ count_cpus()
 void
 count_cpus()
 {
-#ifdef DARWIN
-  /* As of OSX 10.4, Darwin doesn't define _SC_NPROCESSORS_ONLN */
-#include <mach/host_info.h>
-
-  struct host_basic_info info;
-  mach_msg_type_number_t count = HOST_BASIC_INFO_COUNT;
-  
-  if (KERN_SUCCESS == host_info(mach_host_self(), HOST_BASIC_INFO,(host_info_t)(&info),&count)) {
-    if (info.max_cpus > 1) {
-      spin_lock_tries = 1024;
-    }
-  }
-#else
-  int n = sysconf(_SC_NPROCESSORS_ONLN);
+  int n = sysconf(_SC_NPROCESSORS_CONF);
   
   if (n > 1) {
     spin_lock_tries = 1024;
   }
-#endif
 }
 #endif
 #endif
@@ -1783,25 +1807,12 @@ active_tcr_p(TCR *q)
   return false;
 }
 
-#ifdef WINDOWS
+
 OSErr
 xDisposeThread(TCR *tcr)
 {
   return 0;                     /* I don't think that this is ever called. */
 }
-#else
-OSErr
-xDisposeThread(TCR *tcr)
-{
-  if (tcr != (TCR *)ptr_from_lispobj(lisp_global(INITIAL_TCR))) {
-    if (active_tcr_p(tcr) && (tcr != get_tcr(false))) {
-      pthread_cancel((pthread_t)(tcr->osid));
-      return 0;
-    }
-  }
-  return -50;
-}
-#endif
 
 OSErr
 xYieldToThread(TCR *target)
@@ -1819,18 +1830,19 @@ xThreadCurrentStackSpace(TCR *tcr, unsigned *resultP)
 
 
 #ifdef WINDOWS
-LispObj
+Boolean
 create_system_thread(size_t stack_size,
 		     void* stackaddr,
 		     unsigned CALLBACK (*start_routine)(void *),
 		     void* param)
 {
   HANDLE thread_handle;
+  Boolean won = false;
 
   stack_size = ((stack_size+(((1<<16)-1)))&~((1<<16)-1));
 
   thread_handle = (HANDLE)_beginthreadex(NULL, 
-                                         0/*stack_size*/,
+                                         stack_size,
                                          start_routine,
                                          param,
                                          0, 
@@ -1838,18 +1850,20 @@ create_system_thread(size_t stack_size,
 
   if (thread_handle == NULL) {
     wperror("CreateThread");
+  } else {
+    won = true;
+    CloseHandle(thread_handle);
   }
-  return (LispObj) ptr_to_lispobj(thread_handle);
+  return won;
 }
 #else
-LispObj
-create_system_thread(size_t stack_size,
-		     void* stackaddr,
-		     void* (*start_routine)(void *),
-		     void* param)
+Boolean
+create_system_thread(size_t stack_size,  void *stackaddr,
+		     void *(*start_routine)(void *), void *param)
 {
   pthread_attr_t attr;
-  pthread_t returned_thread = (pthread_t) 0;
+  pthread_t returned_thread;
+  int err;
   TCR *current = get_tcr(true);
 
   pthread_attr_init(&attr);
@@ -1862,11 +1876,7 @@ create_system_thread(size_t stack_size,
   stack_size = ensure_stack_limit(stack_size);
   if (stackaddr != NULL) {
     /* Size must have been specified.  Sort of makes sense ... */
-#ifdef DARWIN
-    Fatal("no pthread_attr_setsetstack. "," Which end of stack does address refer to?");
-#else
     pthread_attr_setstack(&attr, stackaddr, stack_size);
-#endif
   } else if (stack_size != DEFAULT_THREAD_STACK_SIZE) {
     pthread_attr_setstacksize(&attr,stack_size);
   }
@@ -1883,10 +1893,10 @@ create_system_thread(size_t stack_size,
      getting suspended (by grabbing TCR_AREA_LOCK)
   */
   LOCK(lisp_global(TCR_AREA_LOCK),current);
-  pthread_create(&returned_thread, &attr, start_routine, param);
+  err = pthread_create(&returned_thread, &attr, start_routine, param);
   UNLOCK(lisp_global(TCR_AREA_LOCK),current);
   pthread_attr_destroy(&attr);
-  return (LispObj) ptr_to_lispobj(returned_thread);
+  return (err == 0);
 }
 #endif
 
@@ -1925,6 +1935,9 @@ get_tcr(Boolean create)
 #endif
 #ifdef X8632
 #define NSAVEREGS 0
+#endif
+#ifdef ARM
+#define NSAVEREGS 0
 #endif
     for (i = 0; i < NSAVEREGS; i++) {
       *(--current->save_vsp) = 0;
@@ -1957,7 +1970,11 @@ pc_luser_restore_windows_context(CONTEXT *pcontext, TCR *tcr, pc where)
 #ifdef WIN_64
     *pcontext = * (CONTEXT *)(pcontext->Rcx);
 #else
-    *pcontext = * (CONTEXT *)(pcontext->Ecx);
+    if (where == restore_windows_context_start) {
+      *pcontext = * (CONTEXT *)((pcontext->Esp)+4);
+    } else {
+      *pcontext = * (CONTEXT *)(pcontext->Ecx);
+    }
 #endif
   } else {
     /* Most of the context has already been restored; fix %rcx
@@ -1980,6 +1997,11 @@ pc_luser_restore_windows_context(CONTEXT *pcontext, TCR *tcr, pc where)
 #endif
   }
   tcr->pending_exception_context = NULL;
+  /* We basically never return from an exception unless we
+     were executing lisp code when the exception returned.
+     If that ever changes, we need to know what valence
+     would have been restored here.*/
+  tcr->valence = TCR_STATE_LISP;
 }
 
 Boolean
@@ -2026,6 +2048,8 @@ suspend_tcr(TCR *tcr)
            point. */
         if (!((where < (pc)lisp_global(HEAP_END)) &&
               (where >= (pc)lisp_global(HEAP_START))) &&
+	    (!((where < (pc)(managed_static_area->active)) &&
+	      (where >= (pc)(readonly_area->low)))) &&
             !((where < spentry_end) && (where >= spentry_start)) &&
             !((where < subprims_end) && (where >= subprims_start)) &&
             !((where < (pc) 0x16000) &&
@@ -2126,13 +2150,15 @@ kill_tcr(TCR *tcr)
     if (osid) {
       result = true;
 #ifdef WINDOWS
-      /* What we really want to de hear is (something like)
+      /* What we really want to do here is (something like)
          forcing the thread to run quit_handler().  For now,
-         mark the TCR as dead and kill thw Windows thread. */
+         mark the TCR as dead and kill the Windows thread. */
       tcr->osid = 0;
       if (!TerminateThread((HANDLE)osid, 0)) {
+        CloseHandle((HANDLE)osid);
         result = false;
       } else {
+        CloseHandle((HANDLE)osid);
         shutdown_thread_tcr(tcr);
       }
 #else
@@ -2244,10 +2270,12 @@ normalize_dead_tcr_areas(TCR *tcr)
     a->active = a->high;
   }
 
+#ifndef ARM
   a = tcr->ts_area;
   if (a) {
     a->active = a->high;
   }
+#endif
 
   a = tcr->cs_area;
   if (a) {
